@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    23-Sep-91 at 20:34:36
-;; Last-Mod:     17-Apr-22 at 12:37:06 by Bob Weiner
+;; Last-Mod:     10-Mar-24 at 11:28:18 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2024  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -16,9 +18,7 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(hbut hpath hargs hmail)))
-(eval-when-compile (mapc #'require '(comint hsettings)))
-(require 'bookmark)
+(eval-and-compile (mapc #'require '(bookmark hvar hsettings comint hbut hpath hargs hmail man hsys-org)))
 
 ;;; ************************************************************************
 ;;; Public declarations
@@ -26,7 +26,11 @@
 
 (declare-function kotl-mode:goto-cell "kotl-mode")
 (declare-function kotl-mode:beginning-of-buffer "kotl-mode")
+(declare-function kotl-mode:goto-cell-ref "kotl-mode")
+(declare-function kcell-view:indent "kcell-view")
+
 (declare-function rmail:msg-to-p "hrmail")
+(declare-function org-roam-id-find "‎ext:org-roam-id")
 
 ;;; ************************************************************************
 ;;; Standard Hyperbole action types
@@ -35,11 +39,17 @@
 (defact annot-bib (key)
   "Follow internal ref KEY within an annotated bibliography, delimiters=[]."
   (interactive "sReference key (no []): ")
-  (let ((key-regexp (concat "^[*]*[ \t]*\\\[" (ebut:key-to-label key) "\\\]"))
-	citation)
-    (if (save-excursion
-	  (goto-char (point-max))
-	  (setq citation (re-search-backward key-regexp nil t)))
+  (let* ((key-regexp (concat "^[*]*[ \t]*\\[" (ebut:key-to-label key) "\\]"))
+	 (lbl-start (hattr:get 'hbut:current 'lbl-start))
+	 (lbl-end (hattr:get 'hbut:current 'lbl-end))
+	 (citation (when (and lbl-start lbl-end)
+		     (save-excursion
+		       (goto-char (point-max))
+	               (and (re-search-backward key-regexp nil t)
+			    (or (< (point) (1- lbl-start))
+				(> (point) (1+ lbl-end)))
+			    (point))))))
+    (if citation
 	(progn (hpath:display-buffer (current-buffer))
 	       (goto-char citation)
 	       (beginning-of-line))
@@ -50,7 +60,7 @@
 Unless point is at the end of the buffer or if a completion has already been
 inserted, delete the completions window."
   (interactive)
-  (if (eobp)
+  (if (smart-eobp)
       (progn (bury-buffer nil)
 	     (delete-window))
     (hargs:completion)))
@@ -58,19 +68,29 @@ inserted, delete the completions window."
 (defact display-boolean (bool-expr)
   "Display a message showing the result value of a BOOL-EXPR.
 Return any non-nil value or t."
+  (interactive "xDisplay bool expr value: ")
   (let ((result (eval bool-expr t)))
-    (message "Boolean result (%s) = %S; Expr: %S"
-	     (if result "True" "False") result bool-expr)
+    (message "Result = %S; Boolean value = %s; Expr = %S"
+	     result (if result "True" "False") bool-expr)
+    (or result t)))
+
+(defact display-value (value)
+  "Display a message showing VALUE (a symbol) and its value.
+Return any non-nil value or t."
+  (interactive "SDisplay symbol's value: ")
+  (let ((result (eval value)))
+    (message "%S" result)
     (or result t)))
 
 (defact display-variable (var)
-  "Display a message showing `var` (a symbol) and its value.
+  "Display a message showing VAR (a symbol) and its value.
 Return any non-nil value or t."
-  (message "%s = %s" var (symbol-value var))
+  (interactive "vDisplay variable's value: ")
+  (message "%s = %S" var (symbol-value var))
   (or (symbol-value var) t))
 
 (defact eval-elisp (lisp-expr)
-  "Evaluate a Lisp expression LISP-EXPR for its side-effects and return any non-nil value."
+  "Evaluate a LISP-EXPR for its side-effects and return any non-nil value."
   (interactive "xLisp to eval: ")
   (eval lisp-expr t))
 
@@ -127,7 +147,7 @@ Optional non-nil second argument INTERNAL-CMD inhibits display of the shell
 command line executed.  Optional non-nil third argument KILL-PREV means
 kill the last output to the shell buffer before executing SHELL-CMD."
   (interactive
-   (let ((default  (car hargs:defaults))
+   (let ((default  (nth 0 hargs:defaults))
 	 (default1 (nth 1 hargs:defaults))
 	 (default2 (nth 2 hargs:defaults)))
      (list (hargs:read "Shell cmd: "
@@ -207,11 +227,12 @@ kill the last output to the shell buffer before executing SHELL-CMD."
     (message msg)))
 
 (defact hyp-config (&optional out-buf)
-  "Insert Hyperbole configuration information at the end of the current buffer or within optional OUT-BUF."
+  "Insert Hyperbole configuration information at the end of the current buffer.
+Insert in optional OUT-BUF if specified."
   (hypb:configuration out-buf))
 
 (defact hyp-request (&optional out-buf)
-  "Insert into optional OUT-BUF a description of how to subscribe or unsubscribe from a Hyperbole mail list via email."
+  "Insert into optional OUT-BUF how to (un)subscribe from a Hyperbole mail list."
   (save-excursion
     (and out-buf (set-buffer out-buf))
     ;; Allows for insertion prior to user's email signature
@@ -277,7 +298,7 @@ This type of link is for use within a single editor session.  Use
   (hpath:find directory))
 
 (defact link-to-ebut (key &optional key-file)
-  "Perform action given by an explicit button, specified by KEY and optional KEY-FILE.
+  "Perform explicit button action specified by KEY and optional KEY-FILE.
 Interactively, KEY-FILE defaults to the current buffer's file name."
   (interactive
    (let (but-lbl
@@ -420,8 +441,8 @@ the window or as close as possible."
 	  (compile-goto-error)
 	(hpath:find-line path line-num))))
 
-(defact link-to-file-line-and-column (path line-num column-num)
-  "Display a file given by PATH scrolled to LINE-NUM with point at COLUMN-NUM."
+(defact link-to-file-line-and-column (path line-num col-num)
+  "Display a file given by PATH scrolled to LINE-NUM with point at COL-NUM."
   (interactive "fPath to link to: \nnDisplay at line number: \nnand column number: ")
   ;; Remove any double quotes and whitespace at the start and end of
   ;; the path that interactive use may have introduced.
@@ -431,9 +452,12 @@ the window or as close as possible."
 	  (error t))
     (if (and (stringp path) (not (file-name-absolute-p path))
 	     (compilation-buffer-p (current-buffer)))
-	(compile-goto-error)
-      (hpath:find-line path line-num))
-    (move-to-column column-num)))
+	(progn (compile-goto-error)
+	       (move-to-column
+		(if (string-match "\\.kotl?\\(\\'\\|#\\)" path)
+		    (+ (kcell-view:indent) col-num)
+		  col-num)))
+      (hpath:find-line path line-num col-num))))
 
 (defact link-to-gbut (key &optional _key-file)
   "Perform an action given by an existing global button, specified by KEY.
@@ -471,7 +495,8 @@ suffix."
     (hypb:error "(link-to-Info-index-entry): Invalid Info index item: `%s'" index-item)))
 
 (defact link-to-Info-node (string)
-  "Display an Info node given by STRING or if not found, try to display it as an Info index item.
+  "Display an Info node given by STRING.
+If not found, try to display it as an Info index item.
 STRING must be a string of the form \"(filename)name\".  During
 button creation, completion for both filename and node names is
 available.  Filename may be given without the .info suffix."
@@ -484,8 +509,9 @@ available.  Filename may be given without the .info suffix."
       (id-info string)
     (hypb:error "(link-to-Info-node): Invalid Info node: `%s'" string)))
 
-(defact link-to-ibut (key &optional but-src point)
-  "Perform an action given by an implicit button, specified by KEY, optional BUT-SRC and POINT.
+(defact link-to-ibut (name-key &optional but-src point)
+  "Activate implicit button given by NAME-KEY, optional BUT-SRC and POINT.
+NAME-KEY must be a normalized key for an ibut <[name]>.
 BUT-SRC defaults to the current buffer's file or if there is no
 attached file, then to its buffer name.  POINT defaults to the
 current point.
@@ -493,10 +519,10 @@ current point.
 When the button with this action type is created, point must be
 on the implicit button to which to link."
   (interactive
-   (let ((ibut-key (ibut:at-p t)))
-     (cond (ibut-key
-	    (list ibut-key (or buffer-file-name (buffer-name)) (point)))
-	   ;; TODO: If default is null below and are creating, rather than editing
+   (let ((ibut-name-key (ibut:at-p t)))
+     (cond (ibut-name-key
+	    (list ibut-name-key (or buffer-file-name (buffer-name)) (point)))
+	   ;; !! TODO: If default is null below and are creating, rather than editing
 	   ;; the link, it would be better to throw an error than create
 	   ;; an invalid link, but it is difficult to tell which operation
 	   ;; is in progress, so ignore this for now.  -- RSW, 01-25-2020
@@ -506,30 +532,30 @@ on the implicit button to which to link."
 	    hargs:defaults)
 	   (t
 	    (hypb:error "(link-to-ibut): Point must be on an implicit button to create a link-to-ibut")))))
-  (when (null key)
+
+  (unless name-key
     (hypb:error "(link-to-ibut): Point must be on an implicit button to create a link-to-ibut"))
   (let (but
 	normalized-file)
-    (cond (but-src
-	   (unless (and (get-buffer but-src)
-			(not (buffer-file-name (get-buffer but-src))))
-	     (setq normalized-file (hpath:normalize but-src))))
-	  (t (setq normalized-file buffer-file-name)))
-    (save-excursion
-      (save-restriction
-	(when but-src
-	  (set-buffer (or (get-buffer but-src) (get-file-buffer normalized-file))))
-	(widen)
-	(when (or (not normalized-file) (hmail:editor-p) (hmail:reader-p))
-	  (hmail:msg-narrow))
-	(when (integerp point)
-	  (goto-char (min point (point-max))))
-	(setq but (ibut:to key))))
+    (if but-src
+	(unless (and (get-buffer but-src)
+		     (not (buffer-file-name (get-buffer but-src))))
+	  (setq normalized-file (hpath:normalize but-src)))
+      (setq normalized-file (hpath:normalize buffer-file-name)))
+    (when but-src
+      (set-buffer (or (get-buffer but-src) (get-file-buffer normalized-file))))
+    (widen)
+    (when (or (not normalized-file) (hmail:editor-p) (hmail:reader-p))
+      (hmail:msg-narrow))
+    (when (integerp point)
+      (goto-char (min point (point-max))))
+    (setq but (ibut:to-text name-key))
     (cond (but
+	   (setq but (ibut:at-p))
 	   (hbut:act but))
-	  (key
-	   (hypb:error "(link-to-ibut): No implicit button `%s' found in `%s'"
-		       (ibut:key-to-label key)
+	  (name-key
+	   (hypb:error "(link-to-ibut): No implicit button named `%s' found in `%s'"
+		       (ibut:key-to-label name-key)
 		       (or but-src (buffer-name))))
 	  (t
 	   (hypb:error "(link-to-ibut): Link reference is null/empty")))))
@@ -538,27 +564,38 @@ on the implicit button to which to link."
   "Display FILE with kcell given by CELL-REF at window top.
 See documentation for `kcell:ref-to-id' for valid cell-ref formats.
 
-If FILE is nil, use the current buffer.
+If FILE is nil, use any source location or the current buffer.
 If CELL-REF is nil, show the first cell in the view."
-  (interactive "fKotl file to link to: \n+KKcell to link to: ")
+  (interactive (hargs:iform-read '(interactive "fKotl file to link to: \n+KKcell to link to: ")))
   (require 'kfile)
-  (cond ((and (stringp cell-ref) (> (length cell-ref) 0)
-	      (eq ?| (aref cell-ref 0)))
-	 ;; Activate view spec in current window.
-	 (kotl-mode:goto-cell cell-ref))
-	((if file
-	     (hpath:find file)
-	   (hpath:display-buffer (current-buffer)))
-	 (if cell-ref
-	     (kotl-mode:goto-cell cell-ref)
-	   (kotl-mode:beginning-of-buffer))
-	 (recenter 0))))
+  ;; May already be in an 'hpath:find' call where the referent buffer
+  ;; has already been made current.  In that case, skip a second
+  ;; display of the buffer.
+  (unless (and (hyperb:stack-frame '(hpath:find))
+	       ;; Perform a loose test that the current buffer
+	       ;; file name matches the path file name since exact
+	       ;; matching of path is likely to be wrong in
+	       ;; certain cases, e.g. with mount point or os path
+	       ;; alterations.
+	       buffer-file-name
+	       file
+	       (or (null file)
+		   (string-empty-p file)
+		   (equal (file-name-nondirectory file)
+			  (file-name-nondirectory buffer-file-name))))
+    (when (or (null file) (string-empty-p file))
+      (setq file (hbut:get-key-src t)))
+    (if (stringp file)
+	(hpath:find file)
+      ;; file can be a buffer from get-key-src call
+      (hpath:display-buffer (or file (current-buffer)))))
+  (kotl-mode:goto-cell-ref cell-ref))
 
 (defact link-to-mail (mail-msg-id &optional mail-file)
   "Display mail msg with MAIL-MSG-ID from optional MAIL-FILE.
 See documentation for the variable `hmail:init-function' for
 information on how to specify a mail reader to use."
-  (interactive "+MMail Msg: ")
+  (interactive (hargs:iform-read '(interactive "+MMail Msg: ")))
   (if (not (fboundp 'rmail:msg-to-p))
       (hypb:error "(link-to-mail): Invoke mail reader before trying to follow a mail link")
     (if (and (listp mail-msg-id) (null mail-file))
@@ -578,6 +615,26 @@ information on how to specify a mail reader to use."
 	(hypb:error "(link-to-mail): No msg `%s' in file \"%s\""
 		    mail-msg-id mail-file)))))
 
+(defact link-to-org-id (id)
+  "Display the Org entry, if any, for ID."
+  (when (stringp id)
+    (let* ((inhibit-message t) ;; Inhibit org-id-find status msgs
+	   (m (or (and (featurep 'org-roam) (org-roam-id-find id 'marker))
+		  (org-id-find id 'marker))))
+      (when m
+	(hact 'link-to-org-id-marker m)))))
+
+(defact link-to-org-id-marker (marker)
+  "Display the Org entry, if any, at MARKER.
+See doc of `ibtypes::org-id' for usage."
+    (unless (markerp marker)
+      (error "(link-to-org-id-marker): Argument must be a marker, not %s" marker))
+    (org-mark-ring-push)
+    (hact 'link-to-buffer-tmp (marker-buffer marker) marker)
+    (move-marker marker nil)
+    (when (featurep 'org-fold) ;; newer Org versions
+      (org-fold-show-context)))
+
 (defact link-to-regexp-match (regexp n source &optional buffer-p)
   "Find REGEXP's Nth occurrence in SOURCE and display location at window top.
 SOURCE is a pathname unless optional BUFFER-P is non-nil, then SOURCE must be
@@ -586,14 +643,14 @@ Return t if found, signal an error if not."
   (interactive "sRegexp to match: \nnOccurrence number: \nfFile to search: ")
   (let ((orig-src source))
     (if buffer-p
-	(if (stringp source)
-	    (setq source (get-buffer source)))
+	(when (stringp source)
+	  (setq source (get-buffer source)))
       ;; Source is a pathname.
       (if (not (stringp source))
 	  (hypb:error
 	   "(link-to-regexp-match): Source parameter is not a filename: `%s'"
 	   orig-src)
-	(setq source (find-file-noselect (hpath:substitute-value source)))))
+	(setq source (hpath:find-noselect source))))
     (if (not (bufferp source))
 	(hypb:error
 	 "(link-to-regexp-match): Invalid source parameter: `%s'" orig-src)
@@ -630,7 +687,7 @@ FILE may be a string or nil, in which case the current buffer is used."
     (setq node (replace-regexp-in-string "[ \t\n\r\f]+" " " node t t)))
   (let (node-point)
     (if file
-        (set-buffer (find-find-noselect (hpath:substitute-value file)))
+        (set-buffer (find-file-noselect (hpath:substitute-value file)))
       (setq file buffer-file-name))
     (save-excursion
       (goto-char (point-min))
@@ -659,52 +716,55 @@ package to display search results."
 Uses `hpath:display-where' setting to control where the man page is displayed."
   (interactive "sManual topic: ")
   (require 'man)
+  (defvar Man-notify-method)
   (let ((Man-notify-method 'meek))
     (hpath:display-buffer (man topic))))
 
 (defact rfc-toc (&optional buf-name opoint sections-start)
   "Compute and display summary of an Internet rfc in BUF-NAME.
 Assume point has already been moved to start of region to summarize.
-Optional OPOINT is point to return to in BUF-NAME after displaying summary."
+Optional OPOINT is point to return to in BUF-NAME after displaying
+summary; otherwise, point remains in the toc occurrence buffer.
+Optional SECTIONS-START limits toc entries to those after that point."
   (interactive)
-  (if buf-name
-      (cond ((get-buffer buf-name)
-	     (switch-to-buffer buf-name))
-	    ((let ((buf (get-file-buffer buf-name)))
-	       (when buf
-		 (switch-to-buffer (setq buf-name buf))
-		 t)))
-	    (t (if opoint (goto-char opoint))
-	       (hypb:error "(rfc-toc): Invalid buffer name: %s" buf-name))))
   (let ((sect-regexp "^[ \t]*[1-9][0-9]*\\.[0-9.]*[ \t]+[^ \t\n\r]")
-	(temp-buffer-show-function 'switch-to-buffer)
-	occur-buffer)
-    (hpath:display-buffer (current-buffer))
+	(rfc-buf-name (buffer-name))
+	(toc-buf-name (format "*toc %s*" buf-name)))
+    (when (get-buffer toc-buf-name)
+      (kill-buffer toc-buf-name))
     (occur sect-regexp nil (list (cons sections-start (point-max))))
-    (setq occur-buffer (set-buffer "*Occur*"))
-    (rename-buffer (format "*%s toc*" buf-name))
+    (select-window (get-buffer-window "*Occur*"))
+    (rename-buffer toc-buf-name)
     (re-search-forward "^[ ]*[0-9]+:" nil t)
     (beginning-of-line)
-    (delete-region (point-min) (point))
-    (insert "Contents of " (buffer-name occur-buffer) ":\n")
-    (set-buffer-modified-p nil)
-    (set-buffer buf-name)
-    (if opoint (goto-char opoint))))
+    (let ((inhibit-read-only t)
+	  (buffer-read-only))
+      (remove-text-properties (point-min) (point) '(read-only))
+      (delete-region (point-min) (point))
+      (insert "Sections of " rfc-buf-name ":\n")
+      (set-buffer-modified-p nil))
+    (when opoint
+      (switch-to-buffer buf-name)
+      (goto-char opoint))))
 
 (defact text-toc (section)
-  "Jump to the text file SECTION referenced by a table of contents entry at point."
+  "Jump to the text file SECTION referenced by a table of contents entry at point.
+SECTION is a string and can be just the leading part of a section heading."
   (interactive "sGo to section named: ")
-  (if (stringp section)
-      (progn
+  (when (stringp section)
+    (setq section (string-trim section))
+    (if (string-match "\\`\\(\\*+\\)[ \t]*" section)
 	(actypes::link-to-regexp-match
-	 (concat "^\\*+[ \t]*" (regexp-quote section))
-	 1 (current-buffer) t)
-	(while (and (= (forward-line -1) 0)
-		    (looking-at "[ \t]*[-=][-=]")))
-	(forward-line 1)
-	(recenter 0))))
+	 (concat "^[ \t]*" (regexp-quote (match-string 1 section))
+		 "[ \t]*" (regexp-quote (substring section (match-end 0))))
+	 2 (current-buffer) t)
+      (actypes::link-to-regexp-match (concat "^[ \t]*" (regexp-quote section))
+				     2 (current-buffer) t))
+    (while (and (= (forward-line -1) 0)
+		(looking-at "[ \t]*[-=][-=]")))
+    (forward-line 1)
+    (recenter 0)))
 
 (provide 'hactypes)
 
 ;;; hactypes.el ends here
-
