@@ -3,7 +3,9 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     6-Oct-91 at 06:34:05
-;; Last-Mod:     24-Jan-22 at 00:18:23 by Bob Weiner
+;; Last-Mod:      3-Oct-23 at 17:49:44 by Mats Lidell
+;;
+;; SPDX-License-Identifier: GPL-3.0-or-later
 ;;
 ;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
@@ -20,6 +22,27 @@
 (defvar hbmap:filename "HYPB"
   "*Filename used for quick access button files.")
 
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defvar hyperb:microsoft-os-p)          ; Defined in hload-path.
+
+(defvar hbmap:dir-user
+  (if (and hyperb:microsoft-os-p
+	   (not (getenv "HOME")))
+      "c:/_hyperb/" "~/.hyperb/")
+  "Per user directory in which to store top level Hyperbole map data.
+Must end with a directory separator.
+Hyperbole will try to create it whenever `hyperb:init' is called.")
+
+(defvar hbmap:dir-filename
+  (expand-file-name  "HBMAP" hbmap:dir-user)
+  "Name of a file that lists all dirs to which a user has written buttons.
+See also `hbmap:dir-user'.
+If you change its value, you will be unable to search for buttons created by
+others who use a different value!")
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
@@ -103,25 +126,6 @@ the error.  Optional NO-SAVE disables saving of the map after operation."
 			 (t 'hbmap-not-writable))))
 	'hbmap-not-readable))))
 
-;;; ************************************************************************
-;;; Private variables
-;;; ************************************************************************
-
-(defvar hbmap:dir-user
-  (if (and hyperb:microsoft-os-p
-	   (not (getenv "HOME")))
-      "c:/_hyperb/" "~/.hyperb/")
-  "Per user directory in which to store top level Hyperbole map data.
-Must end with a directory separator.
-Hyperbole will try to create it whenever 'hyperb:init' is called.")
-
-(defvar hbmap:dir-filename
-  (expand-file-name  "HBMAP" hbmap:dir-user)
-  "Name of a file that lists all dirs to which a user has written buttons.
-See also 'hbmap:dir-user'.
-If you change its value, you will be unable to search for buttons created by
-others who use a different value!")
-
 (provide 'hbmap)
 
 ;;; hbmap.el ends here
