@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    24-Dec-91 at 22:29:28
-;; Last-Mod:     24-Jan-22 at 00:31:29 by Bob Weiner
+;; Last-Mod:      5-Feb-23 at 23:40:25 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -33,7 +35,7 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(hmail hsmail hypb gnus-msg)))
+(eval-and-compile (mapc #'require '(hload-path hmail hsmail hypb gnus-msg)))
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -71,13 +73,6 @@
 	(gnus-summary-display-article article))))
 
 
-;;; Redefine 'gnus-inews-article' from "gnuspost.el" to make it include
-;;; any signature before Hyperbole button data.  Does this by having
-;;; signature inserted within narrowed buffer and then applies a hook to
-;;; have the buffer widened before sending.
-(hypb:function-symbol-replace
-  'gnus-inews-article 'widen 'hmail:msg-narrow)
-
 ;;; Overload this function from "rnewspost.el" for supercite compatibility
 ;;; only when supercite is in use.
 (if (hypb:supercite-p)
@@ -112,17 +107,13 @@ as ARG means don't indent and don't delete any header fields."
 (if (fboundp 'hproperty:but-create)
     (var:append 'gnus-summary-prepare-hook '(hproperty:but-create)))
 
-;;; Try to setup comment addition as the first element of these hooks.
-(if (fboundp 'add-hook)
-    ;; Called from 'news-post-news' if prev unsent article exists and user
-    ;; says erase it.  Add a comment on Hyperbole button support.
-    (progn
-      (add-hook 'news-setup-hook #'smail:comment-add)
-      ;; Called from 'news-post-news' if no prev unsent article exists.
-      ;; Add a comment on Hyperbole button support.
-      (add-hook 'news-reply-mode-hook #'smail:comment-add))
-  (var:append 'news-setup-hook '(smail:comment-add))
-  (var:append 'news-reply-mode-hook '(smail:comment-add)))
+;; Try to setup comment addition as the first element of these hooks.
+;; Called from 'news-post-news' if prev unsent article exists and user
+;; says erase it.  Add a comment on Hyperbole button support.
+(add-hook 'news-setup-hook #'smail:comment-add)
+;; Called from 'news-post-news' if no prev unsent article exists.
+;; Add a comment on Hyperbole button support.
+(add-hook 'news-reply-mode-hook #'smail:comment-add)
 
 (provide 'hgnus)
 
