@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    24-Apr-91 at 03:36:23
-;; Last-Mod:     17-Apr-22 at 12:41:30 by Bob Weiner
+;; Last-Mod:      3-Oct-23 at 17:05:19 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -20,13 +22,22 @@
 ;;   Frames which have been deleted are not restored.
 
 ;;; Code:
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defconst *hhist* nil
+  "List of previously visited Hyperbole button source locations.
+Car of list is most recent.")
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
 
 (defun hhist:add (elt)
-  "Add ELT to hyper-history list if not the same as current or prior location (frame configuration).
-ELT must have been created via a call to 'hhist:element' prior to
+  "Add ELT to hyper-history list.
+Do not add if the same as current or prior location (frame configuration).
+ELT must have been created via a call to `hhist:element' prior to
 changing the current frame configuration somehow."
   ;; Even though this next line looks useless, it cures a problem with
   ;; window buffer correspondences on startup, so don't remove it.
@@ -41,8 +52,8 @@ changing the current frame configuration somehow."
   "Return a history element for current point location."
   (current-frame-configuration))
 
-(defun hhist:remove (&optional arg)
-  "Remove optional prefix ARG entries from history, return to ARGth location.
+(defun hhist:pop (&optional arg)
+  "Return to ARGth saved frame config and remove optional prefix ARG entries.
 The command is ignored with ARG < 1."
   (interactive "p")
   (setq arg (cond ((or (null arg)
@@ -56,7 +67,7 @@ The command is ignored with ARG < 1."
   (let ((prev-config))
     (when *hhist*
       (when (< arg 1)
-	(message "(hhist:remove): No previous location to which to return.")
+	(message "(hhist:pop): No previous location to which to return")
 	(beep))
       (while (and (> arg 0) *hhist*)
 	(setq prev-config (car *hhist*)
@@ -71,14 +82,6 @@ The command is ignored with ARG < 1."
   (interactive)
   (setq *hhist* nil))
 
-;;; ************************************************************************
-;;; Private variables
-;;; ************************************************************************
-
-(defconst *hhist* nil
-  "List of previously visited Hyperbole button source locations.
-Car of list is most recent.")
-
 (provide 'hhist)
 
 ;;; hhist.el ends here
