@@ -3,13 +3,15 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    21-Jun-16 at 14:24:53
-;; Last-Mod:     24-Jan-22 at 00:18:32 by Bob Weiner
+;; Last-Mod:      2-Feb-24 at 22:44:22 by Mats Lidell
 ;;
-;; Copyright (C) 2016, 2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 2016-2024  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of Hyperbole.  It requires the Emacs package
-;; Debbugs 0.9.7 or higher; there were bugs in earlier versions
+;; debbugs-gnu 0.9.7 or higher; there were bugs in earlier versions
 ;; that made it incompatible with the queries Hyperbole issues.
 ;;
 ;;; Commentary:
@@ -71,32 +73,34 @@
 (eval-and-compile (mapc #'require '(hactypes)))
 (eval-when-compile (require 'debbugs-gnu nil t))
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function debbugs-get-status "ext:debbugs-gnu")
+(declare-function debbugs-gnu-bugs "ext:debbugs-gnu")
+(declare-function debbugs-gnu-current-id "ext:debbugs-gnu")
+(declare-function debbugs-gnu-select-report "ext:debbugs-gnu")
+(declare-function debbugs-gnu-show-reports "ext:debbugs-gnu")
+(defvar debbugs-gnu-current-query)
+(defvar debbugs-port)
+
 ;;; ************************************************************************
 ;;; Public variables
 ;;; ************************************************************************
 
 (eval-after-load "debbugs-gnu"
-  #'(progn (push "hyperbole"  debbugs-gnu-all-packages)
-	   (push "oo-browser" debbugs-gnu-all-packages)))
-
-;;; ************************************************************************
-;;; Public declarations
-;;; ************************************************************************
-
-(declare-function debbugs-get-status "ext:debbugs")
-(declare-function debbugs-gnu-bugs "ext:debbugs")
-(declare-function debbugs-gnu-current-id "ext:debbugs")
-(declare-function debbugs-gnu-select-report "ext:debbugs")
-(declare-function debbugs-gnu-show-reports "ext:debbugs")
-(defvar debbugs-gnu-current-query)
-(defvar debbugs-port)
+  '(progn
+     (defvar debbugs-gnu-all-packages)
+     (push "hyperbole"  debbugs-gnu-all-packages)
+     (push "oo-browser" debbugs-gnu-all-packages)))
 
 ;;; ************************************************************************
 ;;; Public implicit button types
 ;;; ************************************************************************
 
 (defib debbugs-gnu-query ()
-  "Display the results of a Debbugs query based on a bug reference string around point.
+  "Display the results of a Debbugs query from a bug reference string around point.
 This works in most types of buffers.  If the query includes a
 single id number, display the original message submission for
 that id and allow browsing of the followup discussion.  Accepts
@@ -124,7 +128,8 @@ Note that `issue' or `debbugs' may be used as well in place of `bug'."
     (debbugs-gnu-show-discussion)))
 
 (defun debbugs-gnu-query:help (_but)
-  "Make a GNU debbugs id number at point (optionally prefixed with a # sign) display the pretty pretted status of the bug id.
+  "Make a Gnu debbugs id number at point display the pretty-printed bug status.
+The id number can optionally be prefixed with a # sign.
 Ignore other types of GNU debbugs query strings."
   (if (and (debbugs-version-sufficient-p)
 	   (debbugs-query:at-p)
@@ -139,33 +144,34 @@ Ignore other types of GNU debbugs query strings."
       (hact 'smart-debbugs-gnu)))
 
 (defun debbugs-gnu-mode:help (&optional _but)
-  "Make a Gnu debbugs listing entry at point pretty print the status of the issue to a window below."
-  (condition-case ()
-      (let ((display-buffer-overriding-action
-	     '(display-buffer-below-selected . nil)))
-	(debbugs-query:status (debbugs-gnu-current-id))
-	(hypb:maximize-window-height))
-    (error nil)))
+  "Make a Gnu debbugs listing at point pretty-print its status to a window below."
+  (ignore-errors
+    (let ((display-buffer-overriding-action
+	   '(display-buffer-below-selected . nil)))
+      (debbugs-query:status (debbugs-gnu-current-id))
+      (hypb:maximize-window-height))))
 
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
 
 (defun debbugs-gnu-show-discussion ()
+  "Display the 2nd message which is the initial bug report.
+This may be in Gnus or Rmail summary mode."
     (debbugs-gnu-select-report)
-    ;; Display the 2nd message which is the initial bug report.  This
-    ;; may be in Gnus or Rmail summary mode.
     (goto-char (point-min))
     (forward-line 1)
     (call-interactively (key-binding "\C-m")))
 
 (defun debbugs-gnu-query:string (url-query-string)
-  "Parse and apply attributes from URL-QUERY-STRING to display the results of a Gnu debbugs query.
-URL-QUERY-STRING must be a valid URL query string (part after the question
-mark) of debbugs attributes and values, i.e. \"attr1=val1&attr2=val2&attr3=val3\"
-URL encoded characters are decoded.  An optional prefix of \"bug#<id-number>?\"
-may also be included at the front of the string to limit the query to a particular
-issue number.  Note that `issue' or `debbugs' may be used as well in place of `bug'."
+  "Show the results of a Gnu debbugs query with attributes from URL-QUERY-STRING.
+URL-QUERY-STRING must be a valid URL query string (part after the
+question mark) of debbugs attributes and values,
+i.e. \"attr1=val1&attr2=val2&attr3=val3\" URL encoded characters
+are decoded.  An optional prefix of \"bug#<id-number>?\" may also
+be included at the front of the string to limit the query to a
+particular issue number.  Note that `issue' or `debbugs' may be
+used as well in place of `bug'."
   (let* ((case-fold-search t)
 	 (id (when (string-match "\\`\\(bug\\|debbugs\\|issue\\)\\s-?#?\\s-?\\(\\([1-9][0-9]*\\)\\|\\?\\)+"
 				 url-query-string)
@@ -181,11 +187,11 @@ issue number.  Note that `issue' or `debbugs' may be used as well in place of `b
     (debbugs-gnu-query:list attr-pair-list)))
 
 (defun debbugs-gnu-query:list (query-attribute-list)
-  "Apply attributes from QUERY-ATTRIBUTE-LIST to display the results of a Gnu debbugs query.
+  "Show the results of a Gnu debbugs query with QUERY-ATTRIBUTE-LIST attributes.
 Each element of the list should be of the form (attribute . attribute-value).
-Attribute may be a symbol or a string.  Common attributes include: status,
-severity, and package."
-  (require 'debbugs-gnu)
+Attribute may be a symbol or a string.  Common attributes
+include: status, severity, and package."
+  (require 'debbugs)
   (setq debbugs-gnu-current-query nil)
   (dolist (attr query-attribute-list)
     (add-to-list 'debbugs-gnu-current-query
@@ -196,7 +202,8 @@ severity, and package."
   (debbugs-gnu-show-reports))
 
 (defun smart-debbugs-gnu ()
-  "Display the discussion on the issue at point when the Action Key is pressed on a Gnu Debbugs listing entry ."
+  "Display the discussion on the issue at point.
+When the Action Key is pressed on a Gnu Debbugs listing entry."
   (debbugs-gnu-show-discussion))
 
 ;; (let ((entries (cdar tabulated-list-entries)))
@@ -214,7 +221,7 @@ severity, and package."
   "Return t if point appears to be within a debbugs id.
 Id number is (match-string 2).  If this is a query with attributes,
 then (match-string 3) = \"?\" and (match-string 4) is the query
-attributes." 
+attributes."
   ;; Point must be before one of the bug#222 characters to match.
   (let ((case-fold-search t))
     (when (string-match "[bugise#0-9]" (char-to-string (following-char)))
@@ -229,8 +236,9 @@ attributes."
 	      ;; (looking-at "[ \t\n\r\f]*\\(bug\\|debbugs\\|issue\\)?[ \t\n\r\f]*#\\([1-9][0-9]*\\)[\].,;?!\)\>\}]?\\([ \t\n\r\f]\\|\\'\\)")
 
 (defun debbugs-query:status (id)
-  "Pretty print to `standard-output' the status attributes of debbugs ID (a positive integer).
-Ignore nil valued attributes.  Return t unless no attributes are printed."
+  "Pretty print to `standard-output' the status attributes of debbugs ID.
+Debbugs ID is a positive integer.  Ignore nil valued attributes.
+Return t unless no attributes are printed."
   (require 'debbugs-gnu)
   ;; The (car (debbugs-get-status id)) is a list of (attribute . value) pairs which we sort below.
   (let ((attrib-list
@@ -254,9 +262,10 @@ Ignore nil valued attributes.  Return t unless no attributes are printed."
 	has-attr))))
 
 (defun debbugs-version-sufficient-p ()
-  "Return t iff debbugs version is sufficient for use with Hyperbole (greater than equal to 0.9.7)."
+  "Return t iff debbugs version is sufficient for use with Hyperbole.
+Must be greater than equal to 0.9.7."
   (save-excursion
-    (let* ((debbugs-src (locate-file "debbugs" load-path '(".el")))
+    (let* ((debbugs-src (locate-file "debbugs-gnu" load-path '(".el")))
 	   (visiting-debbugs-src (when debbugs-src (get-file-buffer debbugs-src)))
 	   debbugs-src-buffer
 	   version)
@@ -269,7 +278,7 @@ Ignore nil valued attributes.  Return t unless no attributes are printed."
 		     (setq version (match-string 1))))
 	  (unless visiting-debbugs-src
 	    (kill-buffer debbugs-src-buffer)))
-	(when (not (equal version ""))
+	(when (and version (not (equal version "")))
 	  (version-list-<= (version-to-list "0.9.7") (version-to-list version)))))))
 
 (provide 'hib-debbugs)
