@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    22-Nov-91 at 01:37:57
-;; Last-Mod:     24-Apr-22 at 17:34:04 by Bob Weiner
+;; Last-Mod:     18-Feb-24 at 11:39:41 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -34,42 +36,61 @@
 ;;; ************************************************************************
 
 (require 'hactypes) ;; This invokes (require 'hargs)
+(require 'hui-mini) ;; For `hui:menu-doc' and `hui:menu-help'
 (require 'seq)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function helm-mode "ext:helm")
+
 ;;; ************************************************************************
 ;;; Public variables
 ;;; ************************************************************************
+
 (defvar kbd-key:named-key-list
   '("add" "backspace" "begin" "bs" "clear" "decimal" "delete" "del"
     "divide" "down" "end" "enter" "esc" "home" "left" "insert"
     "multiply" "newline" "next" "prior" "return" "ret" "right" "rtn"
     "subtract" "tab" "up")
-  "List of dedicated keyboard key names which may be used with modifier keys.  Function keys are handled elsewhere.")
+  "List of dedicated keyboard key names which may be used with modifier keys.
+Function keys are handled elsewhere.")
 
 (defvar kbd-key:named-key-regexp
   (concat
    (mapconcat 'downcase kbd-key:named-key-list "\\|")
    "\\|"
    (mapconcat 'upcase kbd-key:named-key-list "\\|"))
-  "Regexp that matches to any of the dedicated keyboard key names in lower or uppercase.")
+  "Regexp that matches the dedicated keyboard key names in lower or uppercase.")
 
 (defvar kbd-key:modified-key-regexp
-  (concat "\\(\\[?\\([ACHMS]-\\|kp-\\)+\\)\\s-*\\(\\(<?\\<" kbd-key:named-key-regexp "\\>>?"
+  (concat "\\(\\[?\\([ACHMS]-\\|kp-\\)+\\)[ \t\n\r\f]*\\(\\(<?\\<" kbd-key:named-key-regexp "\\>>?"
 	  "\\|<?[fF][0-9][0-9]?>?\\|<[a-zA-Z0-9]+>\\|.\\)\\]?\\)")
-  "Regexp matching to a single modified keyboard key within a human-readable string.
-Group 1 matches to the set of modifier keys.  Group 3 matches to the unmodified key.")
+  "Regexp matching a single modified keyboard key within a human-readable string.
+Group 1 matches to the set of modifier keys.  Group 3 matches to
+the unmodified key.")
 
 ;;; ************************************************************************
-;;; Public declarations
+;;; Private variables
 ;;; ************************************************************************
-(declare-function helm-mode "ext:helm")
+
+(defconst kbd-key:extended-command-prefix "\\_<M-x\\_>"
+  "Normalized prefix regular expression that invokes an extended command.
+Default is \\`M-x'.")
+
+(defconst kbd-key:extended-command-binding-list '(execute-extended-command helm-M-x counsel-M-x)
+  "List of commands that may be bound to \\`M-x' to invoke extended/named commands.")
+
+(defvar kbd-key:mini-menu-key nil
+  "The key sequence that invokes the Hyperbole minibuffer menu.")
 
 ;;; ************************************************************************
 ;;; Public implicit button types
 ;;; ************************************************************************
 
 (defact kbd-key (key-series)
-  "Execute a normalized KEY-SERIES (series of key sequences) without curly braces, {}.
+  "Execute a normalized KEY-SERIES (series of key sequences) without curly braces.
 Each key sequence within KEY-SERIES must be a string of one of the following:
   a Hyperbole minibuffer menu item key sequence,
   a HyControl key sequence,
@@ -81,9 +102,11 @@ Return t if the sequence appears to be valid, else nil."
   (kbd-key:act key-series))
 
 (defib kbd-key ()
-  "Execute a key series (series of key sequences) around point, delimited by curly braces, {}.
-Key sequences should be in human readable form, e.g. {C-x C-b}, or what `key-description' returns.
-Forms such as {\C-b}, {\^b}, and {^M} will not be recognized.
+  "Execute a key series (series of key sequences) around point.
+The key series is delimited by curly braces, {}.  Key sequences
+should be in human readable form, e.g. {C-x C-b}, or what
+`key-description' returns.  Forms such as {\C-b}, {\^b}, and {^M}
+will not be recognized.
 
 Any key sequence within the series must be a string of one of the following:
   a Hyperbole minibuffer menu item key sequence,
@@ -94,48 +117,44 @@ Any key sequence within the series must be a string of one of the following:
 	      (and (looking-at "[{}]") (/= ?\\ (preceding-char))))
     ;; Temporarily make open and close braces have list syntax for
     ;; matching purposes.
-    (let ((open-brace-syntax (hypb:get-raw-syntax-descriptor ?\{))
-	  (close-brace-syntax (hypb:get-raw-syntax-descriptor ?\})))
-      (unwind-protect
-	  (progn (modify-syntax-entry ?\{ "(\}" (syntax-table))
-		 (modify-syntax-entry ?\} ")\}" (syntax-table))
-		 ;; Handle long series, e.g. eval-elisp actions
-		 (let* ((hbut:max-len (max 3000 (hbut:max-len)))
-			(seq-and-pos (or ;; (kbd) calls but only if point is between double quotes
- 				      (and (hbut:label-p t "(kbd \"" "\"\)" t)
-					   (hbut:label-p t "\"" "\"\)" t))
-				      (hbut:label-p t "{`" "'}" t)
-				      (hbut:label-p t "{" "}" t)
-				      ;; Regular dual single quotes (Texinfo smart quotes)
-				      (hbut:label-p t "``" "''" t)
-				      ;; Typical GNU manual key sequences; note
-				      ;; these are special quote marks, not the
-				      ;; standard ASCII characters.
-				      (hbut:label-p t "‘" "’" t)))
-			;; This excludes delimiters
-			(key-series (car seq-and-pos))
-			(start (cadr seq-and-pos))
-			binding)
-		   ;; Match only when start delimiter is preceded by whitespace,
-		   ;; double quotes or is the 1st buffer character, so do not
-		   ;; match to things like ${variable}.
-		   (when (memq (char-before start) '(nil ?\ ?\t ?\n ?\r ?\f ?\"))
-		     (when (and (stringp key-series)
-				(not (eq key-series "")))
-		       ;; Replace any ${} internal or env vars; leave
-		       ;; $VAR untouched for the shell to evaluate.
-		       (let ((hpath:variable-regexp "\\${\\([^}]+\\)}"))
-			 (setq key-series (hpath:substitute-value key-series)))
+    (with-syntax-table text-mode-syntax-table
+      ;; Handle long series, e.g. eval-elisp actions
+      (let* ((hbut:max-len (max 3000 (hbut:max-len)))
+	     (seq-and-pos (or
+			   ;; (kbd) calls but only if point is between double quotes
+ 			   (and (hbut:label-p t "(kbd \"" "\"\)" t)
+				(hbut:label-p t "\"" "\"\)" t))
+			   ;; braces delimiters
+			   (hbut:label-p t "{`" "'}" t)
+			   (hbut:label-p t "{" "}" t)
+			   ;; Regular dual single quotes (Texinfo smart quotes)
+			   (hbut:label-p t "``" "''" t)
+			   ;; Typical GNU manual key sequences; note
+			   ;; these are special quote marks, not the
+			   ;; standard ASCII characters.
+			   (hbut:label-p t "‘" "’" t)))
+	     ;; This excludes delimiters
+	     (key-series (car seq-and-pos))
+	     (start (cadr seq-and-pos))
+	     binding)
+	;; Match only when start delimiter is preceded by whitespace,
+	;; double quotes or is the 1st buffer character, so do not
+	;; match to things like ${variable}.
+	(when (memq (char-before start) '(nil ?\ ?\t ?\n ?\r ?\f ?\"))
+	  (when (and (stringp key-series)
+		     (not (string-equal key-series "")))
+	    ;; Replace any ${} internal or env vars; leave
+	    ;; $VAR untouched for the shell to evaluate.
+	    (let ((hpath:variable-regexp "\\${\\([^}]+\\)}"))
+	      (setq key-series (hpath:substitute-value key-series)))
 
-		       (setq key-series (kbd-key:normalize key-series)
-			     binding (kbd-key:binding key-series)))
-		     (and (stringp key-series)
-			  (or (and binding (not (integerp binding)))
-			      (kbd-key:special-sequence-p key-series))
-			  (ibut:label-set seq-and-pos)
-			  (hact 'kbd-key key-series)))))
-	(hypb:set-raw-syntax-descriptor ?\{ open-brace-syntax)
-	(hypb:set-raw-syntax-descriptor ?\} close-brace-syntax)))))
+	    (setq key-series (kbd-key:normalize key-series)
+		  binding (kbd-key:binding key-series)))
+	  (and (stringp key-series)
+	       (or (and binding (not (integerp binding)))
+		   (kbd-key:special-sequence-p key-series))
+	       (ibut:label-set seq-and-pos)
+	       (hact 'kbd-key key-series)))))))
 
 ;;; ************************************************************************
 ;;; Public functions
@@ -161,7 +180,8 @@ Return t if KEY-SERIES appears valid, else nil."
 	   t))))
 
 (defun kbd-key:execute (key-series)
-   "Execute a possibly non-normalized KEY-SERIES with or without curly brace delimiters.
+  "Execute a possibly non-normalized KEY-SERIES.
+The KEY-SERIES can be with or without curly brace delimiters.
 Return t if KEY-SERIES is a valid key series that is executed, else nil."
   (interactive "sKey series to execute: ")
   (when (and key-series
@@ -169,8 +189,8 @@ Return t if KEY-SERIES is a valid key series that is executed, else nil."
     (hact #'kbd-key:act key-series)))
 
 (defun kbd-key:execute-special-series (key-series)
-  "Execute key series."
-  (if (memq (key-binding [?\M-x]) #'(execute-extended-command counsel-M-x))
+  "Execute KEY-SERIES."
+  (if (memq (key-binding [?\M-x]) '(execute-extended-command counsel-M-x))
       (kbd-key:key-series-to-events key-series)
     ;; Disable helm while processing M-x commands; helm
     ;; gobbles final RET key.  Counsel works without modification.
@@ -187,16 +207,21 @@ Return t if KEY-SERIES is a valid key series that is executed, else nil."
 		 helm-flag orig-binding))))))
 
 (defun kbd-key:maybe-enable-helm (helm-flag orig-M-x-binding)
-  "Enable helm-mode if HELM-FLAG is non-nil.  Restore M-x binding to ORIG-M-X-BINDING."
+  "Enable helm-mode if HELM-FLAG is non-nil.
+Restore \\`M-x' binding to ORIG-M-X-BINDING."
   (when helm-flag (helm-mode 1))
   (global-set-key [?\M-x] orig-M-x-binding))
 
 (defun kbd-key:key-series-to-events (key-series)
-  "Insert the key-series as a series of keyboard events into Emacs' unread input stream.
-Emacs then executes them when its command-loop regains control."
+  "Insert the KEY-SERIES as a series of keyboard events.
+The events are inserted into Emacs unread input stream.  Emacs
+then executes them when its command-loop regains control."
   (setq unread-command-events (nconc unread-command-events
-				     (listify-key-sequence
-				      (kbd-key:kbd key-series)))))
+				     ;; Cons t here to ensure events
+				     ;; are added to command-keys.
+				     (mapcar (lambda (e) (cons t e))
+					     (listify-key-sequence
+					      (kbd-key:kbd key-series))))))
 
 (defun kbd-key:doc (key-series &optional full)
   "Show first line of doc for binding of keyboard KEY-SERIES in minibuffer.
@@ -231,14 +256,17 @@ With optional prefix arg FULL, display full documentation for command."
       (kbd-key:doc kbd-key t))))
 
 (defun kbd-key:is-p (str)
-  "If STR is a curly-brace {} delimited key series, return the non-delimited, normalized form, else nil.
-Key sequences should be in human readable form, e.g. {C-x C-b}, or what `key-description' returns.
-Forms such as {\C-b}, {\^b}, and {^M} will not be recognized.
+  "Return the non-delimited, normalized form, of a delimited key series, STR.
+When STR is a curly-brace {} delimited key series, a
+non-delimited, normalized form is returned, else nil.  Key
+sequences should be in human readable form, e.g. {\\`C-x' \\`C-b'}, or
+what `key-description' returns.  Forms such as {\\`C-b'}, {\^b}, and
+{^M} will not be recognized.
 
 Any key sequence within the series must be a string of one of the following:
   a Hyperbole minibuffer menu item key sequence,
   a HyControl key sequence,
-  a M-x extended command,
+  a \\`M-x' extended command,
   or a valid key sequence together with its interactive arguments."
   ;; Handle long series, e.g. eval-elisp actions
   (let* ((hbut:max-len (max 3000 (hbut:max-len)))
@@ -253,7 +281,7 @@ Any key sequence within the series must be a string of one of the following:
 			 (kbd-key:remove-delimiters str "‘" "’")))
 	 binding)
     (when (and (stringp key-series)
-	       (not (eq key-series "")))
+	       (not (string-equal key-series "")))
       (setq key-series (kbd-key:normalize key-series)
 	    binding (kbd-key:binding key-series)))
     (and (stringp key-series)
@@ -262,9 +290,10 @@ Any key sequence within the series must be a string of one of the following:
 	 key-series)))
 
 (defun kbd-key:normalize (key-series)
-  "Normalize a human-readable string of keyboard keys, KEY-SERIES (without any surrounding {}).
-Return the normalized but still human-readable format.
-Use `kbd-key:key-series-to-events' to add the key series to Emacs'
+  "Normalize a human-readable string of keyboard keys, KEY-SERIES.
+The KEY-SERIES is without any surrounding {}.  Return the
+normalized but still human-readable format.  Use
+`kbd-key:key-series-to-events' to add the key series to Emacs'
 keyboad input queue, as if they had been typed by the user."
   (interactive "kKeyboard key sequence to normalize (no {}): ")
   ;;
@@ -286,34 +315,34 @@ keyboad input queue, as if they had been typed by the user."
 		   norm-key-series (kbd-key:mark-spaces-to-keep norm-key-series "\\[" "\\]")
 		   norm-key-series (kbd-key:mark-spaces-to-keep norm-key-series "<" ">")
 		   norm-key-series (kbd-key:mark-spaces-to-keep norm-key-series "\"" "\"")
-		   norm-key-series (hypb:replace-match-string
-				    "<DEL>\\|<DELETE>\\|@key{DEL}\\|\\<DEL\\>" norm-key-series " DEL " t)
-		   norm-key-series (hypb:replace-match-string
-				    "<BS>\\|<BACKSPACE>\\|@key{BS}\\|\\<BS\\>" norm-key-series " BS " t)
-		   norm-key-series (hypb:replace-match-string
+		   norm-key-series (replace-regexp-in-string
+				    "<DEL>\\|<DELETE>\\|@key{DEL}\\|\\<DEL\\>" " DEL " norm-key-series nil t)
+		   norm-key-series (replace-regexp-in-string
+				    "<BS>\\|<BACKSPACE>\\|@key{BS}\\|\\<BS\\>" " BS " norm-key-series nil t)
+		   norm-key-series (replace-regexp-in-string
 				    "<RET>\\|<RTN>\\|<RETURN>\\|@key{RET}\\|@key{RTN}\\|\\<RETURN\\>\\|\\<RET\\>\\|\\<RTN\\>"
-				    norm-key-series " RET " t)
-		   norm-key-series (hypb:replace-match-string
-				    "<TAB>\\|@key{TAB}\\|\\<TAB\\>" norm-key-series " TAB " t)
+				    " RET " norm-key-series nil t)
+		   norm-key-series (replace-regexp-in-string
+				    "<TAB>\\|@key{TAB}\\|\\<TAB\\>" " TAB " norm-key-series nil t)
 		   ;; Includes conversion of spaces-to-keep markup to
 		   ;; SPC; otherwise, later calls to `kbd' will remove
 		   ;; these spaces.
-		   norm-key-series (hypb:replace-match-string
-				    "\\\\ \\|\0\0\0\\|<SPC>\\|@key{SPC}\\|\\<SPC\\>" norm-key-series " SPC " t)
-		   norm-key-series (hypb:replace-match-string
-				    "<ESC>\\|<ESCAPE>\\|@key{ESC}\\|\\<ESC\\(APE\\)?\\>" norm-key-series " M-" t)
+		   norm-key-series (replace-regexp-in-string
+				    "\\\\ \\|\0\0\0\\|<SPC>\\|@key{SPC}\\|\\<SPC\\>" " SPC " norm-key-series nil t)
+		   norm-key-series (replace-regexp-in-string
+				    "<ESC>\\|<ESCAPE>\\|@key{ESC}\\|\\<ESC\\(APE\\)?\\>" " M-" norm-key-series nil t)
 		   ;; ESC ESC
-		   norm-key-series (hypb:replace-match-string
-				    "M-\\s-*M-" norm-key-series " ESC M-" t)
+		   norm-key-series (replace-regexp-in-string
+				    "M-[ \t\n\r\f]*M-" " ESC M-" norm-key-series nil t)
 		   ;; Separate with a space any keys with a modifier
-		   norm-key-series (hypb:replace-match-string kbd-key:modified-key-regexp
-							      norm-key-series " \\1\\3 ")
+		   norm-key-series (replace-regexp-in-string kbd-key:modified-key-regexp
+							      " \\1\\3 " norm-key-series)
 		   ;; Normalize regular whitespace to single spaces
-		   norm-key-series (hypb:replace-match-string "[ \t\n\r\f]+" norm-key-series " " t)
+		   norm-key-series (replace-regexp-in-string "[ \t\n\r\f]+" " " norm-key-series nil t)
 
 		   ;; Unqote special {} chars.
-		   norm-key-series (hypb:replace-match-string "\\\\\\([{}]\\)"
-							      norm-key-series "\\1")
+		   norm-key-series (replace-regexp-in-string "\\\\\\([{}]\\)"
+							      "\\1" norm-key-series)
 		   norm-key-series (hpath:trim norm-key-series))
 	     ;; (while (string-match "\\`\\(C-u\\|M-\\)\\(-?[0-9]+\\)" norm-key-series)
 	     ;;   (setq arg (string-to-number (match-string 2 norm-key-series))
@@ -325,7 +354,8 @@ keyboad input queue, as if they had been typed by the user."
 	(t (error "(kbd-key:normalize): requires a string argument, not `%s'" key-series))))
 
 (defun kbd-key:remove-delimiters (str start-delim end-delim)
-  "Return STR sans START-DELIM and END-DELIM (strings) iff it starts and ends with these delimiters."
+  "Return STR sans START-DELIM and END-DELIM (strings).
+Return nil if STR does not start and end with the given delimiters."
   (when (and (string-match (format "\\`%s" (regexp-quote start-delim)) str)
 	     (string-match (format "%s\\'"  (regexp-quote end-delim)) str))
     (string-trim str start-delim end-delim)))
@@ -350,6 +380,7 @@ For an approximate inverse of this, see `key-description'."
 ;; try to parse <event> strings nor does it have optional second
 ;; parameter, need-vector.
 (defun kbd-key:parse (string)
+  "Parse STRING for keys."
   (let ((case-fold-search nil)
 	(len (length string)) ; We won't alter string in the loop below.
 	(pos 0)
@@ -445,13 +476,14 @@ For an approximate inverse of this, see `key-description'."
       res)))
 
 (defun kbd-key:extended-command-p (key-series)
-  "Return non-nil if the string KEY-SERIES is a normalized extended command invocation, i.e. M-x command."
+  "Return non-nil if the KEY-SERIES is a normalized extended command invocation.
+That is, `M-x command'."
   (when (stringp key-series)
     (string-match kbd-key:extended-command-prefix key-series)))
 
 (defun kbd-key:hyperbole-hycontrol-key-p (key-series)
-  "Return t if normalized, non-nil KEY-SERIES is given when in a HyControl mode, else nil.
-Allows for multiple key sequences strung together."
+  "Return t if KEY-SERIES is normalized, non-nil and in HyControl mode, else nil.
+Allow for multiple key sequences strung together."
   (and key-series
        (featurep 'hycontrol)
        (or hycontrol-windows-mode hycontrol-frames-mode)
@@ -461,8 +493,9 @@ Allows for multiple key sequences strung together."
        t))
 
 (defun kbd-key:hyperbole-mini-menu-key-p (key-series)
-  "Return t if normalized KEY-SERIES appears to invoke a Hyperbole menu item or sequence of keys, else nil.
-Also, initialize `kbd-key:mini-menu-key' to the key sequence that invokes the Hyperbole minibuffer menu."
+  "Return non-nil if KEY-SERIES invoke a Hyperbole menu item or sequence of keys.
+KEY-SERIES is normalized.  Also, initialize `kbd-key:mini-menu-key' to the
+key sequence that invokes the Hyperbole minibuffer menu."
   (when (stringp key-series)
     (unless (and (stringp kbd-key:mini-menu-key) (not (string-empty-p kbd-key:mini-menu-key)))
       (setq kbd-key:mini-menu-key (regexp-quote (kbd-key:normalize (key-description (car (where-is-internal 'hyperbole)))))))
@@ -470,7 +503,8 @@ Also, initialize `kbd-key:mini-menu-key' to the key sequence that invokes the Hy
       t)))
 
 (defun kbd-key:key-and-arguments (key-series)
-  "Return t if normalized KEY-SERIES appears to be a bound key sequence possibly with following interactive arguments, else nil."
+  "Return t if normalized KEY-SERIES appears to be a bound key sequence, else nil.
+KEY-SERIES can have following interactive arguments."
   (let ((prefix-binding (and (stringp key-series) (kbd-key:binding (substring key-series 0 (seq-position key-series ?\ ))))))
     ;; Just ensure that 1st character is bound to something that is
     ;; not a self-insert-command or a number.
@@ -480,8 +514,8 @@ Also, initialize `kbd-key:mini-menu-key' to the key sequence that invokes the Hy
 	 t)))
 
 (defun kbd-key:mark-spaces-to-keep (string start-delim end-delim)
-  "Return STRING with all spaces between any START-DELIM string and END-DELIM string marked for non-replacement."
-  (let ((regexp (format "\\(%s\\S-*\\)\\s-\\(.*%s\\)"
+  "Return STRING with all spaces between START-DELIM and END-DELIM marked to keep."
+  (let ((regexp (format "\\(%s[^ \t\n\r\f]*\\)[ \t\n\r\f]\\(.*%s\\)"
 			start-delim end-delim))
 	(start 0)
 	(end)
@@ -491,7 +525,7 @@ Also, initialize `kbd-key:mini-menu-key' to the key sequence that invokes the Hy
 	    end (match-end 0)
 	    substring (match-string 0 string)
 	    string (concat (substring string 0 start)
-			   (hypb:replace-match-string "\\s-" substring "\0\0\0" t)
+			   (replace-regexp-in-string "[ \t\n\r\f]" "\0\0\0" substring nil t)
 			   (if (< end (length string))
 			       (substring string end)
 			     ""))
@@ -502,30 +536,16 @@ Also, initialize `kbd-key:mini-menu-key' to the key sequence that invokes the Hy
   "Return non-nil if normalized KEY-SERIES string is one of the following:
 a Hyperbole minibuffer menu item key sequence,
 a HyControl key sequence,
-a M-x extended command,
+a \\`M-x' extended command,
   or a valid key sequence together with its interactive arguments."
   (or (kbd-key:hyperbole-mini-menu-key-p key-series)
       (kbd-key:hyperbole-hycontrol-key-p key-series)
       (kbd-key:extended-command-p key-series)
       (kbd-key:key-and-arguments key-series)))
 
-;;; ************************************************************************
-;;; Private variables
-;;; ************************************************************************
-
-(defconst kbd-key:extended-command-prefix
-  (format "\\_<%s\\_>" (kbd-key:normalize "M-x"))
-  "Normalized prefix regular expression that invokes an extended command; by default, M-x.")
-
-(defconst kbd-key:extended-command-binding-list '(execute-extended-command helm-M-x counsel-M-x)
-  "List of commands that may be bound to M-x to invoke extended/named commands.")
-
-(defvar kbd-key:mini-menu-key nil
-  "The key sequence that invokes the Hyperbole minibuffer menu.")
-;; Set above variable
+;; Set `kbd-key:mini-menu-key'
 (kbd-key:hyperbole-mini-menu-key-p "")
 
-
 (provide 'hib-kbd)
 
 ;;; hib-kbd.el ends here
