@@ -3,9 +3,11 @@
 ;; Author:       Markus Freericks <Mfx@cs.tu-berlin.de> / Bob Weiner
 ;;
 ;; Orig-Date:     1-Aug-91
-;; Last-Mod:     24-Jan-22 at 00:18:46 by Bob Weiner
+;; Last-Mod:     17-Jan-24 at 23:57:46 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -46,12 +48,23 @@
 ;; C-c C-c gets you to the occurence
 
 ;;; Code:
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function hpath:display-buffer "hpath")
+
 ;;; ************************************************************************
 ;;; Public variables
 ;;; ************************************************************************
 
 (defconst moccur-source-prefix "@loc> "
-  "Prefix for lines indicating source of matches.")
+  "String prefix for lines indicating the source of the matches.
+
+It should not contain any characters needing regular expression quoting.
+
+This expression should be followed immediately by a buffer or file name
+indicating the source of any of its Hyperbole buttons.")
 
 ;;; ************************************************************************
 ;;; Public functions
@@ -63,6 +76,7 @@ With optional FILE-REGEXP (a pattern which matches to files in a
 single directory), search matching files rather than current
 buffers.  The lines are shown in a buffer named *Moccur* which
 serves as a menu to find any of the occurrences in this buffer.
+With optional NO-FOLD-SEARCH non-nil do case sensitive search.
 
 \\{moccur-mode-map}"
   (interactive "sRegexp to find occurrences of: \nsFiles to search (default current file buffers): ")
@@ -84,9 +98,7 @@ serves as a menu to find any of the occurrences in this buffer.
     (insert "Lines matching '" regexp "':\n\n")
     (let ((currbuf) (currfile) (kill-buf)
 	  ;; Disable syntax highlighting of new buffers created by this command.
-	  (font-lock-auto-fontify) ;; For XEmacs and InfoDock
-	  (font-lock-global-modes) ;; For GNU Emacs
-	  )
+	  (font-lock-global-modes))
       (while buffers
 	(setq currbuf (car buffers)
 	      currfile (if (stringp currbuf) currbuf)
@@ -183,8 +195,10 @@ serves as a menu to find any of the occurrences in this buffer.
   (setq mode-name "Moccur"))
 
 (defun moccur-noselect ()
-  "Return (destination-buffer line-number occur-match-text) for the current moccur buffer line.
-Signal an error if not on a valid occurrence line."
+  "Return match data for the current moccur buffer line.
+Match data is returned as a list (destination-buffer line-number
+occur-match-text).  Signal an error if not on a valid occurrence
+line."
   (if (not (eq major-mode 'moccur-mode))
       (error "'moccur-to' must be called within a moccur buffer")
     (let (beg file-path lineno dstbuf occur-match)
