@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    30-May-94 at 00:11:57
-;; Last-Mod:     24-Jan-22 at 00:18:46 by Bob Weiner
+;; Last-Mod:     20-Jan-24 at 20:16:44 by Mats Lidell
 ;;
-;; Copyright (C) 1994-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1994-2024  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -29,21 +31,46 @@
 
 (eval-when-compile (mapc #'require '(hsettings hmouse-drv hmouse-sh)))
 
+;;; ************************************************************************
+;;; Private declarations
+;;; ************************************************************************
+
+(defvar hyperbole-mode-map)             ; "hyperbole.el"
+
+(declare-function hkey-initialize "hyperbole")
+(declare-function hmouse-get-bindings "hmouse-sh")
+(declare-function hmouse-unshifted-setup "hmouse-sh")
+(declare-function hmouse-shifted-setup "hmouse-sh")
+(declare-function hkey-set-key "hyperbole")
+
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defvar hmouse-bindings nil
+  "List of (key . binding) pairs for Hyperbole mouse keys.")
+
+(defvar hmouse-bindings-flag nil
+  "Non-nil means Hyperbole mouse bindings are in use, else nil.")
+
+(defvar hmouse-previous-bindings nil
+  "List of prior (key . binding) pairs for mouse keys rebound by Hyperbole.")
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
 
 (defun hmouse-check-action-key ()
-  "After use of the Action Mouse Key, ensure both depress and release events are assigned to the key.
-Returns t iff the key is properly bound, else nil."
+  "After Action Mouse Key use, ensure both depress and release events are bound.
+Return t iff the key is properly bound, else nil."
   (and (or (and (eventp action-key-depress-args) (eventp action-key-release-args))
 	   (not (or action-key-depress-args action-key-release-args)))
        (where-is-internal 'action-key-depress-emacs hyperbole-mode-map t)
        (where-is-internal 'action-mouse-key-emacs hyperbole-mode-map t)))
 
 (defun hmouse-check-assist-key ()
-  "After use of the Assist Mouse Key, ensure both depress and release events are assigned to the key.
-Returns t iff the key is properly bound, else nil."
+  "After Assist Mouse Key use, ensure both depress and release events are bound.
+Return t iff the key is properly bound, else nil."
   (and (or (and (eventp assist-key-depress-args) (eventp assist-key-release-args))
 	   (not (or assist-key-depress-args assist-key-release-args)))
        (where-is-internal 'assist-key-depress-emacs hyperbole-mode-map t)
@@ -58,7 +85,7 @@ bindings after Smart Key setup."
     ;; Do nothing when running in batch mode.
     (noninteractive)
     ;;
-    ;; GNU Emacs, XEmacs or InfoDock
+    ;; GNU Emacs or InfoDock
     (t
      (mapcar
        (lambda (key-and-binding)
@@ -66,8 +93,10 @@ bindings after Smart Key setup."
        key-binding-list))))
 
 (defun hmouse-install (&optional arg)
-  "Bind the two rightmost shifted mouse keys to the Action and Assist Keys, initializing Hyperbole mouse buttons.
-With optional prefix ARG or under InfoDock, also binds the unshifted middle mouse key to the Action Key.
+  "Initialize Hyperbole mouse buttons.
+Bind the two rightmost shifted mouse keys to the Action and
+Assist Keys.  With optional prefix ARG or under InfoDock, also
+bind the unshifted middle mouse key to the Action Key.
 
 The standard Hyperbole configuration is Action Key = shift-middle mouse key;
 Assist Key = shift-right mouse key."
@@ -91,7 +120,7 @@ Assist Key = shift-right mouse key."
 		 "{Shift-Mouse-2} invokes"))))
 
 (defun hmouse-add-unshifted-smart-keys ()
-  "Bind mouse-2 to the Action Key and mouse-3 to the Assist Key."
+  "Bind \\`mouse-2' to the Action Key and \\`mouse-3' to the Assist Key."
   (interactive)
   (require 'hyperbole)
   (hmouse-unshifted-setup))
@@ -120,29 +149,20 @@ mouse key the Paste Key instead of the Action Key."
 
 ;; Define function to reload Smart Key bindings and actions after a source code change.
 (defun hmouse-update-smart-keys ()
-  "Reload the contexts and actions associated with the Smart Keys after any programmatic change is made."
+  "Reload the contexts and actions associated with the Smart Keys.
+Use after any programmatic change is made."
   (interactive)
-  (hkey-initialize)
-  (makunbound 'hkey-alist)
-  (makunbound 'hmouse-alist)
-  (let ((load-prefer-newer t))
-    ;; This also reloads "hui-window" where mouse-only actions are defined.
-    (mapc #'load '("hui-mouse" "hibtypes" "hactypes")))
-  (message "Hyperbole Smart Key and Smart Mouse Key actions have been updated."))
-
-
-;;; ************************************************************************
-;;; Private variables
-;;; ************************************************************************
-
-(defvar hmouse-bindings nil
-  "List of (key . binding) pairs for Hyperbole mouse keys.")
-
-(defvar hmouse-bindings-flag nil
-  "True if Hyperbole mouse bindings are in use, else nil.")
-
-(defvar hmouse-previous-bindings nil
-  "List of prior (key . binding) pairs for mouse keys rebound by Hyperbole.")
+  (let ((load-prefer-newer t)
+	(ftrs '(hui-mouse hui-window hibtypes hactypes)))
+    (load "hui-mini")
+    (hkey-initialize)
+    (makunbound 'hkey-alist)
+    (makunbound 'hmouse-alist)
+    (mapc (lambda (feature)
+	    (setq features (delq feature features)))
+	  ftrs)
+    (mapc #'require ftrs)
+    (message "Hyperbole Smart Keys and menus have been updated")))
 
 (provide 'hmouse-key)
 
