@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     9-May-91 at 04:22:02
-;; Last-Mod:     24-Jan-22 at 00:18:47 by Bob Weiner
+;; Last-Mod:     18-Jan-24 at 19:53:39 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -25,17 +27,15 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(hvar hmail hact rmail rmailsum rmailedit)))
+(eval-and-compile (mapc #'require '(hload-path hvar hmail hact rmail rmailsum rmailedit)))
 (load "hsmail")
 
-;;; ************************************************************************
-;;; Public variables
-;;; ************************************************************************
-
 ;;; ************************************************************************
 ;;; Public declarations
 ;;; ************************************************************************
+
 (defvar rmail-old-text)
+(defvar message-setup-hook)             ; "message.el"
 
 ;;; ************************************************************************
 ;;; Public functions
@@ -96,14 +96,16 @@ This includes Hyperbole button data."
 	(end (rmail-msgend rmail-current-message)))
     (narrow-to-region beg end)))
 
-(defun Rmail-msg-next ()        (rmail-next-undeleted-message 1))
+(defun Rmail-msg-next ()
+  "Go to next message."
+  (rmail-next-undeleted-message 1))
 
 (defun Rmail-msg-num ()
   "Return number of Rmail message that point is within."
   (interactive)
   (let ((count 0) opoint)
     (save-excursion
-     (while (and (not (eobp))
+     (while (and (not (smart-eobp))
 		 (progn (setq opoint (point))
 			(re-search-backward "^\^_" nil t)))
        (if (= opoint (point))
@@ -111,7 +113,9 @@ This includes Hyperbole button data."
 	 (setq count (1+ count)))))
     count))
 
-(defun Rmail-msg-prev ()        (rmail-previous-undeleted-message 1))
+(defun Rmail-msg-prev ()
+  "Go to previous message."
+  (rmail-previous-undeleted-message 1))
 
 (defun Rmail-msg-to-p (mail-msg-id mail-file)
   "Set current buffer to start of msg with MAIL-MSG-ID in MAIL-FILE.
@@ -167,15 +171,18 @@ Return t if successful, else nil."
 ;;; Overloaded functions
 ;;; ************************************************************************
 
-(if (featurep 'rmail-hyperbole)
+(if (featurep 'rmail-hyperbole)         ;FIXME: Nowhere to be found.
     ;; No overloads are necessary, the needed features are built-in.
     nil
 
-;;; else
-;;;
-;;; Redefine version of this function from "rmailedit.el" to include any
-;;; hidden Hyperbole button data when computing message length.
-(defun rmail-cease-edit ()
+;; else
+;;
+;; Redefine version of this function from "rmailedit.el" to include any
+;; hidden Hyperbole button data when computing message length.
+;; FIXME: Copy&redefine like this is *evil*.  Use an advice or a hook.
+;; We can make changes to `rmail.el' if needed.
+(advice-add 'rmail-cease-edit :override 'hrmail--rmail-cease-edit)
+(defun hrmail--rmail-cease-edit ()
   "Finish editing message; switch back to Rmail proper."
   (interactive)
   ;; Make sure buffer ends with a newline.
@@ -251,11 +258,14 @@ Return t if successful, else nil."
   (setq buffer-read-only t))
 
 
-;;; Redefine version of this function from "rmail.el" to include any
-;;; Hyperbole button data.
-(defun rmail-forward (resend)
+;; Redefine version of this function from "rmail.el" to include any
+;; Hyperbole button data.
+;; FIXME: Copy&redefine like this is *evil*.  Use an advice or a hook.
+;; We can make changes to `rmail.el' if needed.
+(advice-add 'rmail-forward :override 'hrmail--rmail-forward)
+(defun hrmail--rmail-forward (resend)
   "Forward the current message to another user.
-With prefix argument, \"resend\" the message instead of forwarding it;
+With prefix argument, RESEND the message instead of forwarding it;
 see the documentation of `rmail-resend'."
   (interactive "P")
   (if resend
@@ -320,31 +330,29 @@ see the documentation of `rmail-resend'."
 		(insert-buffer-substring forward-buffer)
 		(hmail:msg-narrow))))))))
 
-;;; Redefine version of 'rmail-get-new-mail' from "rmail.el" to highlight
-;;; Hyperbole buttons when possible.
-;;;
-(if (boundp 'rmail-get-new-mail-post-hook)
-    (add-hook 'rmail-get-new-mail-post-hook
-	      (lambda ()
-		(if (fboundp 'hproperty:but-create)
-		    (progn (widen) (hproperty:but-create)
-			   (rmail-show-message)))))
-  (hypb:function-overload 'rmail-get-new-mail nil
-			  '(if (fboundp 'hproperty:but-create)
-			       (progn (widen) (hproperty:but-create)
-				      (rmail-show-message)))))
+;; Redefine version of 'rmail-get-new-mail' from "rmail.el" to highlight
+;; Hyperbole buttons when possible.
+;;
 
-;;; Redefine version of 'rmail-new-summary' from "rmailsum.el" to
-;;; highlight Hyperbole buttons when possible.
-;;;
-(if (boundp 'rmail-summary-create-post-hook)
-    (add-hook 'rmail-summary-create-post-hook
-	      (lambda ()
-		(if (fboundp 'hproperty:but-create)
-		    (hproperty:but-create))))
-  (hypb:function-overload 'rmail-new-summary nil
-			  '(if (fboundp 'hproperty:but-create)
-			       (hproperty:but-create))))
+(defun hrmail--show-msg-and-buttons (&rest _)
+  "Show message and buttons."
+  (if (fboundp 'hproperty:but-create)
+      (progn (widen) (hproperty:but-create)
+	     (rmail-show-message))))
+(if (boundp 'rmail-get-new-mail-post-hook) ;FIXME: Doesn't exist.  XEmacs?
+    (add-hook 'rmail-get-new-mail-post-hook 'hrmail--show-msg-and-buttons)
+  ;; FIXME: Why change `rmail-get-new-mail' rather than `rmail-show-message'?
+  (advice-add 'rmail-get-new-mail :after 'hrmail--show-msg-and-buttons))
+
+;; Redefine version of 'rmail-new-summary' from "rmailsum.el" to
+;; highlight Hyperbole buttons when possible.
+;;
+(defun hrmail--highlight-buttons (&rest _)
+  "Highlight buttons."
+  (if (fboundp 'hproperty:but-create) (hproperty:but-create)))
+(if (boundp 'rmail-summary-create-post-hook) ;FIXME: Doesn't exist.  XEmacs?
+    (add-hook 'rmail-summary-create-post-hook 'hrmail--highlight-buttons)
+  (advice-add 'rmail-new-summary :after 'hrmail--highlight-buttons))
 
 ;; end not InfoDock
 )
