@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    15-Apr-91 at 00:48:49
-;; Last-Mod:     24-Apr-22 at 13:14:18 by Bob Weiner
+;; Last-Mod:     20-Jan-24 at 15:49:24 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2024  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -24,6 +26,7 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
+(require 'cl-lib)
 (require 'hversion)
 (require 'hvar)
 (require 'browse-url)
@@ -31,7 +34,14 @@
 ;;; ************************************************************************
 ;;; Public declarations
 ;;; ************************************************************************
+
+(defvar helm-allow-mouse)
+(defvar htz:local)                      ; "htz.el"
+
 (declare-function hproperty:but-create "hui-em-but")
+(declare-function hproperty:but-flash "hui-em-but")
+(declare-function hyperbole-minibuffer-menu "hui-mini")
+(declare-function hyperbole-menubar-menu "hui-menu")
 
 ;;; Read the comments and modify as desired.
 
@@ -86,22 +96,22 @@
 ;; The Smart Menu system is an attractive in-buffer menu system that
 ;; predates Emacs menu systems; it is included in InfoDock.
 (defvar hkey-always-display-menu nil
-  "*Non-nil means always display the Smart Menu window when the Action or Assist Key is pressed and the Smart Menu system has been loaded.
-If a Smart Menu is already displayed, perform another Action or Assist Key
-function.")
+  "*Non-nil means a Smart Key press pops up the Smart Menu window.
+The Smart Menu system must have already been loaded.  If a Smart
+Menu is already displayed, perform another Action or Assist Key function.")
 
 (defcustom hmouse-middle-flag (and (boundp 'infodock-version) infodock-version t)
-  "*Under InfoDock or when t, additionally bind the middle mouse button as an
-Action Key."
+  "*Non-nil means additionally bind the middle mouse button as an Action Key."
   :type 'boolean
   :group 'hyperbole-keys)
 
 (defcustom smart-scroll-proportional t
-  "*Non-nil means Smart Keys should scroll relative to current line when pressed at the end of a line.
-Action Key moves current line to top of window.  Assist Key moves current
-line to bottom of window.  Repeated presses then scroll up or down a
-windowful.  Nil value instead ignores current line and always scrolls up or
-down a windowful."
+  "*Non-nil means Smart Key scroll behavior is relative to current line.
+Smart Keys will scroll relative to current line when pressed at
+the end of a line.  Action Key moves current line to top of the
+window.  Assist Key moves current line to bottom of the window.
+Repeated presses then scroll up or down a windowful.  Nil value
+ignores current line and always scrolls up or down a windowful."
   :type 'boolean
   :group 'hyperbole-keys)
 
@@ -128,13 +138,14 @@ down a windowful."
 
 (defun hyperbole-update-menus ()
   "Rebuild all Hyperbole menus with any updated settings."
+  (interactive)
+  (load "hui-menu")
+  (load "hui-mini")
   (hyperbole-menubar-menu)
   (hyperbole-minibuffer-menu))
 
 (defcustom hyperbole-default-web-search-term-max-lines 2
-  "Provide a default search term using the selected text if the
-active region contains less than or equal to this number of
-lines"
+  "Provide a default search term from max this number of lines from selected text."
   :type 'integer
   :group 'hyperbole-commands)
 
@@ -146,22 +157,34 @@ lines"
        (buffer-substring-no-properties (region-beginning) (region-end))))
 
 (defun hyperbole-read-web-search-arguments (&optional service-name search-term)
-  "Read from the keyboard a list of (web-search-service-string search-term-string) if not given as arguments."
-  (let ((completion-ignore-case t))
+  "Read from the keyboard a list of (web-search-service-string search-term-string).
+With optional non-empty SERVICE-NAME and SEARCH-TERM arguments,
+use those instead of reading from the keyboard."
+  (let ((completion-ignore-case t)
+	cmd-or-url)
     (while (or (not (stringp service-name)) (equal service-name ""))
       (setq service-name (completing-read "Search service: " hyperbole-web-search-alist
 					  nil t)))
-    (while (or (not (stringp search-term)) (equal search-term ""))
-      (setq search-term (read-string (format "Search %s for: " service-name)
-				     (hyperbole-default-web-search-term))))
+    (setq cmd-or-url (cdr (assoc service-name hyperbole-web-search-alist)))
+    (unless (functionp cmd-or-url)
+      (while (or (not (stringp search-term)) (equal search-term ""))
+	(setq search-term (read-string (format "Search %s for: " service-name)
+				       (hyperbole-default-web-search-term)))))
     (list service-name search-term)))
 
-(defun hyperbole-web-search (&optional service-name search-term)
-  "Search web SERVICE-NAME for SEARCH-TERM.
-Both arguments are optional and are prompted for when not given or when null.
-Uses `hyperbole-web-search-alist' to match each service to its search url.
-Uses `hyperbole-web-search-browser-function' and the `browse-url'
-package to display search results."
+(defun hyperbole-web-search (&optional service-name search-term return-search-expr-flag)
+  "Search web SERVICE-NAME for SEARCH-TERM (both arguments are optional).
+With optional RETURN-SEARCH-EXPR-FLAG, return the search expression.
+
+Use `hyperbole-web-search-alist' to match each service to its search
+url or function.
+Use `hyperbole-web-search-browser-function' and the `browse-url'
+package to display search results.
+
+If SERVICE-NAME is not given or is null and it is associated with
+a function rather than a search url in `hyperbole-web-search-alist',
+don't prompt for SEARCH-TERM; the function will prompt for that when
+run."
   (interactive)
   (cl-multiple-value-bind (service-name search-term)
       (hyperbole-read-web-search-arguments service-name search-term)
@@ -169,12 +192,19 @@ package to display search results."
 	  (search-pat (cdr (assoc service-name hyperbole-web-search-alist
 				  (lambda (service1 service2)
 				    (equal (downcase service1) (downcase service2)))))))
-      (setq search-term (browse-url-url-encode-chars search-term "[*\"()',=;?% ]"))
-      (cond ((stringp search-pat)
-	     (browse-url (format search-pat search-term)))
-	    ((functionp search-pat)
-	     (funcall search-pat search-term))
-	    (t (user-error "(Hyperbole): Invalid web search service `%s'" service-name))))))
+      (unless (null search-term)
+	(setq search-term (browse-url-url-encode-chars search-term "[*\"()',=;?% ]")))
+      (if return-search-expr-flag
+	  (cond ((stringp search-pat)
+		 (format search-pat search-term))
+		((functionp search-pat)
+		 (list search-pat))
+		(t (user-error "(Hyperbole): Invalid web search service `%s'" service-name)))
+	(cond ((stringp search-pat)
+	       (browse-url (format search-pat search-term)))
+	      ((functionp search-pat)
+	       (funcall search-pat))
+	      (t (user-error "(Hyperbole): Invalid web search service `%s'" service-name)))))))
 
 ;; This must be defined before the defcustom `inhbit-hyperbole-messaging'.
 ;;;###autoload
@@ -230,6 +260,7 @@ Hyperbole, and then restart Emacs."
     ("Bing" . "http://www.bing.com/search?q=%s")
     ;; Wikipedia Dictionary
     ("Dictionary" . "https://en.wiktionary.org/wiki/%s")
+    ("ducKduckgo" . "https://duckduckgo.com/?q=%s")
     ("Elisp" . "http://www.google.com/search?q=%s+filetype:el")
     ;; Facebook Hashtags
     ("Facebook" . "https://www.facebook.com/hashtag/%s")
@@ -259,11 +290,12 @@ then runs the search."
   :group 'hyperbole-commands)
 
 ;;; ************************************************************************
-;;; GNU EMACS AND XEMACS CONFIGURATION
+;;; GNU EMACS CONFIGURATION
 ;;; ************************************************************************
 
 ;; No-op unless set by one of the conditionals below.
-(defun hui:but-flash ())
+(defun hui:but-flash ()
+  "Button flash No-op.")
 
 (cond ((not noninteractive)
        (require 'hui-em-but)
