@@ -3,7 +3,9 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     9-May-91 at 04:50:20
-;; Last-Mod:     17-Apr-22 at 11:23:02 by Bob Weiner
+;; Last-Mod:     27-Aug-23 at 12:39:11 by Bob Weiner
+;;
+;; SPDX-License-Identifier: GPL-3.0-or-later
 ;;
 ;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
@@ -55,7 +57,7 @@ for a comment.")
   "Add a comment to the current outgoing message.
 Only if Hyperbole has been loaded and `inhibit-hyperbole-messaging' is nil.
 Optional COMMENT-FORM is evaluated to obtain the string to add to the
-message.  If not given, 'smail:comment' is evaluated by default."
+message.  If not given, `smail:comment' is evaluated by default."
   (if (and (featurep 'hyperbole) (not inhibit-hyperbole-messaging))
       (let ((comment (eval (or comment-form smail:comment) t)))
 	(if comment
@@ -69,7 +71,7 @@ message.  If not given, 'smail:comment' is evaluated by default."
 
 (defun smail:widen ()
   "Widens outgoing mail buffer to include Hyperbole button data."
-  (if (fboundp #'mail+narrow) (mail+narrow) (widen)))
+  (if (fboundp 'mail+narrow) (mail+narrow) (widen)))
 
 ;; Redefine this function from "message.el" to include Hyperbole button
 ;; data when yanking in a message and to highlight buttons if possible.
@@ -77,7 +79,7 @@ message.  If not given, 'smail:comment' is evaluated by default."
   "See `message-yank-original'.
 
 If supercite is in use, header fields are never deleted.
-Use (setq sc-nuke-mail-headers 'all) to have them removed."
+Use (setq sc-nuke-mail-headers \\='all) to have them removed."
   (let ((modified (buffer-modified-p))
 	body-text)
 	(when (and message-reply-buffer
@@ -141,7 +143,7 @@ Use (setq sc-nuke-mail-headers 'all) to have them removed."
 Put point before the text and mark after.
 
 Normally indent each nonblank line ARG spaces (default 3).
-However, if ‘mail-yank-prefix’ is non-nil, insert that prefix
+However, if `mail-yank-prefix’ is non-nil, insert that prefix
 on each line when `mail-indent-citation' is called.
 
 Apply `mail-citation-hook', `mail-yank-hook' or `mail-yank-hooks'
@@ -151,7 +153,7 @@ Just \\[universal-argument] as argument means don't indent,
 insert no prefix, and don't delete any header fields.
 
 If supercite is in use, header fields are never deleted.
-Use (setq sc-nuke-mail-headers 'all) to have them removed."
+Use (setq sc-nuke-mail-headers \\='all) to have them removed."
   (interactive "P")
   (if mail-reply-action
       (let ((start (point))
@@ -206,8 +208,8 @@ Use (setq sc-nuke-mail-headers 'all) to have them removed."
 		(goto-char start)
 		(let ((mail-indentation-spaces (if arg (prefix-numeric-value arg)
 						 mail-indentation-spaces))
-		      ;; Avoid error in Transient Mark mode
-		      ;; on account of mark's being inactive.
+		      ;; Avoid error in Transient Mark mode when the
+		      ;; mark is inactive.
 		      (mark-even-if-inactive t))
 		  (cond ((and (boundp 'mail-citation-hook) mail-citation-hook)
 			 ;; Bind mail-citation-header to the inserted message's header.
