@@ -3,7 +3,9 @@
 ;; Author:       Masanobu Umeda             / Bob Weiner
 ;;
 ;; Orig-Date:    14-Oct-91 at 07:22:08
-;; Last-Mod:     24-Jan-22 at 00:18:47 by Bob Weiner
+;; Last-Mod:     19-Jan-24 at 12:13:19 by Mats Lidell
+;;
+;; SPDX-License-Identifier: GPL-3.0-or-later
 ;;
 ;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
@@ -32,6 +34,69 @@
 (require 'calendar)
 (require 'cal-julian)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function hypb:call-process-p "hypb")
+
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defvar htz:local
+  (let ((local-tz (or (getenv "TZ") (getenv "TIMEZONE")
+		      (if (fboundp 'current-time-zone)
+			  (car (cdr (current-time-zone))))
+		      (progn
+			(require 'hypb)
+			(hypb:call-process-p
+			 "date" nil '(if (re-search-backward
+					  " \\([-+a-zA-Z0-9]+\\) [0-9]+$" nil t)
+					 (buffer-substring (match-beginning 1)
+							   (match-end 1))))))))
+    (if (and (stringp local-tz) (string-match " " local-tz))
+	;; Windows returns things like "Eastern Daylight Time", so
+	;; abbreviate to the first letter of each word.
+	(concat (mapcar (lambda (s) (aref s 0)) (split-string local-tz)))
+      local-tz))
+  "Holds string giving the timezone for the local machine.")
+
+(defvar htz:world-timezones
+  '(("PST" .  -800)
+    ("PDT" .  -700)
+    ("MST" .  -700)
+    ("MDT" .  -600)
+    ("CST" .  -600)
+    ("CDT" .  -500)
+    ("EST" .  -500)
+    ("EDT" .  -400)
+    ("AST" .  -400)
+    ("NST" .  -330)
+    ("UT"  .  +000)
+    ("GMT" .  +000)
+    ("BST" .  +100)
+    ("MET" .  +100)
+    ("EET" .  +200)
+    ("JST" .  +900)
+    ("GMT+1"  .  +100) ("GMT+2"  .  +200) ("GMT+3"  .  +300)
+    ("GMT+4"  .  +400) ("GMT+5"  .  +500) ("GMT+6"  .  +600)
+    ("GMT+7"  .  +700) ("GMT+8"  .  +800) ("GMT+9"  .  +900)
+    ("GMT+10" . +1000) ("GMT+11" . +1100) ("GMT+12" . +1200) ("GMT+13" . +1300)
+    ("GMT-1"  .  -100) ("GMT-2"  .  -200) ("GMT-3"  .  -300)
+    ("GMT-4"  .  -400) ("GMT-5"  .  -500) ("GMT-6"  .  -600)
+    ("GMT-7"  .  -700) ("GMT-8"  .  -800) ("GMT-9"  .  -900)
+    ("GMT-10" . -1000) ("GMT-11" . -1100) ("GMT-12" . -1200))
+  "Time differentials of timezone from GMT in +-HHMM form.
+Use `current-time-zone' instead where possible.")
+
+(defvar htz:months-assoc
+  '(("JAN" .  1)("FEB" .  2)("MAR" .  3)
+    ("APR" .  4)("MAY" .  5)("JUN" .  6)
+    ("JUL" .  7)("AUG" .  8)("SEP" .  9)
+    ("OCT" . 10)("NOV" . 11)("DEC" . 12))
+  "Alist of first three letters of a month and its numerical representation.")
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
@@ -62,11 +127,12 @@ Optional 2nd argument TIMEZONE specifies a timezone to be represented in."
 
 (defun htz:date-parse (date &optional parsed-current-date)
   "Parse DATE string and return a vector [year month day time timezone].
-If a two-digit year, the first two digits of the current year are prepended.
-Timezone in DATE is optional, it defaults to the value of `htz:local'.
+If a two-digit year, the first two digits of the current year are
+prepended.  Timezone in DATE is optional, it defaults to the
+value of `htz:local'.
 
 Recognizes the following styles:
- (1) \"(1 30 1999)\" or \"(1 30 1999)\"  `calendar-julian-date'  requires PARSED-CURRENT-DATE arg
+ (1) \"(1 30 1999)\" `calendar-julian-date' requires PARSED-CURRENT-DATE arg
  (2) \"14 Apr 89 03:20[:12] [GMT]\"
  (3) \"Fri, 17 Mar [19]89 4:01[:33] [GMT]\"
  (4) \"Mon Jan 16 16:12[:37] [GMT] 1989\"
@@ -184,7 +250,7 @@ Optional 2nd argument TIMEZONE specifies an output timezone to use."
 ;;;
 
 (defun htz:date-sortable-gmt (&optional date local)
-  "Convert optional DATE or current date  to a sortable date string in Greenwich Mean Time.
+  "Convert optional DATE or current date to a sortable date string in GMT.
 Optional argument LOCAL specifies the local timezone of the DATE."
   (htz:date-sortable date local "GMT"))
 
@@ -297,7 +363,7 @@ Optional argument TIMEZONE specifies a time zone."
       (zerop (% year 400))))
 
 (defun htz:time-fix (year month day hour minute second)
-  "Fix date and time."
+  "Fix date and time given by YEAR, MONTH, DAY, HOUR, MINUTE and SECOND."
   (cond ((<= 24 hour)			; 24 -> 00
 	 (setq hour (- hour 24))
 	 (setq day  (1+ day))
@@ -344,65 +410,6 @@ Optional argument TIMEZONE specifies a time zone."
 	(/ timezone 100))
     (error "(htz:zone-to-hour): Nil timezone sent as argument")))
 
-
-;;; ************************************************************************
-;;; Private variables
-;;; ************************************************************************
-
-(defvar htz:local
-  (let ((local-tz (or (getenv "TZ") (getenv "TIMEZONE")
-		      (if (fboundp 'current-time-zone)
-			  (car (cdr (current-time-zone))))
-		      (progn
-			(require 'hypb)
-			(hypb:call-process-p
-			 "date" nil '(if (re-search-backward
-					  " \\([-+a-zA-Z0-9]+\\) [0-9]+$" nil t)
-					 (buffer-substring (match-beginning 1)
-							   (match-end 1))))))))
-    (if (and (stringp local-tz) (string-match " " local-tz))
-	;; Windows returns things like "Eastern Daylight Time", so
-	;; abbreviate to the first letter of each word.
-	(concat (mapcar (lambda (s) (aref s 0)) (split-string local-tz)))
-      local-tz))
-  "Holds string giving the timezone for the local machine.")
-
-(defvar htz:world-timezones
-  '(("PST" .  -800)
-    ("PDT" .  -700)
-    ("MST" .  -700)
-    ("MDT" .  -600)
-    ("CST" .  -600)
-    ("CDT" .  -500)
-    ("EST" .  -500)
-    ("EDT" .  -400)
-    ("AST" .  -400)
-    ("NST" .  -330)
-    ("UT"  .  +000)
-    ("GMT" .  +000)
-    ("BST" .  +100)
-    ("MET" .  +100)
-    ("EET" .  +200)
-    ("JST" .  +900)
-    ("GMT+1"  .  +100) ("GMT+2"  .  +200) ("GMT+3"  .  +300)
-    ("GMT+4"  .  +400) ("GMT+5"  .  +500) ("GMT+6"  .  +600)
-    ("GMT+7"  .  +700) ("GMT+8"  .  +800) ("GMT+9"  .  +900)
-    ("GMT+10" . +1000) ("GMT+11" . +1100) ("GMT+12" . +1200) ("GMT+13" . +1300)
-    ("GMT-1"  .  -100) ("GMT-2"  .  -200) ("GMT-3"  .  -300)
-    ("GMT-4"  .  -400) ("GMT-5"  .  -500) ("GMT-6"  .  -600)
-    ("GMT-7"  .  -700) ("GMT-8"  .  -800) ("GMT-9"  .  -900)
-    ("GMT-10" . -1000) ("GMT-11" . -1100) ("GMT-12" . -1200))
-  "Time differentials of timezone from GMT in +-HHMM form.
-Use `current-time-zone' instead where possible.")
-
-(defvar htz:months-assoc
-  '(("JAN" .  1)("FEB" .  2)("MAR" .  3)
-    ("APR" .  4)("MAY" .  5)("JUN" .  6)
-    ("JUL" .  7)("AUG" .  8)("SEP" .  9)
-    ("OCT" . 10)("NOV" . 11)("DEC" . 12))
-  "Alist of first three letters of a month and its numerical representation.")
-
-
 (provide 'htz)
 
 ;;; htz.el ends here
