@@ -3,7 +3,9 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    28-Oct-94 at 10:59:44
-;; Last-Mod:     26-Feb-22 at 17:03:44 by Bob Weiner
+;; Last-Mod:      2-Feb-24 at 21:41:16 by Mats Lidell
+;;
+;; SPDX-License-Identifier: GPL-3.0-or-later
 ;;
 ;; Copyright (C) 1994-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
@@ -19,12 +21,32 @@
 
 (eval-and-compile (mapc #'require '(hpath hui-jmenu hyrolo-menu browse-url easymenu)))
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function gbut:label-list "hbut")
+(declare-function ebut:list "hbut")
+
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defvar hui-menu-max-list-length 24
+  "Positive integer that caps the length of a Hyperbole dynamic menu lists.")
+
+(defvar hui-menu-order-explicit-buttons t
+  "When non-nil (default), explicit button menu list is lexicographically ordered.
+Otherwise, explicit buttons are listed in their order of appearance within
+the current buffer.")
 
 ;;; ************************************************************************
 ;;; Private functions
 ;;; ************************************************************************
 
 (defmacro hui-menu-browser (title browser-option)
+  "Browser menu with a TITLE.
+BROWSER-OPTION marks current active menu option as selected."
   `(list
     (list ,title
 	  ["Chrome (Google)"
@@ -68,6 +90,7 @@
 
 ;; List explicit buttons in the current buffer for menu activation.
 (defun hui-menu-explicit-buttons (rest-of-menu)
+  "Explicit button menu to go before REST-OF-MENU."
   (delq nil
 	(append
 	 '(["Manual"   (id-info "(hyperbole)Explicit Buttons") t]
@@ -87,7 +110,7 @@
 				 (not hui-menu-order-explicit-buttons))
 			   :style toggle :selected hui-menu-order-explicit-buttons]
 			  "Activate:")
-			(mapcar (lambda (label) (vector label `(ebut:act ,label) t))
+			(mapcar (lambda (label) (vector label `(ebut:act-label ,label) t))
 				(if hui-menu-order-explicit-buttons
 				    (sort labels #'string-lessp)
 				  labels))
@@ -105,6 +128,7 @@ Return t if cutoff, else nil."
 
 ;; List existing global buttons for menu activation.
 (defun hui-menu-global-buttons (rest-of-menu)
+  "Global button menu to go before REST-OF-MENU."
   (delq nil
 	(append
 	 '(["Manual" (id-info "(hyperbole)Global Buttons") t]
@@ -128,6 +152,7 @@ Return t if cutoff, else nil."
 		      (format "(%s)" (key-description (where-is-internal command nil t))))))
 
 (defun hui-menu-key-bindings (rest-of-menu)
+  "Key binding menu to go before REST-OF-MENU."
   (nconc
    (list
     (vector (hui-menu-key-binding-item "Action-Key          \t\t\t" 'hkey-either)                        '(hui:bind-key #'hkey-either) t)                    ;; {M-RET}
@@ -144,6 +169,7 @@ Return t if cutoff, else nil."
 
 ;; Dynamically compute submenus for Screen menu
 (defun hui-menu-screen (_ignored)
+  "Screen menu."
   (list
    ["Manual" (id-info "(hyperbole)HyControl") t]
    "----"
@@ -155,7 +181,7 @@ Return t if cutoff, else nil."
    (hui-menu-of-windows)))
 
 (defun hui-menu-web-search ()
-  ;; Pulldown menu
+  "Web search pulldown menu."
   (let* (service
 	 action)
     (mapcar (lambda (service-and-action)
@@ -180,7 +206,7 @@ Return t if cutoff, else nil."
 		  (if (= (aref hyperb:version 0) ?0)
 		      (substring hyperb:version 1)
 		    hyperb:version))
-	  #'(hypb:display-file-with-logo "HY-ABOUT")
+	  '(hypb:display-file-with-logo "HY-ABOUT")
 	  t))
 
 (defconst hui-menu-options
@@ -302,7 +328,8 @@ Return t if cutoff, else nil."
   (force-mode-line-update))
 
 (defun hyperbole-popup-menu (&optional rebuild-flag)
-  "Popup the Hyperbole menubar menu."
+  "Popup the Hyperbole menubar menu.
+With optional REBUILD-FLAG rebuild the menu."
   (interactive "P")
   (popup-menu (infodock-hyperbole-menu rebuild-flag)))
 
@@ -340,7 +367,7 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		  t]
 		 "----"
 		 ["Activate-Button-in-Buffer" hui:hbut-act t]
-		 ["Back-to-Prior-Location" (hhist:remove current-prefix-arg)
+		 ["Back-to-Prior-Location" hhist:pop
 		  (and (boundp '*hhist*) *hhist*)]
 		 '("Button-File"
 		   ["Manual"  (id-info "(hyperbole)Button Files") t]
@@ -353,9 +380,13 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		 '("Documentation"
 		   ["Manual"      (id-info "(hyperbole)Top") t]
 		   "----"
+		   ["About"       (hypb:display-file-with-logo "HY-ABOUT") t]
+		   ["Concepts"    (find-file (expand-file-name
+					      "HY-CONCEPTS.kotl" hyperb:dir)) t]
 		   ["Demonstration"  hyperbole-demo t]
+		   ["Files"       (hypb:display-file-with-logo "MANIFEST") t]
 		   ["Glossary"    (id-info "(hyperbole)Glossary") t]
-		   ["Manifest"    (hypb:display-file-with-logo "MANIFEST") t]
+		   ["New-Features" (hypb:display-file-with-logo "HY-NEWS") t]
 		   ["Smart-Key-Summary" (id-browse-file (hypb:hkey-help-file)) t]
 		   ("Types"
 		    ["Action-Types-Manual"
@@ -366,7 +397,8 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		    ["Show-Action-Types"
 		     (hui:htype-help-current-window 'actypes) t]
 		    ["Show-Implicit-Button-Types"
-		     (hui:htype-help-current-window 'ibtypes 'no-sort) t]))
+		     (hui:htype-help-current-window 'ibtypes 'no-sort) t])
+		   ["WhyUse"     (find-file (expand-file-name "HY-WHY.kotl" hyperb:dir)) t])
 		 '("Explicit-Button"
 		   :filter hui-menu-explicit-buttons
 		   ["Activate" hui:ebut-act t]
@@ -379,6 +411,7 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		    ["Buffer-Buttons"   (hui:hbut-report -1) t]
 		    ["Current-Button"   (hui:hbut-report)    t]
 		    ["Ordered-Buttons"  (hui:hbut-report 1)  t])
+		   ["Link"   hui:ebut-link-directly t]
 		   ["Rename" hui:ebut-rename t]
 		   ["Search" hui:ebut-search t]
 		   ["Types"
@@ -408,15 +441,18 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		   ["Delete" hui:gbut-delete t]
 		   ["Edit"   hui:gbut-edit t]
 		   ["Help"   gbut:help t]
+		   ["Link"   hui:gbut-link-directly t]
                    ["Rename" hui:gbut-rename t])
 		 '("Implicit-Button"
 		   ["Manual"   (id-info "(hyperbole)Implicit Buttons") t]
 		   "----"
 		   ["Activate" hui:ibut-act t]
+		   ["Create"   hui:ibut-create t]
 		   ["Delete-Type" (hui:htype-delete 'ibtypes) t]
 		   ["Edit"   hui:ibut-edit t]
 		   ["Help"   hui:hbut-help t]
-		   ["Label"  hui:ibut-label-create t]
+		   ["Link"   hui:ibut-link-directly t]
+		   ["Name"   hui:ibut-label-create t]
 		   ["Rename" hui:ibut-rename t]
 		   ["Types"  (hui:htype-help 'ibtypes 'no-sort) t])
 		 '("Koutliner"
@@ -466,18 +502,6 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		 '("Screen (HyControl)" :filter hui-menu-screen)
 		 hui-menu-hywconfig)))))
 
-;;; ************************************************************************
-;;; Private variables
-;;; ************************************************************************
-
-(defvar hui-menu-max-list-length 24
-  "Positive integer that caps the length of a Hyperbole dynamic menu lists.")
-
-(defvar hui-menu-order-explicit-buttons t
-  "When non-nil (default), explicit button menu list is lexicographically ordered.
-Otherwise, explicit buttons are listed in their order of appearance within
-the current buffer.")
-
 (provide 'hui-menu)
 
 ;;; hui-menu.el ends here
