@@ -3,26 +3,35 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    19-Nov-17
-;; Last-Mod:     24-Jan-22 at 00:18:48 by Bob Weiner
+;; Last-Mod:     20-Jan-24 at 20:20:00 by Mats Lidell
 ;;
-;; Copyright (C) 2017-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 2017-2024  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
 
 ;;; Commentary:
+;; Ignore loading of this file unless the Treemacs package v2 or
+;; greater has been installed.
 
 ;;; Code:
 ;;; ************************************************************************
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (require 'treemacs nil t))
+(require 'package)
+(require 'seq)
+
+(or (require 'treemacs nil t)
+    (and (package-installed-p 'treemacs)
+	 (package-activate 'treemacs)))
 
 (defvar treemacs-version)
 
-(unless (and (featurep 'treemacs) (string-greaterp treemacs-version "v2"))
-  (error "(hui-treemacs): Hyperbole requires Treemacs package version 2.0 or greater"))
+(unless (string-greaterp treemacs-version "v2")
+  (error "(hui-treemacs): Hyperbole requires Treemacs package version 2.0 or greater, not %s" treemacs-version))
 
 ;;; ************************************************************************
 ;;; Public declarations
@@ -37,6 +46,16 @@
 (declare-function treemacs-toggle-node "ext:treemacs-interface")
 (defvar aw-ignored-buffers)
 
+(defvar assist-flag)                    ; "hmouse-drv.el"
+(defvar action-key-eol-function)        ; "hmouse-drv.el"
+(defvar assist-key-eol-function)        ; "hmouse-drv.el"
+(defvar action-key-depress-window)      ; "hmouse-drv.el"
+
+(declare-function first-line-p "hui-mouse")
+(declare-function last-line-p "hui-mouse")
+(declare-function hact "hact")
+(declare-function package-activate "package")
+
 ;;; ************************************************************************
 ;;; smart-treemacs functions
 ;;; ************************************************************************
@@ -67,7 +86,7 @@ If key is pressed:
      `assist-key-eol-function', typically to scroll down proportionally,
      if an Asisst Key press;
  (4) on the first line of the buffer (other than the end of line),
-     dired is run on the current directory of this Treemacs;
+     Dired is run on the current directory of this Treemacs;
  (5) at the end of the first or last line of the buffer,
      this Treemacs invocation is quit."
 
@@ -92,7 +111,7 @@ If key is pressed:
 
 ;;;###autoload
 (defun smart-treemacs-modeline ()
-  "Toggle display of Treemacs file viewer based on Smart Action Key click on a modeline.
+  "Toggle display of Treemacs from Smart Action Key click on a modeline.
 
 When pressed on the Treemacs buffer modeline or Treemacs is displaying
 the default directory of the buffer modeline clicked upon, then
@@ -122,3 +141,4 @@ Suitable for use as a value of `action-key-modeline-buffer-id-function'."
 
 (provide 'hui-treemacs)
 ;;; hui-treemacs.el ends here
+
