@@ -1,17 +1,20 @@
 ;;; hyperbole.el --- GNU Hyperbole: The Everyday Hypertextual Information Manager  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1992-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1992-2024  Free Software Foundation, Inc.
 
-;; Author:           Bob Weiner
-;; Maintainer:       Bob Weiner <rsw@gnu.org>, Mats Lidell <matsl@gnu.org>
-;; Created:          06-Oct-92 at 11:52:51
-;; Last-Mod:      1-May-22 at 10:31:04 by Bob Weiner
-;; Released:         03-May-21
-;; Version:          8.0.0
-;; Keywords:         comm, convenience, files, frames, hypermedia, languages, mail, matching, mouse, multimedia, outlines, tools, wp
-;; Package:          hyperbole
-;; Package-Requires: ((emacs "27.0"))
-;; URL:              http://www.gnu.org/software/hyperbole
+;; Authors:      Robert Weiner <rsw@gnu.org>
+;; Maintainer:   Mats Lidell <matsl@gnu.org>
+;; Maintainers:  Robert Weiner <rsw@gnu.org>, Mats Lidell <matsl@gnu.org>
+;; Created:      06-Oct-92 at 11:52:51
+;; Last-Mod:     10-Mar-24 at 12:50:16 by Bob Weiner
+;; Released:     10-Mar-24
+;; Version:      9.0.1
+;; Keywords:     comm, convenience, files, frames, hypermedia, languages, mail, matching, mouse, multimedia, outlines, tools, wp
+;; Package:      hyperbole
+;; Package-Requires: ((emacs "27.1"))
+;; URL:          http://www.gnu.org/software/hyperbole
 
 ;; See the "HY-COPY" file for license information.
 
@@ -61,7 +64,7 @@
 ;; 
 ;; ----
 ;;
-;; See the "INSTALL" file for installation instructions and the "README" file
+;; See the "INSTALL" file for installation instructions and the "README.md" file
 ;; for general information.
 ;;
 ;; There is no need to manually edit this file unless there are specific
@@ -86,9 +89,7 @@
   "Hyperbole customizations category."
   :group 'applications)
 
-(defgroup hyperbole-buttons nil
-  "Hyperbole explicit, global and implicit button customizations."
-  :group 'hyperbole)
+;; defgroup hyperbole-buttons is in "hib-social.el"
 
 (defgroup hyperbole-commands nil
   "Hyperbole command customizations."
@@ -98,7 +99,7 @@
   "Hyperbole keyboard and mouse key customizations."
   :group 'hyperbole)
 
-;; defgroup hyperbole-rolo is in "hyrolo.el".
+;; defgroup hyperbole-hyrolo is in "hyrolo.el".
 
 (defgroup hyperbole-screen nil
   "Hyperbole screen/display customizations, typically frame or window-related."
@@ -111,9 +112,8 @@
     (setq features (delq 'hload-path features)
 	  features (delq 'hversion features)))
 
-  ;; Defines hyperb:path-being-loaded, hyperb:stack-frame,
-  ;; (hyperb:window-system) and hyperb:dir, which are used later in
-  ;; this file.  Also adds Hyperbole to the load-path if need be.
+  ;; Defines hyperb:stack-frame, hyperb:window-system and hyperb:dir, which are used
+  ;; later in this file.  Also adds Hyperbole to the load-path if need be.
   ;;
   ;; This handles the case when the Hyperbole package directory is not yet in load-path.
   (unless (or (require 'hversion nil t)
@@ -133,13 +133,16 @@
 See `hkey-initialize'.")
 
 (defcustom hyperbole-mode-lighter " Hypb"
-  "Text to display in the minor-mode area of the modeline when the Hyperbole global minor mode is active."
+  "String to display in mode line when the Hyperbole global minor mode is enabled.
+Use nil for no Hyperbole mode indicator."
   :type 'string
   :group 'hyperbole)
 
 ;;;###autoload
 (define-minor-mode hyperbole-mode
-  "Toggle Hyperbole global minor mode, the Everyday Hypertextual Information Manager.
+  "Toggle Hyperbole global minor mode.
+
+Hyperbole is the Everyday Hypertextual Information Manager.
 
 When Hyperbole mode is enabled, the `hyperbole-mode' variable
 is non-nil, Hyperbole menus are enabled, as are Hyperbole keys.
@@ -149,9 +152,7 @@ Info documentation at \"(hyperbole)Top\".
 
 \\{hyperbole-mode-map}"
   :global t
-  :keymap 'hyperbole-mode-map
   :lighter hyperbole-mode-lighter
-  :require 'hyperbole
   (if hyperbole-mode
       (hyperbole--enable-mode)
     (hyperbole--disable-mode)))
@@ -159,10 +160,27 @@ Info documentation at \"(hyperbole)Top\".
 (defvar hyperbole--mark-even-if-inactive
   "Stores value of `mark-even-if-inactive' prior to enabling `hyperbole-mode'.")
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(defvar Info-directory-list)            ; "info.el"
+
+(declare-function vertico-mouse-mode "ext:vertico")
+
+(declare-function info-initialize "info")
+(declare-function hmouse-install "hmouse-key")
+(declare-function hui-search-web "hui-mini")
+(declare-function hkey-operate "hmouse-drv")
+(declare-function facemenu-keymap "facemenu")
+(declare-function hkey-help "hmouse-drv")
+(declare-function hkey-either "hmouse-drv")
+
 ;;; ************************************************************************
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
+(require 'hload-path)
 ;; Avoid any potential library name conflict by giving the load directory.
 (require 'set (expand-file-name "set" hyperb:dir))
 (require 'hypb)
@@ -173,10 +191,13 @@ Info documentation at \"(hyperbole)Top\".
 ;;; ************************************************************************
 
 (defcustom hkey-init t
-  "*A non-nil value (default) at system load time binds the Action and Assist Keyboard Keys, as well as other keys.
-{\\[hkey-either]} invokes the Action Key and {C-u \\[hkey-either]} invokes the Assist Key.
-Additionally, {\\[hkey-help]} shows what the Action Key will do in the current
-context (wherever point is).  {C-u \\[hkey-help]} shows what the Assist Key will do."
+  "*A non-nil value (default) at system load time binds Hyperbole keys.
+Keys bound are the Action and Assist Keyboard Keys, as well as
+other keys.  {\\[hkey-either]} invokes the Action Key and
+{\\`C-u' \\[hkey-either]} invokes the Assist Key.  Additionally,
+{\\[hkey-help]} shows what the Action Key will do in the current
+context (wherever point is).  {\\`C-u' \\[hkey-help]} shows what the
+Assist Key will do."
   :type 'boolean
   :group 'hyperbole-keys)
 
@@ -189,6 +210,7 @@ context (wherever point is).  {C-u \\[hkey-help]} shows what the Assist Key will
 `hkey-initialize' must have already been called or the list will be empty."
   hyperbole-mode-map)
 
+;; Use `hkey-set-key' instead.
 (make-obsolete 'hkey-global-set-key 'hkey-set-key "8.0.0")
 (defun hkey-global-set-key (key command &optional _no-add)
   "Define a Hyperbole KEY bound to COMMAND.  Optional third arg, NO-ADD is ignored."
@@ -196,12 +218,12 @@ context (wherever point is).  {C-u \\[hkey-help]} shows what the Assist Key will
 
 (make-obsolete 'hkey-maybe-global-set-key 'hkey-maybe-set-key "8.0.0")
 (defun hkey-maybe-global-set-key (key command &optional _no-add)
-  "Define a Hyperbole KEY bound to COMMAND if KEY is not bound in `hyperbole-mode-map'.
+  "Define a Hyperbole KEY bound to COMMAND if KEY is not in `hyperbole-mode-map'.
 Third argument NO-ADD is ignored."
   (hkey-maybe-set-key key command))
 
 (defun hkey-maybe-set-key (key command &optional _no-add)
-  "Define a Hyperbole KEY bound to COMMAND if KEY is not bound in `hyperbole-mode-map'.
+  "Define a Hyperbole KEY bound to COMMAND if KEY is not in `hyperbole-mode-map'.
 Third argument NO-ADD is ignored."
   (let ((lookup-result (lookup-key hyperbole-mode-map key)))
     (cond ((integerp lookup-result)
@@ -237,7 +259,9 @@ of the commands."
     ;; Setup so Hyperbole menus can be accessed from a key.  If not
     ;; already bound to a key, this typically binds the command `hyperbole'
     ;; globally to {C-h h} and activates Hyperbole minor mode.
-    (unless (where-is-internal #'hyperbole (current-global-map))
+    (unless (or (where-is-internal #'hyperbole (current-global-map))
+		(null help-char)
+		(not (keymapp (lookup-key (current-global-map) (char-to-string help-char)))))
       ;; In GNU Emacs, this binding replaces a command that shows
       ;; the word hello in foreign languages; this binding makes this
       ;; key much more useful.
@@ -250,14 +274,14 @@ of the commands."
     ;; and load the Hyperbole mouse key bindings.
     (unless (where-is-internal #'hkey-either)
       ;; Need to map all these variants to ensure can override
-      ;; org-meta-return in Org mode when desired.
+      ;; `org-meta-return' in Org mode when desired.
       (mapc (lambda (key) (hkey-set-key (kbd key) #'hkey-either))
 	    '("\M-\C-m" "M-<return>" "M-RET" "ESC <return>" "ESC RET")))
     ;;
     ;; Typically bind the key, {C-h A}, for Action Key help and {C-u C-h A} for Assist key
     ;; help.
-    (or (where-is-internal #'hkey-help)
-	(hkey-set-key (vector help-char ?A) #'hkey-help))
+    (unless (where-is-internal #'hkey-help)
+      (hkey-set-key (vector help-char ?A) #'hkey-help))
     ;;
     ;; Define virtual key used to activate hyperbole minor modeline menu
     ;; (hkey-set-key [hyperbole] (infodock-hyperbole-menu t))
@@ -337,7 +361,7 @@ of the commands."
 (if (boundp 'mouse-position-function)
     (setq mouse-position-function
 	  (lambda (frame-x-dot-y)
-	    "Make `mouse-position' and `mouse-pixel-position' always return the selected frame.
+	    "Make `mouse-position' and `mouse-pixel-position' return the selected frame.
 Under macOS and Windows 7 at least, upon initial selection of a new
 frame, those functions by default still return the prior frame."
 	    (if (consp frame-x-dot-y) (setcar frame-x-dot-y (selected-frame)))
@@ -371,65 +395,18 @@ frame, those functions by default still return the prior frame."
 (add-hook 'temp-buffer-show-hook #'hkey-help-show)
 (setq temp-buffer-show-function #'hkey-help-show)
 
-;;; ************************************************************************
-;;; Autoloads
-;;; ************************************************************************
-
-;; New autoload generation function defined only in Emacs 28
-(unless (fboundp #'make-directory-autoloads)
-  (defun make-directory-autoloads (dir output-file)
-    "Update autoload definitions for Lisp files in the directories DIRS.
-DIR can be either a single directory or a list of
-directories.  (The latter usage is discouraged.)
-
-The autoloads will be written to OUTPUT-FILE.  If any Lisp file
-binds ‘generated-autoload-file’ as a file-local variable, write
-its autoloads into the specified file instead.
-
-The function does NOT recursively descend into subdirectories of the
-directory or directories specified."
-    ;; Don't use a 'let' on this next line or it will fail.
-    (setq generated-autoload-file output-file)
-    (hypb:with-suppressed-warnings ((obsolete update-directory-autoloads))
-      (update-directory-autoloads dir))))
-
-;; Menu items could call this function before Info is loaded.
-(autoload 'Info-goto-node   "info"       "Jump to specific Info node."  t)
-
-;; Auto-autoload doesn't work for next item because it is defined
-;; within a condition-case, so autoload it here.
-(autoload 'Vm-init          "hvm"    "Initializes Hyperbole Vm support." t)
 
 ;;; ************************************************************************
 ;;; Outline Mode Aliases
 ;;; ************************************************************************
 
 (require 'outline)
-(unless (fboundp 'outline-hide-body)
-  (defalias 'outline-hide-body 'hide-body))
-(unless (fboundp 'outline-hide-entry)
-  (defalias 'outline-hide-entry 'hide-entry))
-(unless (fboundp 'outline-show-entry)
-  (defalias 'outline-show-entry 'show-entry))
-(unless (fboundp 'outline-show-all)
-  (defalias 'outline-show-all 'show-all))
-(unless (fboundp 'outline-hide-subtree)
-  (defalias 'outline-hide-subtree 'hide-subtree))
-(unless (fboundp 'outline-show-subtree)
-  (defalias 'outline-show-subtree 'show-subtree))
-(unless (fboundp 'outline-flag-region)
-  (defun outline-flag-region (from to flag)
-    "Hide or show lines from FROM to TO, according to FLAG.
-If FLAG is nil then text is shown, while if FLAG is t the text is hidden."
-    (if flag
-	(subst-char-in-region from to ?\n ?\r t)
-      (subst-char-in-region from to ?\r ?\n t))))
 (unless (fboundp 'outline-invisible-in-p)
   (defun outline-invisible-in-p (beg end)
     "Return t if there is an invisible character between BEG and END, else nil."
     (catch 'result
       (let ((p beg))
-	(while (< p end) 
+	(while (< p end)
 	  (when (eq (get-char-property p 'invisible) 'outline)
 	    (throw 'result t))
 	  (setq p (1+ p))))
@@ -455,17 +432,10 @@ If FLAG is nil then text is shown, while if FLAG is t the text is hidden."
 ;; Mh-e
 (var:append 'mh-inc-folder-hook '(Mh-init))
 ;;
-;; VM support is based on V5.72 beta of VM.  If you have a version of VM
-;; earlier than 5.70 beta, you should either upgrade or comment out the
-;; following line so that Hyperbole support for VM is not enabled.
-(var:append 'vm-mode-hook       '(Vm-init))
-;;
 ;; Hyperbole mail composer support configuration.
 ;;
 (var:append 'message-mode-hook   (list (lambda () (require 'hsmail))))
 (var:append 'mh-letter-mode-hook (list (lambda () (require 'hsmail))))
-(var:append 'vm-mail-mode-hook   (list (lambda () (require 'hsmail))))
-
 
 ;;; ************************************************************************
 ;;; URL Browsing
@@ -500,13 +470,6 @@ If FLAG is nil then text is shown, while if FLAG is t the text is hidden."
 		   user-mail-address)
 	      (concat (user-login-name) (hypb:domain-name)))))
   ;;
-  ;; When running from git source and not a release package, ensure
-  ;; *-autoloads.el files are already generated or generate them.
-  ;; Then ensure they are loaded.
-  (unless noninteractive
-    (hyperb:maybe-generate-autoloads))
-  (hyperb:maybe-load-autoloads)
-  ;;
   ;; Modify syntactic character pairs for use with implicit button activations.
   (hbut:modify-syntax)
   ;;
@@ -526,58 +489,34 @@ If FLAG is nil then text is shown, while if FLAG is t the text is hidden."
 				'buttons
 			      t)))
   ;;
+  ;; This next function call must be run before any tests involving Org
+  ;; in case the user has installed a new version of Org but Emacs has
+  ;; loaded parts of Org before his load path is finalized.  It loads
+  ;; the newer version of Org, if any, assuming `load-path' is configured
+  ;; correctly.
+  (hsys-org-fix-version)
+  ;;
+  ;; When vertico-mode is used, vertico-mouse-mode is needed for the
+  ;; Action Key to properly select completions from the candidate
+  ;; list, so force its usage when vertico-mode is invoked.
+  ;; vertico-mouse-mode should be an autoload.
+  (when (fboundp #'vertico-mouse-mode)
+    (add-hook 'vertico-mode-hook (lambda () (vertico-mouse-mode 1))))
+  ;;
   ;; Hyperbole initialization is complete.
   (message "Initializing Hyperbole...done"))
+  
 
-(defun hyperb:autoloads-exist-p ()
-  "Return t if all Hyperbole autoload files exist or nil otherwise."
-  (and (file-readable-p (expand-file-name "hyperbole-autoloads.el" hyperb:dir))
-       (file-readable-p (expand-file-name "kotl-autoloads.el"
-					  (expand-file-name "kotl" hyperb:dir)))))
-
-(defun hyperb:maybe-generate-autoloads ()
-  "Ensure Hyperbole *-autoload.el files are already generated or generate them.
-This is used only when running from git source and not a package release."
-  (unless (hyperb:autoloads-exist-p)
-    (hyperb:generate-autoloads)))
-
-(defun hyperb:generate-autoloads ()
-  "Renerate Hyperbole *-autoloads.el files whether they already exist or not."
-  (let* ((default-directory hyperb:dir)
-	 (backup-inhibited t)
-	 (find-file-hook) ;; Prevent header insertion
-	 (al-file (expand-file-name "hyperbole-autoloads.el")))
-    ;; (make-local-variable 'generated-autoload-file)
-    (with-current-buffer (find-file-noselect al-file)
-      (make-directory-autoloads "." al-file))
-    (setq al-file (expand-file-name "kotl/kotl-autoloads.el"))
-    (with-current-buffer (find-file-noselect al-file)
-      (make-directory-autoloads "." al-file)))
-  (unless (hyperb:autoloads-exist-p)
-    (error (format "Hyperbole failed to generate autoload files; try running 'make src' in a shell in %s" hyperb:dir))))
-
-(defun hyperb:maybe-load-autoloads ()
-  "Load Hyperbole autoload files that have not already been loaded."
-  (let* ((default-directory hyperb:dir)
-	 (hypb-autoloads (expand-file-name "hyperbole-autoloads.el"))
-	 (kotl-autoloads (expand-file-name "kotl/kotl-autoloads.el")))
-    (unless (featurep 'hyperbole-autoloads)
-      (when (file-readable-p hypb-autoloads)
-        (load-file hypb-autoloads)))
-    (unless (featurep 'kotl-autoloads)
-      (when (file-readable-p kotl-autoloads)
-        (load-file kotl-autoloads)))))
-
-;; This call loads the rest of the Hyperbole system.
-(require 'hinit)
+  ;; This call loads the rest of the Hyperbole system.
+  (require 'hinit)
 
 (defun hyperbole--enable-mode ()
   "Enable Hyperbole global minor mode."
-  ;; Store the current value and set `mark-even-if-inactive' to nil so
-  ;; can select delimited things if the region is not active when
-  ;; hyperbole-mode is enabled.
-  (setq hyperbole--mark-even-if-inactive mark-even-if-inactive
-	mark-even-if-inactive nil)
+  ;; If Hyperbole is loaded without the user's init being run,
+  ;; then force execution of (hyperb:init) when the mode is enabled.
+  (unless (and hkey-init (where-is-internal #'hkey-help))
+    (hyperb:init)
+    (remove-hook 'after-init-hook #'hyperb:init))
   ;;
   ;; Abbreviate MSWindows /cygdrive mount point paths.
   (when (file-exists-p "/cygdrive")
@@ -609,11 +548,6 @@ This is used only when running from git source and not a package release."
 				     directory-abbrev-alist)
 	hpath:posix-mount-point-to-mswindows-alist nil)
   ;;
-  ;; Reset the value of `mark-even-if-inactive' if the user has not
-  ;; changed it while Hyperbole was active.
-  (unless mark-even-if-inactive
-    (setq mark-even-if-inactive hyperbole--mark-even-if-inactive))
-  ;;
   (remove-hook (if (boundp 'write-file-functions)
 		   'write-file-functions
 		 'write-file-hooks)
@@ -629,15 +563,22 @@ This is used only when running from git source and not a package release."
   ;; Initialize after other key bindings are loaded at startup.
   (add-hook 'after-init-hook #'hyperb:init t))
 
-;; !! FIXME: This next expression activates Hyperbole for compatibility
-;; with prior releases where (require 'hyperbole) was enough to
-;; activate its key bindings.  However, loading a file should not
-;; change Emacs's behavior, so after educating users to add this
-;; next line to their Emacs initializations, it should be removed.
-(hyperbole-mode 1)
-
 (makunbound 'hyperbole-loading)
 
+;; Autoload this form so that when `package.el' activates Hyperbole's autoloads
+;; it also sets up Kotl's autoloads.
+;;;###autoload
+(let ((us (if (fboundp 'macroexp-file-name)
+              (macroexp-file-name)
+	    load-file-name)))
+  (when us
+    ;; Contrary to the usual ELPA autoloads files, `kotl-autoloads'
+    ;; does not add its directory to `load-path', so let's do it here
+    ;; by hand.
+    (add-to-list 'load-path
+                 (expand-file-name "kotl" (file-name-directory us)))
+    (require 'kotl-autoloads nil t)))
+
 (provide 'hyperbole)
 
 ;;; hyperbole.el ends here
