@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    15-Mar-89
-;; Last-Mod:     24-Jan-22 at 00:23:35 by Bob Weiner
+;; Last-Mod:     20-Jan-24 at 20:20:58 by Mats Lidell
 ;;
-;; Copyright (C) 1989-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1989-2024  Free Software Foundation, Inc.
 ;; See the "../HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -48,9 +50,8 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(require 'hargs)
+(require 'hargs) ;; this requires 'hypb
 (require 'ring)
-(require 'set)
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -58,7 +59,7 @@
 
 (define-obsolete-variable-alias 'wconfig-ring-max 'hywconfig-ring-max "06.00")
 (defcustom hywconfig-ring-max 10
-  "*Maximum length of Hyperbole window configuration ring before oldest elements are deleted."
+  "*Max length of Hyperbole window config ring before oldest elements are deleted."
   :type `(integer
           :match ,(lambda (_widget value) (and (integerp value) (> value 0))))
   :group 'hyperbole-screen)
@@ -73,58 +74,63 @@
 (defun hywconfig-add-by-name (name)
   "Save the current window configuration under the string NAME.
 When called interactively and a window configuration already exists under
-NAME, confirms whether or not to replace it."
+NAME, confirm whether or not to replace it.
+
+Return t if name is added or replaced, else nil."
   (interactive "sName for current window configuration: ")
-  (or (stringp name)
-      (error "(hywconfig-add-by-name): `name' argument is not a string: %s" name))
-  (let ((set:equal-op  (lambda (key elt) (equal key (car elt))))
-	(wconfig-names (hywconfig-get-names)))
-    (if (or (not (called-interactively-p 'interactive))
-	    (not (set:member name wconfig-names))
+  (unless (stringp name)
+    (error "(hywconfig-add-by-name): `name' argument is not a string: %s" name))
+  (when (or (not (called-interactively-p 'interactive))
+	    (not (member name (hywconfig-named-get-names)))
 	    (y-or-n-p
 	     (format "Replace existing `%s' window configuration? " name)))
-	(progn (hywconfig-set-names (set:replace name (current-window-configuration)
-						 wconfig-names))
-	       (if (called-interactively-p 'interactive)
-		   (message
-		    (substitute-command-keys
-		     (format "Window configuration `%s' saved.  Use {\\[hywconfig-restore-by-name]} to restore." name))))))))
+    (hywconfig-named-put name (current-window-configuration))
+    (when (called-interactively-p 'interactive)
+      (message
+       (substitute-command-keys
+	(format "Window configuration `%s' saved.  Use {\\[hywconfig-restore-by-name]} to restore." name))))
+    t))
 
 ;;;###autoload
 (defun hywconfig-delete-by-name (name)
-  "Deletes frame-specific window configuration saved under NAME."
+  "Delete frame-specific window configuration saved with NAME.
+
+Return t if name exists and is deleted, else nil."
   (interactive (list (hargs:read-match "Delete window configuration named: "
-				       (hywconfig-get-names) nil t)))
+				       (hywconfig-named-get-names) nil t)))
   (cond ((null name)
 	 (message "There is no named window configuration to delete."))
 	((not (stringp name))
 	 (error "(hywconfig-delete-by-name): `name' argument is not a string: %s" name))
-	(t (let ((set:equal-op (lambda (key elt) (equal key (car elt)))))
-	     (hywconfig-set-names (set:remove name (hywconfig-get-names)))
-	     (if (called-interactively-p 'interactive)
-		 (message "Window configuration `%s' has been deleted." name))))))
+	(t (let ((removed-flag (hywconfig-named-remove name)))
+	     (when (and removed-flag (called-interactively-p 'interactive))
+	       (message "Window configuration `%s' has been deleted." name))
+	     removed-flag))))
 
 ;;;###autoload
 (defun hywconfig-restore-by-name (name)
-  "Restore frame-specific window configuration saved under NAME."
+  "Restore frame-specific window configuration saved with NAME.
+
+Return t if name exists and is restored, else nil."
   (interactive (list (hargs:read-match "Restore window configuration named: "
-				       (hywconfig-get-names) nil t)))
+				       (hywconfig-named-get-names) nil t)))
   (cond ((null name)
 	 (message "There is no named window configuration to restore."))
 	((not (stringp name))
 	 (error "(hywconfig-restore-by-name): `name' argument is not a string: %s" name))
-	(t (let ((wconfig (set:get name (hywconfig-get-names))))
+	(t (let ((wconfig (hywconfig-named-get name)))
 	     (if wconfig
 		 (progn (hywconfig-set-window-configuration wconfig)
-			(if (called-interactively-p 'interactive)
-			    (message "Window configuration `%s' is now active." name)))
+			(when (called-interactively-p 'interactive)
+			  (message "Window configuration `%s' is now active." name))
+			t)
 	       (error "(hywconfig-restore-by-name): No window configuration for this frame named `%s'" name))))))
 
 ;;; Window configuration ring management (like text kill ring).
 ;;;###autoload
 (defun hywconfig-delete-pop ()
-  "Replace the current frame's window configuration with the one most recently saved to the ring.
-Then deletes this new configuration from the ring."
+  "Replace the selected frame's window configuration with the most recently saved.
+Then delete this new configuration from the ring."
   (interactive)
   (let ((ring (hywconfig-get-ring)))
     (if (ring-empty-p ring)
@@ -134,25 +140,35 @@ Then deletes this new configuration from the ring."
 	(hywconfig-set-window-configuration (ring-ref ring 0))
 	(ring-remove ring 0)))))
 
+;;;###autoload
+(defun hywconfig-delete-pop-continue ()
+  "Replace selected frame's window configuration with the most recently saved.
+Then delete this new configuration from the ring.  If the hywconfig
+ring is not empty, then stay in the hywconfig menu."
+  (interactive)
+  (hywconfig-delete-pop)
+  (unless (hywconfig-ring-empty-p)
+    (hyperbole 'win)))
+
 ;;;###autoload
 (defun hywconfig-ring-empty-p ()
-  "Return t if the wconfig ring for the current frame is empty; nil otherwise."
+  "Return t if the wconfig ring for the selected frame is empty; nil otherwise."
   (ring-empty-p (hywconfig-get-ring)))
 
 ;;;###autoload
 (defun hywconfig-ring-save ()
-  "Save the current frame's window configuration onto the save ring.
+  "Save the selected frame's window configuration onto the save ring.
 Use {\\[hywconfig-yank-pop]} to restore it at a later time."
   (interactive)
   (ring-insert (hywconfig-get-ring) (current-window-configuration))
-  (if (called-interactively-p 'interactive)
-      (message
-       (substitute-command-keys
-	"Window configuration saved.  Use {\\[hywconfig-yank-pop]} to restore."))))
+  (when (called-interactively-p 'interactive)
+    (message
+     (substitute-command-keys
+      "Window configuration saved.  Use {\\[hywconfig-yank-pop]} to restore."))))
 
 ;;;###autoload
 (defun hywconfig-yank-pop (n)
-  "Replace the current frame's window configuration with the prefix arg Nth prior one in save ring.
+  "Replace selected frame's window config with prefix arg Nth prior one in ring.
 Interactively, default value of N = 1, means the last saved window
 configuration is displayed.
 
@@ -167,38 +183,71 @@ oldest one comes the newest one."
       (ring-insert-at-beginning ring prev)
       (hywconfig-set-window-configuration (ring-ref ring 0)))))
 
+;;;###autoload
+(defun hywconfig-yank-pop-continue (n)
+  "Replace selected frame's window config with prefix arg Nth prior one in ring.
+If there are more than one entries in the ring, then stay in the hywconfig menu.
+
+Interactively, default value of N = 1, means the last saved window
+configuration is displayed.
+
+The sequence of window configurations wraps around, so that after the
+oldest one comes the newest one."
+  (interactive "p")
+  (hywconfig-yank-pop n)
+  (when (> (ring-length (hywconfig-get-ring)) 1)
+    (hyperbole 'win)))
+
 ;;; ************************************************************************
 ;;; Private functions
 ;;; ************************************************************************
 
-(defun hywconfig-get-names ()
-  "Return the current frame's list of named window configurations."
-  (let* ((frame (selected-frame))
-	 (names (frame-parameter frame 'hywconfig-names)))
-    (if (not names)
-	(set-frame-parameter frame 'hywconfig-names (setq names (set:create))))
-    names))
+(defun hywconfig-named-get (name)
+  "Return the selected frame's window configuration with NAME."
+  (with-suppressed-warnings ((obsolete lax-plist-get)) ;; Obsolete since 29.1, use plist-get
+    (lax-plist-get (hywconfig-named-get-entries) name)))
 
-(defun hywconfig-set-names (names)
-  "Set the current frame's list of named window configurations."
-  (set-frame-parameter (selected-frame) 'hywconfig-names names))
+(defun hywconfig-named-get-names ()
+  "Return the selected frame's list of window configuration names."
+  (hypb:map-plist (lambda (name _wconfig) name) (hywconfig-named-get-entries)))
+
+(defun hywconfig-named-get-entries ()
+  "Get the selected frame's plist of named window configurations."
+  (frame-parameter (selected-frame) 'named-hywconfigs))
+
+(defun hywconfig-named-put (name wconfig)
+  "Add WCONFIG with NAME to selected frame's plist of named window configurations."
+  (hywconfig-named-set-entries
+   (with-suppressed-warnings ((obsolete lax-plist-put)) ;; Obsolete since 29.1, use plist-get
+     (lax-plist-put (hywconfig-named-get-entries) name wconfig))))
+
+(defun hywconfig-named-remove (name)
+  "Remove the selected frame's stored window configuration with NAME."
+  (let ((plist (hywconfig-named-get-entries)))
+    (prog1 (hypb:remove-from-plist plist name)
+      (hywconfig-named-set-entries plist))))
+
+(defun hywconfig-named-set-entries (entries)
+  "Set the selected frame's plist of named window configuration ENTRIES."
+  (set-frame-parameter (selected-frame) 'named-hywconfigs entries))
 
 (defun hywconfig-get-ring ()
-  "Return the current frame's window configuration ring."
+  "Return the selected frame's window configuration ring."
   (let* ((frame (selected-frame))
 	 (ring (frame-parameter frame 'hywconfig-ring)))
-    (if (not ring)
-	(set-frame-parameter frame 'hywconfig-ring (setq ring (make-ring hywconfig-ring-max))))
+    (unless ring
+      (set-frame-parameter frame 'hywconfig-ring (setq ring (make-ring hywconfig-ring-max))))
     ring))
 
 (defun hywconfig-set-window-configuration (wconfig)
-  (if (window-configuration-p wconfig)
-      (condition-case nil
-	  (progn (set-window-configuration wconfig) t)
-	(error (message "(HyWconfig): Invalid window configuration, `%s'" wconfig)
-	       (beep)
-	       (sit-for 2)
-	       nil))))
+  "Return window configureation WCONFIG within the selected frame."
+  (when (window-configuration-p wconfig)
+    (condition-case nil
+	(progn (set-window-configuration wconfig) t)
+      (error (message "(HyWconfig): Invalid window configuration, `%s'" wconfig)
+	     (beep)
+	     (sit-for 2)
+	     nil))))
 
 (run-hooks 'hywconfig-load-hook)
 
