@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    26-Feb-98
-;; Last-Mod:     12-Feb-22 at 10:42:20 by Mats Lidell
+;; Last-Mod:      5-Nov-23 at 11:45:00 by Bob Weiner
 ;;
-;; Copyright (C) 1998-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1998-2022  Free Software Foundation, Inc.
 ;; See the "../HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -36,36 +38,25 @@
 ;;; ************************************************************************
 
 (defvar kexport:input-filename nil
-  "This is automatically set to the full pathname of the file presently being exported.")
+  "This is automatically set to the full pathname of the exported file.")
 
 (defvar kexport:output-filename nil
-  "This is automatically set to the full pathname of the file presently being exported.")
-
-(defcustom kexport:html-body-attributes
-  "BGCOLOR=\"#FFFFFF\"" ;; white background
-  "*String of HTML attributes attached to the <BODY> tag of an HTML exported koutline file."
-  :type 'string
-  :group 'hyperbole-koutliner)
+  "This is automatically set to the full pathname of the exported file.")
 
 (defcustom kexport:html-description
-  "Created by Hyperbole's outliner.\nSee \"(hyperbole)Koutliner\" for more information."
+  "Created by Hyperbole's outliner.\nSee &quot;(hyperbole)Koutliner&quot; for more information."
   "*String to insert as the HTML-exported document's description, or nil for none."
   :type '(choice (const nil)
 		 (string))
   :group 'hyperbole-koutliner)
 
 (defcustom kexport:html-keywords nil
-  "*String of comma separated keywords to include with an HTML-exported document, or nil for none."
+  "*String of comma separated keywords to include with an HTML-exported document.
+If nil, use no keywords."
   :type '(choice (const nil)
 		 (string))
   :group 'hyperbole-koutliner)
 
-(defcustom kexport:label-html-font-attributes
-  "COLOR=\"#C100C1\" SIZE=\"-1\""
-  "*String of HTML font attributes attached to kcell labels when exported."
-  :type 'string
-  :group 'hyperbole-koutliner)
-
 
 (defvar kexport:kcell-reference-regexp
   "[0-9a-zA-Z][.0-9a-zA-Z]*=\\([.0-9a-zA-Z]+\\)")
@@ -93,7 +84,7 @@
     'kexport:html-url)
    ;;
    ;; make mail addresses into hyperbuttons
-   (cons mail-address-regexp "<a href=\"mailto:\\1\"><i>\\1</i></a>\\2")
+   (cons hypb:mail-address-regexp "<a href=\"mailto:\\1\"><i>\\1</i></a>\\2")
    ;;
    ;; make klinks into hyperlinks
    (cons (concat "&lt;\\s-*@\\s-*" kexport:kcell-reference-regexp
@@ -108,9 +99,9 @@
    (cons (format "&lt;\\s-*\\([^ \t\n\r,<>]+\\)\\s-*,\\s-*%s[^=&>]*&gt;"
 		 kexport:kcell-partial-reference-regexp)
 	 'kexport:html-file-klink))
-  "*List of (regexp . replacement-pattern) elements applied in order to the
-contents of each kcell from a koutline exported to HTML format.  Replacement
-pattern may be:
+  "*List of (regexp . replacement-pattern) elements applied in order.
+The elements are applied to the contents of each kcell from a
+koutline exported to HTML format.  Replacement pattern may be:
   a string with references to regexp's grouping numbers, e.g. \\1, or
   a function of one argument (it is passed the string being replaced in)
   which returns the modified string.  The function may use expressions such
@@ -125,10 +116,6 @@ Font Awesome Free is free and GPL friendly.")
 (defconst kexport:font-awesome-css-include
   "<style>
 
-button {
- display: inline;
-}
-
 span.nobreak {
   white-space: nowrap;
 }
@@ -141,26 +128,64 @@ li {
  list-style-type: none;
 }
 
-.collapsible {
+ul {
+  margin-top: 0px;
+  margin-bottom: 0px;
+  padding-left: 30px;
+}
+
+.tdone {
+  vertical-align: text-bottom;
+  width: 1%;
+}
+
+.tdtwo {
+  vertical-align: text-bottom;
+  width: 2%;
+}
+
+.btn {
   all: unset;
   background-color: inherit;
   cursor: pointer;
   display: block;
-  font-size: 0;
   outline: inherit;
 }
 
-.collapsible:hover {
+.btn:hover {
   background-color: #FAFAD2;
-  font-size: 0;
+  font-size: 20px;
 }
 
 .content {
   display: block;
-  font-size: 0;
+  font-size: 20px;
+  margin-top: 0px;
+  margin-bottom: 0px;
 }
+
+.label {
+  display: block;
+  color: #C100C1;
+}
+
+h1 {
+  margin-top: 0px;
+  margin-bottom: 0px;
+}
+
+pre {
+  margin-top: 0px;
+  margin-bottom: 0px;
+  font-size: 20px;
+}
+
+body {
+  background-color: white;
+}
+
 </style>\n"
-  "CSS that styles collapsible HTML-exported Koutline parent cells")
+  "CSS that styles collapsible HTML-exported Koutline parent cells.")
 
 (defconst kexport:font-awesome-css-include-with-menus
   "<style>
@@ -187,6 +212,7 @@ li {
 
 .collapsible:hover {
   background-color: #FAFAD2;
+  font-size: 20px;
 }
 
 .content {
@@ -258,7 +284,7 @@ nav > menu > menuitem > a + menu:after{
    position:absolute;
    border:10px solid transparent;
    border-top: 10px solid white;
-   left:12px;
+   left:20px;
    top: -40px;
 }
 nav menuitem > menu > menuitem > a + menu:after{
@@ -294,35 +320,46 @@ menuitem > menu > menuitem.hover > menu > menuitem{
 }
 /* End Drop-down menu CSS */
 </style>\n"
-  "CSS that styles collapsible HTML-exported Koutline parent cells and menus")
+  "CSS that styles collapsible HTML-exported Koutline parent cells and menus.")
 
-(defconst kexport:font-awesome-collapsible-javascript
+(defconst kexport:font-awesome-collapsible-javascript-btn
   "<script>
-var coll = document.getElementsByClassName('collapsible');
-var i;
+var allSpan = document.getElementsByClassName('btn');
 
 function childElt(elt, tag)
 {
     return elt.getElementsByTagName(tag)[0];
 }
 
-for (i = 0; i < coll.length; i++) {
-  coll[i].addEventListener('click', function() {
-    var icon = childElt(this, 'span');
-    var content = this.nextElementSibling;
-    if (content.style.display === 'none') {
-      content.style.display = 'block';
-      icon.classList.add('fas', 'fa-chevron-down');
-      icon.classList.remove('fa-chevron-right');
-    } else {
-      content.style.display = 'none';
-      icon.classList.add('fas', 'fa-chevron-right');
-      icon.classList.remove('fa-chevron-down');
+for (var x = 0; x < allSpan.length; x++)
+{
+  allSpan[x].onclick = function()
+  {
+    if (this.parentNode)
+    {
+      var icon = childElt(this, 'span');
+      var childList = this.parentNode.getElementsByTagName('li');
+      for (var y = 0; y < childList.length; y++)
+      {
+        var currentState = childList[y].style.display;
+        if (currentState == 'none')
+        {
+          childList[y].style.display = 'block';
+          icon.classList.add('fas', 'fa-chevron-down');
+          icon.classList.remove('fa-chevron-right');
+        }
+        else
+        {
+          childList[y].style.display='none';
+          icon.classList.add('fas', 'fa-chevron-right');
+          icon.classList.remove('fa-chevron-down');
+        }
+      }
     }
-  });
+  }
 }
 </script>\n"
-  "JavaScript which expands/collapses HTML-exported Koutline parent cells")
+  "JavaScript which expands/collapses HTML-exported Koutline parent cells.")
 
 ;;; ************************************************************************
 ;;; Public functions
@@ -330,24 +367,29 @@ for (i = 0; i < coll.length; i++) {
 
 ;;;###autoload
 (defun kexport:koutline (&optional soft-newlines-flag)
-  "Export the current buffer's koutline to the same named file with a \".html\" suffix.
+  "Export current buffer's koutline to the same named file with a \".html\" suffix.
 Return the pathname of the html file created.
 
-By default, this retains newlines within cells as they are.  With optional prefix arg, SOFT-NEWLINES-FLAG,
-hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks."
+By default, this retains newlines within cells as they are.  With
+optional prefix arg, SOFT-NEWLINES-FLAG, hard newlines are not
+used.  Also converts Urls and Klinks into Html hyperlinks."
   (interactive "P")
   (let ((export-from buffer-file-name)
   	(output-to (concat (file-name-sans-extension buffer-file-name) ".html")))
-    (kexport:html export-from output-to soft-newlines-flag)
+    (if (and buffer-file-name (string-match "\\.kotl?\\'" buffer-file-name))
+	(kexport:html export-from output-to soft-newlines-flag)
+      (error "(kexport:koutline): current buffer must be a Koutline .kotl file"))
     output-to))
 
 ;;;###autoload
 (defun kexport:display (&optional soft-newlines-flag)
-  "Export the current buffer's koutline to the same named file with a \".html\" suffix and display it in a web browser.
+  "Export the current buffer's koutline and display it in a web browser.
+The buffer is exported to the same named file with a \".html\" suffix.
 Return the pathname of the html file created.
 
-By default, this retains newlines within cells as they are.  With optional prefix arg, SOFT-NEWLINES-FLAG,
-hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks."
+By default, this retains newlines within cells as they are.  With
+optional prefix arg, SOFT-NEWLINES-FLAG, hard newlines are not
+used.  Also converts Urls and Klinks into Html hyperlinks."
   (interactive "P")
   (let ((html-file (kexport:koutline soft-newlines-flag)))
     (hact 'www-url (concat "file://" html-file))
@@ -356,8 +398,9 @@ hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks.
 ;;;###autoload
 (defun kexport:html (export-from output-to &optional soft-newlines-flag)
   "Export a koutline buffer or file in EXPORT-FROM to html format in OUTPUT-TO.
-By default, this retains newlines within cells as they are.  With optional prefix arg, SOFT-NEWLINES-FLAG,
-hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks.
+By default, this retains newlines within cells as they are.  With
+optional prefix arg, SOFT-NEWLINES-FLAG, hard newlines are not
+used.  Also converts Urls and Klinks into Html hyperlinks.
 !! STILL TODO:
   Make delimited pathnames into file links (but not if within klinks).
   Copy attributes stored in cell 0 and attributes from each cell."
@@ -378,27 +421,22 @@ hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks.
 		 (buffer-name (find-file-noselect export-from)))
 		(t (error
 		    "(kexport:html): `%s' is an invalid `export-from' argument" export-from))))
-	 (font-lock-auto-fontify) ;; Prevent syntax highlighting
-	 (font-lock-mode-disable-list '(html-mode))
-	 (font-lock-mode-enable-list)
-	 (html-mode-hook)
-	 (hm--html-mode-hook)
-	 (psgml-mode-hook)
-	 (output-to-buf-name
+	 (output-to-buf
 	  (cond ((or (bufferp output-to)
 		     (get-buffer output-to))
-		 (buffer-name (get-buffer output-to)))
+		 (get-buffer output-to))
 		((get-file-buffer output-to)
-		 (buffer-name (get-file-buffer output-to)))
+		 (get-file-buffer output-to))
 		((stringp output-to)
-		 (buffer-name (find-file-noselect output-to)))
+		 (find-file-noselect output-to))
 		(t (error
 		    "(kexport:html): `%s' is an invalid `output-to' argument" output-to))))
-	 (standard-output (get-buffer output-to-buf-name))
+	 (standard-output output-to-buf)
 	 ;; Get any title attribute from cell 0, invisible root of the outline
 	 (title (kcell:get-attr (kcell-view:cell-from-ref 0) 'title)))
 
     (with-current-buffer standard-output
+      (font-lock-mode 0) ;; Prevent syntax highlighting
       (setq buffer-read-only nil
 	    kexport:output-filename buffer-file-name)
       (erase-buffer))
@@ -409,7 +447,7 @@ hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks.
 
 	;; If called interactively, prompt user for the title to use.
 	(if (called-interactively-p 'interactive)
-	    (setq title (read-string (format "Title for %s: " output-to-buf-name)
+	    (setq title (read-string (format "Title for %s: " (buffer-name output-to-buf))
 				     title))
 	  ;; Otherwise, use any previously retrieved title attribute or if
 	  ;; none, then the name of the current file sans the .kotl suffix.
@@ -419,110 +457,75 @@ hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks.
 	  (when (string-match "\n" title)
 	    (setq title (substring title 0 (match-beginning 0)))))
 
-	(princ "<html><head>\n\n")
-	(princ "<a id=\"top\"></a><a id=\"k0\"></a>\n")
-	(princ (format "<title>%s</title>\n" title))
+	(princ "<!DOCTYPE html>\n")
+	(princ "<html lang=\"en\">\n")
+
+        ;; HEAD
+        (princ "<head>\n")
+        (princ (format "<title>%s</title>\n" title))
+
+        (princ "<meta charset=\"utf-8\">\n\n")
 	(if kexport:html-description
-	    (princ (format "<meta id=\"description\" content=\"%s\">\n"
+	    (princ (format "<meta name=\"description\" content=\"%s\">\n"
 			   kexport:html-description)))
 	(if kexport:html-keywords
 	    (princ (format "<meta id=\"keywords\" content=\"%s\">\n"
 			   kexport:html-keywords)))
 	(princ "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">")
+
 	;; CSS
 	(princ (format "<link rel=\"stylesheet\" href=\"%s\">\n" kexport:font-awesome-css-url))
 	(princ kexport:font-awesome-css-include)
-	;; HTML
+
 	(princ "</head>\n\n")
-	(princ (format "<body %s>\n\n" kexport:html-body-attributes))
-	(princ (format "<h1>%s</h1>\n\n" title))
-	;; (princ (format "<label for=\"show-menu\" class=\"show-menu\"><h1>%s</h1></label>\n\n" title))
-	;; (princ "<input type=\"checkbox\" id=\"show-menu\" role=\"button\">")
-	;; (princ "<nav>
-	;; 	<menu>
-	;; 		<menuitem id=\"title-menu\">
-	;; 			<a>Dropdown</a>
-	;; 			<menu>")
-	;; (let (text)
-	;;   (kview:map-siblings (lambda (kv)
-	;; 			(setq text (kcell-view:contents))
-	;; 			(princ (format "<menuitem><a href=\"#k%s\">%s</a></menuitem>\n"
-	;; 				       (kcell-view:label)
-	;; 				       (substring text 0 (string-match "\n" text)))))
-	;; 		      kview t))
-	;; (princ "         		</menu>
-	;; 		</menuitem>
-	;; 	</menu>
-   	;;     </nav>\n")
+
+        ;; BODY
+	(princ "<body>\n\n")
+        (princ (format "<h1>%s</h1>\n\n" title))
+
 	(let* ((separator
-		(hypb:replace-match-string
-		 ">" (hypb:replace-match-string
-		      "<" (kview:label-separator kview) "&lt;")
-		 "&gt;"))
-	       i is-parent is-last-sibling no-sibling-stack level label contents)
+		(replace-regexp-in-string
+		 ">" "&gt;"
+		 (replace-regexp-in-string
+		  "<" "&lt;" (kview:label-separator kotl-kview))))
+               no-sibling-stack)
+
+          (princ "<ul>\n")
+
 	  (kview:map-tree
 	   (lambda (_kview)
-	     (setq level (kcell-view:level)
-		   i level
-		   is-parent (kcell-view:child-p)
-		   is-last-sibling (not (kcell-view:sibling-p)))
-	     (when is-parent
-	       (push is-last-sibling no-sibling-stack)
-	       (princ "<button type=\"button\" class=\"collapsible\">\n"))
-	     (while (> i 1)
-	       (princ "<ul>")
-	       (setq i (1- i)))
-	     (princ "<li list-style-type=none>\n<table><tr valign=text-bottom>\n")
-	     ;; (princ "<td width=1% valign=top>")
-	     (princ "<td width=1%>")
-	     (princ (format "<span class=\"fas fa-chevron-down fa-fw\"%s></span>"
-			    (if is-parent
-				""
-			      ;; Fill same space for alignment but don't
-			      ;; show collapsible chevron when not a parent
-			      " style=\"visibility:hidden\"")))
-	     (princ "</td>\n")
-	     ;; (princ "<td width=2% valign=top>\n")
-	     (princ "<td width=2%>\n")
-	     (setq label (kcell-view:label))
-	     (princ (format "<a id=\"k%s\"></a>" label))
-	     (princ (format "<a id=\"k%s\"></a>\n" (kcell-view:idstamp)))
-	     (princ (format
-		     "<pre><font %s>%s%s</font></pre>\n"
-		     kexport:label-html-font-attributes
-		     label separator))
-	     (princ "</td>\n<td>\n")
-	     (setq contents (kcell-view:contents))
-	     (when (string-match "\\`\\([-_$%#@~^&*=+|/A-Za-z0-9 ]+\\):.*\\S-"
-				 contents)
-	       (princ (format "<a id=\"%s\"></a>"
-			      (substring contents 0 (match-end 1)))))
-	     (setq contents (kexport:html-markup contents))
-	     (if soft-newlines-flag
-		 (princ contents)
-	       (princ "<pre>") (princ contents) (princ "</pre>"))
-	     (princ "</td>\n")
-	     (princ "</tr></table></li>")
-	     (setq i level)
-	     (while (> i 1)
-	       (princ "</ul>")
-	       (setq i (1- i)))
-	     (cond (is-parent
-		    (princ "\n</button>\n<div class=\"content\">\n"))
-		   ((and (/= level 1) is-last-sibling)
-		    (princ "\n</div>")
-		    (while (pop no-sibling-stack)
-		      (princ "</div>"))))
-	     (when (not is-parent)
-	       (terpri) (terpri)))
-	   kview t)
+	     (let ((is-parent (kcell-view:child-p))
+	           (is-last-sibling (not (kcell-view:sibling-p))))
+
+               (princ (format "<!-- Level: %s, is-parent: %s, is-last-sibling: %s -->\n"
+                              (kcell-view:level) is-parent is-last-sibling))
+               (princ "<li>\n")
+
+               (kexport:princ-cell is-parent separator soft-newlines-flag)
+
+               (if is-parent
+                   (progn
+                     (princ "<ul>\n")
+                     (push is-last-sibling no-sibling-stack))
+                 (princ "</li>\n")
+                 (when is-last-sibling
+                   (princ "</ul>\n")
+                   (princ "</li>\n")
+                   (pop no-sibling-stack)
+                   (while (pop no-sibling-stack)
+                     (princ "</ul>\n")
+                     (princ "</li>\n"))))))
+	   kotl-kview t)
+
+          (princ "</ul>\n")
+
 	  ;; Remove any extra newline at the end of any <pre> text
 	  (save-excursion
 	    (goto-char (point-min))
 	    (when (re-search-forward "\r?\n\\'" nil t)
 	      (replace-match "" nil nil))))
 	;; JavaScript
-	(princ kexport:font-awesome-collapsible-javascript)
+	(princ kexport:font-awesome-collapsible-javascript-btn)
 	(princ "</body></html>\n")))
     (with-current-buffer standard-output
       (save-buffer))))
@@ -531,9 +534,36 @@ hard newlines are not used.  Also converts Urls and Klinks into Html hyperlinks.
 ;;; Private functions
 ;;; ************************************************************************
 
+(defun kexport:princ-cell (is-parent separator soft-newlines-flag)
+  "Export the contents of a cell."
+  (let (contents label)
+    (setq contents
+          (let ((cnt1 (kcell-view:contents)))
+            (concat
+	     (when (string-match "\\`\\([-_$%#@~^&*=+|/A-Za-z0-9 ]+\\):.*\\S-" cnt1)
+	       (format "<div id=\"%s\"></div>"
+                       (replace-regexp-in-string "[ \t]" "_" (substring cnt1 0 (match-end 1)))))
+	     (let ((cnt2 (kexport:html-markup cnt1)))
+	       (if soft-newlines-flag
+		   cnt2
+	         (concat "<pre>" cnt2 "</pre>"))))))
+    (setq label (kcell-view:label))
+    (setq contents
+          (concat "<table><tr>\n<td class=\"tdone\">\n"
+                  (format "<span class=\"fas fa-chevron-down fa-fw\"%s></span>\n"
+                          (if is-parent " " " style=\"visibility:hidden\""))
+                  "</td>\n<td class=\"tdtwo\">"
+                  (format "<span id=\"k%s\"></span>" label)
+                  (format "<span id=\"k%s\"></span>" (kcell-view:idstamp))
+                  (format "<pre class=\"label\">%s%s</pre>" label separator)
+                  "</td>\n<td>"
+                  contents
+                  "</td>\n</tr>\n</table>\n"))
+    (princ (format "<div class=\"%scontent\">\n%s</div>\n" (if is-parent "btn " "") contents))))
+
 (defun kexport:html-file-klink (string)
   "Convert STRING containing a klink with a file reference to Html format.
-Works exclusively within a call to `hypb:replace-match-string'."
+Works exclusively within a call to `replace-regexp-in-string'."
   (let ((filename (substring string (match-beginning 1)
 			     (match-end 1))))
     (if (equal filename (file-name-nondirectory
@@ -549,13 +579,13 @@ Works exclusively within a call to `hypb:replace-match-string'."
   "Perform replacements on STRING specified by `kexport:html-replacement-alist'."
   (mapc
    (lambda (elt)
-     (setq string (hypb:replace-match-string (car elt) string (cdr elt))))
+     (setq string (replace-regexp-in-string (car elt) (cdr elt) string)))
    kexport:html-replacement-alist)
   string)
 
 (defun kexport:html-url (string)
   "Convert STRING containing a Url to Html format.
-Works exclusively within a call to `hypb:replace-match-string'."
+Works exclusively within a call to `replace-regexp-in-string'."
   (let* ((url (substring string (match-beginning hpath:url-grpn)
 			 (match-end hpath:url-grpn)))
 	 (last-str-char (length string))
