@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    10/31/93
-;; Last-Mod:     17-Apr-22 at 12:17:44 by Bob Weiner
+;; Last-Mod:      4-Oct-23 at 19:10:12 by Mats Lidell
 ;;
-;; Copyright (C) 1993-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1993-2022  Free Software Foundation, Inc.
 ;; See the "../HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -17,7 +19,7 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(kproperty kmenu kview)))
+(eval-and-compile (mapc #'require '(kproperty kmenu kview kvspec kcell)))
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -58,26 +60,26 @@ Return the new kview."
     ;; Finding the file may have already done a kfile:read as invoked through
     ;; kotl-mode via a file local variable setting.  If so, don't read it
     ;; again.
-    (unless (kview:is-p kview)
+    (unless (kview:is-p kotl-kview)
       (kfile:read buffer existing-file))
     (unless (derived-mode-p 'kotl-mode)
       (kotl-mode))
-    kview))
+    kotl-kview))
 
 ;;;###autoload
 (defun kfile:is-p ()
-  "Iff current buffer contains an unformatted or formatted koutline, return file format version string, else nil."
+  "Iff current buffer contains a koutline, return file format version string.
+If not a koutline, return nil. This works both for unformatted
+and formatted koutlines."
   (let (ver-string)
     (save-excursion
       (save-restriction
 	(widen)
 	(goto-char (point-min))
-	(condition-case ()
-	    (progn
-	      (setq ver-string (read (current-buffer)))
-	      (and (stringp ver-string) (string-match "^Kotl-" ver-string)
-		   ver-string))
-	  (error nil))))))
+	(ignore-errors
+	  (setq ver-string (read (current-buffer)))
+	  (and (stringp ver-string) (string-match "^Kotl-" ver-string)
+	       ver-string))))))
 
 ;;;###autoload
 (defun kfile:view (file-name)
@@ -100,20 +102,19 @@ Return the new kview."
 ;;; ************************************************************************
 
 (defun kfile:create (buffer)
-  "Create a new koutline file attached to BUFFER, with a single empty level 1 kotl cell.
-Return file's kview."
+  "Create a new koutline file attached to BUFFER and return file's kview.
+File is created with a single empty level 1 kotl cell."
   (or buffer (setq buffer (current-buffer)))
   (unless (bufferp buffer)
     (error "(kfile:create): Invalid buffer argument, %s" buffer))
   (set-buffer buffer)
-  (when buffer-read-only
-    (error "(kfile:create): %s is read-only" buffer))
+  (barf-if-buffer-read-only)
   (widen)
 
-  (let ((empty-p (zerop (buffer-size)))
+  (let ((empty-flag (zerop (buffer-size)))
 	import-from view standard-output)
 
-    (unless empty-p
+    (unless empty-flag
       ;; This is a foreign file whose elements must be converted into
       ;; koutline cells.
       (setq import-from (kimport:copy-and-set-buffer buffer))
@@ -134,7 +135,7 @@ Return file's kview."
     ;; Ensure that display is narrowed to cell region only.
     (kfile:narrow-to-kcells)
     (goto-char (point-min))
-    (if empty-p
+    (if empty-flag
 	;; This is a new koutline file.  Always need at least one visible
 	;; cell within a view. Insert initial empty cell.
 	(progn (kview:add-cell "1" 1)
@@ -155,15 +156,16 @@ Return file's kview."
 
     view))
 
-(defun kfile:read (buffer existing-file-p &optional ver-string)
-  "Create a new kotl view by reading BUFFER or create an empty view when EXISTING-FILE-P is nil.
-Optional VER-STRING is the outline format version number for the BUFFER that
-was previously read by calling `kfile:is-p'.
+(defun kfile:read (buffer existing-file-flag &optional ver-string)
+  "Create a new kotl view by reading BUFFER.
+Create an empty view when EXISTING-FILE-FLAG is nil.  Optional
+VER-STRING is the outline format version number for the BUFFER
+that was previously read by calling `kfile:is-p'.
 
 Return the new view."
   (cond ((not (bufferp buffer))
 	 (error "(kfile:read): Argument must be a buffer, `%s'" buffer))
-	((and (zerop (buffer-size)) (not existing-file-p))
+	((and (zerop (buffer-size)) (not existing-file-flag))
 	 (kfile:create buffer))
 	((progn
 	   (set-buffer buffer)
@@ -257,12 +259,12 @@ If V3-FLAG is true, read as a version-3 buffer."
   "Update kfile internal structure so that view is ready for saving to a file.
 Leave outline file expanded with structure data showing unless optional
 VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
-  (let* ((top (kview:top-cell kview))
+  (let* ((top (kview:top-cell kotl-kview))
 	 (file buffer-file-name)
-	 (label-type (kview:label-type kview))
-	 (label-min-width (kview:label-min-width kview))
-	 (label-separator (kview:label-separator kview))
-	 (level-indent (kview:level-indent kview))
+	 (label-type (kview:label-type kotl-kview))
+	 (label-min-width (kview:label-min-width kotl-kview))
+	 (label-separator (kview:label-separator kotl-kview))
+	 (level-indent (kview:level-indent kotl-kview))
 	 ;; If this happens to be non-nil, it is virtually impossible to save
 	 ;; a file, so ensure it is nil.
 	 (debug-on-error))
@@ -287,7 +289,7 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
 		kcell-num
 		(kcell-data:create cell (kcell-view:idstamp-integer)))
 	  (setq kcell-num (1+ kcell-num)))
-	kview t)
+	kotl-kview t)
       ;; Save top cell, 0, last since above loop may increment the total
       ;; number of cells counter stored in it, if any invalid cells are
       ;; encountered.
@@ -349,7 +351,7 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
   (set-buffer-modified-p t)
   ;; This next line must come before the save-buffer so write-file-functions
   ;; can make use of it.
-  (kview:set-buffer kview (current-buffer))
+  (kview:set-buffer kotl-kview (current-buffer))
   (save-buffer))
 
 ;;; ************************************************************************
@@ -395,7 +397,7 @@ hidden."
     (while
 	(progn
 	  (skip-chars-forward "\n")
-	  ;; !!! Won't work if label-type is 'no.
+	  ;; !! TODO: Won't work if label-type is 'no.
 	  ;; Here we search past the cell identifier
 	  ;; for the location at which to place cell properties.
 	  ;; Be sure not to skip past a period which may terminate the label.
@@ -403,8 +405,8 @@ hidden."
 	    (setq kcell-data (car kcell-list)
 		  ;; Repair invalid idstamps on the fly.
 		  idstamp (if (vectorp kcell-data)
-			      (or (kcell-data:idstamp kcell-data) (kview:id-increment kview))
-			    (kview:id-increment kview)))
+			      (or (kcell-data:idstamp kcell-data) (kview:id-increment kotl-kview))
+			    (kview:id-increment kotl-kview)))
 	    (kproperty:set 'idstamp idstamp)
 	    (kproperty:set 'kcell (car kcell-list))
 	    (setq kcell-list (cdr kcell-list)))
@@ -421,7 +423,7 @@ hidden."
     (while
 	(progn
 	  (skip-chars-forward "\n")
-	  ;; !! Todo: Won't work if label-type is 'no.
+	  ;; !! TODO: Won't work if label-type is 'no.
 	  ;; Here we search past the cell identifier
 	  ;; for the location at which to place cell properties.
 	  ;; Be sure not to skip past a period which may terminate the label.
@@ -429,8 +431,8 @@ hidden."
 	    (setq kcell-data (aref kcell-vector kcell-num)
 		  ;; Repair invalid idstamps on the fly.
 		  idstamp (if (vectorp kcell-data)
-			      (or (kcell-data:idstamp kcell-data) (kview:id-increment kview))
-			    (kview:id-increment kview)))
+			      (or (kcell-data:idstamp kcell-data) (kview:id-increment kotl-kview))
+			    (kview:id-increment kotl-kview)))
 	    (kproperty:set 'idstamp idstamp)
 	    (kproperty:set 'kcell (kcell-data:to-kcell-v3 kcell-data))
 	    (setq kcell-num (1+ kcell-num)))
@@ -439,7 +441,7 @@ hidden."
 (defun kfile:narrow-to-kcells ()
   "Narrow kotl file to kcell section only."
   (interactive)
-  (when (kview:is-p kview)
+  (when (kview:is-p kotl-kview)
     (let ((start-text) (end-text))
       (save-excursion
 	(widen)
@@ -470,7 +472,7 @@ handle, whenever this is possible."
 	    (let ((print-escape-newlines kfile:escape-newlines))
 	      (prin1 object (current-buffer)))
 	    (goto-char (point-min))
-	    (while (not (eobp))
+	    (while (not (smart-eobp))
 	      ;; (message "%06d" (- (point-max) (point)))
 	      (cond
 	       ((looking-at "\\s\(")
@@ -494,9 +496,7 @@ handle, whenever this is possible."
 		    (delete-region (match-beginning 0) (match-end 0))
 		  (error "Malformed quote"))
 		(backward-sexp 1))
-	       ((condition-case ()
-		    (prog1 t (down-list 1))
-		  (error nil))
+	       ((ignore-errors (prog1 t (down-list 1)))
 		(backward-char 1)
 		(skip-chars-backward " \t")
 		(delete-region
@@ -504,9 +504,7 @@ handle, whenever this is possible."
 		 (progn (skip-chars-forward " \t") (point)))
 		(when (not (eq ?' (char-after (1- (point)))))
 		  (insert ?\n)))
-	       ((condition-case ()
-		    (prog1 t (up-list 1))
-		  (error nil))
+	       ((ignore-errors (prog1 t (up-list 1)))
 		(while (looking-at "\\s)")
 		  (forward-char 1))
 		(skip-chars-backward " \t")
