@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    15-Nov-93 at 11:57:05
-;; Last-Mod:     11-Apr-22 at 23:51:03 by Bob Weiner
+;; Last-Mod:     25-Feb-24 at 18:03:16 by Mats Lidell
 ;;
-;; Copyright (C) 1993-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1993-2022  Free Software Foundation, Inc.
 ;; See the "../HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -19,7 +21,7 @@
 
 (require 'kfile)
 (require 'hyrolo)
-(require 'kotl-mode)
+(require 'klabel)
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -33,12 +35,12 @@
 This determines the type of importation done on a file when `kimport:file' is
 called if the major mode of the import file matches the car of an element in
 this list.  If there is no match, then `kimport:suffix-alist' is checked.  If
-that yields no match, the element in this list whose car is 't is used.  It
+that yields no match, the element in this list whose car is t is used.  It
 normally does an import of a koutline or text file.
 
 Each importation-function must take two arguments, a buffer/file to import
 and a buffer/file into which to insert the imported elements and a third
-optional argument, CHILDREN-P, which when non-nil means insert imported cells
+optional argument, CHILDREN-FLAG, which when non-nil means insert imported cells
 as the initial set of children of the current cell, if any.
 
    `outline-mode'  - imported as an Emacs outline whose entries begin with
@@ -50,33 +52,34 @@ as the initial set of children of the current cell, if any.
 
 ;;;###autoload
 (defvar kimport:suffix-alist
-  '(("\\.org$" . kimport:star-outline)
-    ("\\.otl$" . kimport:star-outline)
-    ("\\.aug$" . kimport:aug-post-outline))
-  "Alist of (buffer-name-suffix-regexp . importation-function) elements.
+  '((".org" . kimport:star-outline)
+    (".otl" . kimport:star-outline)
+    (".aug" . kimport:aug-post-outline))
+  "Alist of (buffer-name-suffix . importation-function) elements.
 This determines the type of importation done on a file when `kimport:file' is
 called.  Each importation-function must take two arguments, a buffer/file to
 import and a buffer/file into which to insert the imported elements.
-A third optional argument, CHILDREN-P, may be given; when non-nil, insert
+A third optional argument, CHILDREN-FLAG, may be given; when non-nil, insert
 imported cells as the initial set of children of the current cell, if any.
 
    .org  - import format is an Emacs outline whose entries begin with asterisks;
    .otl  - import format is an Emacs outline whose entries begin with asterisks;
    .kot
-   .kotl - import format is a Koutline
+   .kotl - import format is a Koutline;
    .aug  - import format is an Augment post-numbered outline
            (see https://dougengelbart.org/content/view/148/).")
 
-(defconst kimport:star-heading "^\\(\\*+\\)"
-  "Regular expression matching a star outline heading with the number of stars given by groupoing 1.")
+(defconst kimport:star-heading "^[ \t]*\\(\\*+\\)"
+  "Regular expression matching a star outline heading.
+The number of stars is given by grouping 1.")
 
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
 
 ;;;###autoload
-(defun kimport:file (import-from output-to &optional children-p)
-  "Import a buffer or file IMPORT-FROM into the koutline in buffer or file OUTPUT-TO.
+(defun kimport:file (import-from output-to &optional children-flag)
+  "Import a buffer or file IMPORT-FROM into a koutline in buffer or file OUTPUT-TO.
 
 Any suffix in IMPORT-FROM's buffer name is used to determine the type of
 importation.  All others are imported as text, one paragraph per cell.
@@ -96,21 +99,15 @@ information on specific importation formats."
 			 import-from))))
 	(function))
     (set-buffer import-buf-name)
-    (if (setq function (cdr (assq major-mode kimport:mode-alist)))
-	nil
-      (let ((import-suffix (if (string-match "\\..+\\'" import-buf-name)
-			       (match-string 0 import-buf-name)))
-	    (suffix-alist kimport:suffix-alist)
-	    suffix-regexp)
-	(while (and import-suffix suffix-alist)
-	  (setq suffix-regexp (car (car suffix-alist))
-		function (cdr (car suffix-alist))
-		suffix-alist (cdr suffix-alist))
-	  (if (string-match suffix-regexp import-suffix)
-	      nil
-	    (setq function nil)))
-	(if function nil (setq function (cdr (assq t kimport:mode-alist))))))
-    (funcall function import-from output-to children-p)))
+    (unless (setq function (cdr (assq major-mode kimport:mode-alist)))
+      (let ((import-suffix (when (string-match "\\.[^.]+\\'" import-buf-name)
+			     (match-string 0 import-buf-name))))
+	(setq function (or (cdr (assoc import-suffix kimport:suffix-alist))
+			   (cdr (assq t kimport:mode-alist))))))
+    (when hkey-debug
+      (message "(kimport:file): Buffer: %s; Import-Function: %s"
+	       import-buf-name function))
+    (funcall function import-from output-to children-flag)))
 
 (defun kimport:insert-buffer (buffer)
   "Insert after point the contents of BUFFER.
@@ -121,10 +118,10 @@ BUFFER may be a buffer or a buffer name."
   (kotl-mode:add-indent-to-region))
 
 ;;;###autoload
-(defun kimport:insert-file (import-from children-p)
+(defun kimport:insert-file (import-from children-flag)
   "Insert each element in IMPORT-FROM as a separate cell in the current view.
 Insert as sibling cells following the current cell unless prefix arg,
-CHILDREN-P is non-nil, then insert as the initial children of the current
+CHILDREN-FLAG is non-nil, then insert as the initial children of the current
 cell.
 
 IMPORT-FROM may be a buffer name or file name (file name completion is
@@ -138,7 +135,7 @@ importation is determined."
 	      "Buffer or file to insert as children of current cell: "
 	    "Buffer or file to insert as siblings of current cell: "))
 	 current-prefix-arg))
-  (kimport:file import-from (current-buffer) children-p))
+  (kimport:file import-from (current-buffer) children-flag))
 
 (defun kimport:insert-file-contents (filename)
   "Insert contents of file FILENAME into current cell after point.
@@ -177,7 +174,7 @@ Interactively, second arg is non-nil if prefix ARG is supplied."
 ;;;
 
 ;;;###autoload
-(defun kimport:aug-post-outline (import-from output-to &optional children-p)
+(defun kimport:aug-post-outline (import-from output-to &optional children-flag)
   "Insert Augment outline statements from IMPORT-FROM into koutline OUTPUT-TO.
 Displays and leaves point in OUTPUT-TO.  See documentation for
 `kimport:initialize' for valid values of IMPORT-FROM and OUTPUT-TO and for
@@ -190,20 +187,20 @@ Each statement to be imported is delimited by an Augment relative id at the
 end of the statement.  \"1\" = level 1, \"1a\" = level 2 in outline and so
 on."
   (interactive "FImport from Augment post-numbered buffer/file: \nFBuffer/file to insert cells into: \nP")
-  (let ((output-level 1) (klabel "1")
-	initially-empty-output no-renumber orig-point count total)
+  (let ((output-level 1) (klabel "1") (count 0)
+	max-pos-and-count initially-empty-output no-renumber orig-point total)
     ;; Don't change the order of import-from and output-to inits here.
     (setq import-from (kimport:copy-and-set-buffer import-from)
-	  output-to (kimport:initialize output-to)
+	  output-to (kimport:initialize output-to t)
 	  orig-point (point)
 	  initially-empty-output (zerop (- (point-max) (point-min)))
 	  no-renumber (or initially-empty-output
-			  (not (if children-p
+			  (not (if children-flag
 				   (kcell-view:child-p)
 				 (kcell-view:sibling-p)))))
 
-    (if (eq import-from output-to)
-	(error "(kimport:aug-post-outline): Import and output buffers may not be the same."))
+    (when (eq import-from output-to)
+      (error "(kimport:aug-post-outline): Import and output buffers may not be the same."))
 
     (set-buffer import-from)
     (outline-show-all)
@@ -211,31 +208,37 @@ on."
       (goto-char (point-min))
       ;; Total number of Augment statements.
       (setq total (count-matches " +\\([0-9][0-9a-z]*\\)\n\\(\n\\|\\'\\)"))
-      (if initially-empty-output
-	  nil
+      (unless initially-empty-output
 	;; Insert first cell as sibling of current cell.
 	(set-buffer output-to)
-	(if children-p
-	    ;; Insert as children.
-	    (progn (setq klabel (klabel:child (kcell-view:label))
-			 output-level (klabel:level klabel))
-		   ;; Move to end of this cell since cell insertion will
-		   ;; occur at point.
-		   (goto-char (kcell-view:end)))
-	;; Insert as successors.
-	(setq klabel (klabel:increment (kcell-view:label))
-	      output-level (klabel:level klabel))
-	;; Move to start of line of next tree since cell insertion will occur
-	;; at point.
-	(goto-char (kotl-mode:tree-end))))
-      (setq count (kimport:aug-post-statements
-		   import-from output-to klabel output-level 1 0 total)))
+	(cond (children-flag
+	       ;; Insert as children.
+	       (setq klabel (klabel:child (kcell-view:label))
+		     output-level (klabel:level klabel))
+	       ;; Move to end of this cell since cell insertion will
+	       ;; occur at point.
+	       (goto-char (kcell-view:end)))
+	      ((string-empty-p (kcell-view:contents))
+	       ;; This is an unused cell, fill in from here.
+	       (setq klabel (kcell-view:label)
+		     output-level (klabel:level klabel))
+	       (goto-char (kcell-view:start)))
+	      (t ;; Insert as successors.
+	       (setq klabel (klabel:increment (kcell-view:label))
+		     output-level (klabel:level klabel))
+	       ;; Move to start of line of next tree since cell insertion
+	       ;; will occur at point.
+	       (goto-char (kotl-mode:tree-end)))))
+      (setq max-pos-and-count (kimport:aug-post-statements
+			       import-from output-to klabel
+			       output-level 1 count total 0)
+	    count (cdr max-pos-and-count)))
     (pop-to-buffer output-to)
     (kfile:narrow-to-kcells)
-    (if no-renumber nil (klabel-type:update-labels klabel))
+    (unless no-renumber
+      (klabel-type:update-labels klabel))
     (goto-char orig-point)
-    (if (kotl-mode:buffer-empty-p)
-	nil
+    (unless (kotl-mode:buffer-empty-p)
       (kotl-mode:to-valid-position))
     (message "Imported %d of %d Augment statements." count total)))
 
@@ -244,28 +247,34 @@ on."
 ;;;
 
 ;;;###autoload
-(defun kimport:star-outline (import-from output-to &optional children-p)
+(defun kimport:star-outline (import-from output-to &optional children-flag)
   "Insert star outline nodes from IMPORT-FROM into koutline OUTPUT-TO.
-Displays and leaves point in OUTPUT-TO.  See documentation for
-`kimport:initialize' for valid values of IMPORT-FROM and OUTPUT-TO and for
-an explanation of where imported cells are placed.
+Display and leave point in OUTPUT-TO.  See documentation for
+`kimport:initialize' for valid values of IMPORT-FROM and
+OUTPUT-TO and for an explanation of where imported cells are
+placed.
 
 \"* \" = level 1, \"** \" = level 2 in outline and so on."
-  (interactive "FImport from star delimited cells buffer/file: \nFBuffer/file to insert cells into: \nP")
-  (let ((output-level 1) (klabel "1")
-	initially-empty-output no-renumber orig-point count total)
+  (interactive "fImport from star delimited cells buffer/file: \nFBuffer/file to insert cells into: \nP")
+  (when (and buffer-file-name (directory-name-p import-from))
+    (setq import-from buffer-file-name))
+  (when (and buffer-file-name (directory-name-p output-to))
+    (setq output-to (concat (file-name-sans-extension buffer-file-name) ".kotl")))
+  (let ((output-level 1) (klabel "1") (count 0)
+	max-pos-and-count initially-empty-output no-renumber orig-point total)
     ;; Don't change the order of import-from and output-to inits here.
     (setq import-from (kimport:copy-and-set-buffer import-from)
-	  output-to (kimport:initialize output-to)
+	  ;; Set current buffer to output-to
+	  output-to (kimport:initialize output-to t)
 	  orig-point (point)
 	  initially-empty-output (zerop (- (point-max) (point-min)))
 	  no-renumber (or initially-empty-output
-			  (not (if children-p
+			  (not (if children-flag
 				   (kcell-view:child-p)
 				 (kcell-view:sibling-p)))))
 
-    (if (eq import-from output-to)
-	(error "(kimport:star-outline): Import and output buffers may not be the same."))
+    (when (eq import-from output-to)
+      (error "(kimport:star-outline): Import and output buffers may not be the same."))
 
     (set-buffer import-from)
     (outline-show-all)
@@ -274,35 +283,40 @@ an explanation of where imported cells are placed.
       ;; If initial text in buffer is not a star outline node, add a star to
       ;; make it one, so it is not deleted from the import.
       (unless (looking-at kimport:star-heading)
-	  (insert "* "))
+	(insert "* "))
       (goto-char (point-min))
       ;; Total number of top-level cells.
-      (setq total (count-matches (concat kimport:star-heading "[ \t\n\r]")))
-      (if initially-empty-output
-	  nil
+      (setq total (count-matches "^[ \t]*\\*\\s-"))
+      (unless initially-empty-output
 	;; Insert first cell as sibling of current cell.
 	(set-buffer output-to)
-	(if children-p
-	    ;; Insert as children.
-	    (progn (setq klabel (klabel:child (kcell-view:label))
-			 output-level (klabel:level klabel))
-		   ;; Move to end of this cell since cell insertion will
-		   ;; occur at point.
-		   (goto-char (kcell-view:end)))
-	;; Insert as successors.
-	(setq klabel (klabel:increment (kcell-view:label))
-	      output-level (klabel:level klabel))
-	;; Move to start of line of next tree since cell insertion will occur
-	;; at point.
-	(goto-char (kotl-mode:tree-end))))
-      (setq count (kimport:star-entries
-		   import-from output-to klabel output-level 1 0 total)))
+	(cond (children-flag
+	       ;; Insert as children.
+	       (setq klabel (klabel:child (kcell-view:label))
+		     output-level (klabel:level klabel))
+	       ;; Move to end of this cell since cell insertion will
+	       ;; occur at point.
+	       (goto-char (kcell-view:end)))
+	      ((string-empty-p (kcell-view:contents))
+	       ;; This is an unused cell, fill in from here.
+	       (setq klabel (kcell-view:label)
+		     output-level (klabel:level klabel))
+	       (goto-char (kcell-view:start)))
+	      (t ;; Insert as successors.
+	       (setq klabel (klabel:increment (kcell-view:label))
+		     output-level (klabel:level klabel))
+	       ;; Move to start of line of next tree since cell insertion
+	       ;; will occur at point.
+	       (goto-char (kotl-mode:tree-end)))))
+      (setq max-pos-and-count (kimport:star-entries
+			       import-from output-to klabel output-level 1 count total 0)
+	    count (cdr max-pos-and-count)))
     (pop-to-buffer output-to)
     (kfile:narrow-to-kcells)
-    (if no-renumber nil (klabel-type:update-labels klabel))
+    (unless no-renumber
+      (klabel-type:update-labels klabel))
     (goto-char orig-point)
-    (if (kotl-mode:buffer-empty-p)
-	nil
+    (unless (kotl-mode:buffer-empty-p)
       (kotl-mode:to-valid-position))
     (message "Imported %d of %d star outline trees." count total)))
 
@@ -311,81 +325,89 @@ an explanation of where imported cells are placed.
 ;;;
 
 ;;;###autoload
-(defun kimport:text (import-from output-to &optional children-p)
+(defun kimport:text (import-from output-to &optional children-flag)
   "Insert text paragraphs from IMPORT-FROM into koutline OUTPUT-TO.
-Displays and leaves point in OUTPUT-TO.  See documentation for
-`kimport:initialize' for valid values of IMPORT-FROM and OUTPUT-TO and for
-an explanation of where imported cells are placed.
+Display and leave point in OUTPUT-TO.  See documentation for
+`kimport:initialize' for valid values of IMPORT-FROM and
+OUTPUT-TO and for an explanation of where imported cells are
+placed.
 
-Text paragraphs are imported as a sequence of same level cells.  Koutlines
-are imported with their structure intact.
-
-The variable, `paragraph-start,' is used to determine paragraphs."
+Import Koutlines with their structure intact.  Import text
+paragraphs as a sequence of same level cells.  The variable,
+`paragraph-start,' is used to determine paragraphs."
   (interactive "FImport from text/koutline buffer/file: \nFInsert cells into koutline buffer/file: \nP")
-  (let ((klabel "1") (output-level 1) (count 0) initially-empty-output
-	no-renumber orig-point total)
+  (when (and buffer-file-name (directory-name-p import-from))
+    (setq import-from buffer-file-name))
+  (when (and buffer-file-name (directory-name-p output-to))
+    (setq output-to (concat (file-name-sans-extension buffer-file-name) ".kotl")))
+  (let ((output-level 1) (klabel "1") (count 0)
+        initially-empty-output no-renumber orig-point total)
     ;; Don't change the order of import-from and output-to inits here.
     (setq import-from (kimport:copy-and-set-buffer import-from)
-	  output-to (kimport:initialize output-to)
+	  ;; Set current buffer to output-to
+	  output-to (kimport:initialize output-to t)
 	  orig-point (point)
 	  initially-empty-output (zerop (- (point-max) (point-min)))
 	  no-renumber (or initially-empty-output
-			  (not (if children-p
+			  (not (if children-flag
 				   (kcell-view:child-p)
 				 (kcell-view:sibling-p)))))
 
-    (if (eq import-from output-to)
-	(error "(kimport:text): Import and output buffers may not be the same."))
+    (when (eq import-from output-to)
+      (error "(kimport:text): Import and output buffers may not be the same."))
 
     (set-buffer import-from)
-    (let ((kotl-import (eq major-mode 'kotl-mode))
+    (let ((kotl-import (derived-mode-p 'kotl-mode))
 	  visible-cells)
       (save-excursion
-	(if initially-empty-output
-	    nil
+	(goto-char (point-min))
+	(unless initially-empty-output
 	  ;; Insert first cell as sibling of current cell.
 	  (set-buffer output-to)
-	  (if children-p
-	      ;; Insert as children.
-	      (progn (setq klabel (klabel:child (kcell-view:label))
-			   output-level (klabel:level klabel))
-		     ;; Move to end of this cell since cell insertion will
-		     ;; occur at point.
-		     (goto-char (kcell-view:end)))
-	    ;; Insert as successors.
-	    (setq klabel (klabel:increment (kcell-view:label))
-		  output-level (klabel:level klabel))
-	    ;; Move to start of line of next tree since cell insertion will occur
-	    ;; at point.
-	    (goto-char (kotl-mode:tree-end)))
-	  (set-buffer import-from))
+	  (cond (children-flag
+		 ;; Insert as children.
+		 (setq klabel (klabel:child (kcell-view:label))
+		       output-level (klabel:level klabel))
+		 ;; Move to end of this cell since cell insertion will
+		 ;; occur at point.
+		 (goto-char (kcell-view:end)))
+		((string-empty-p (kcell-view:contents))
+		 ;; This is an unused cell, fill in from here.
+		 (setq klabel (kcell-view:label)
+		       output-level (klabel:level klabel))
+		 (goto-char (kcell-view:start)))
+		(t ;; Insert as successors.
+		 (setq klabel (klabel:increment (kcell-view:label))
+		       output-level (klabel:level klabel))
+		 ;; Move to start of line of next tree since cell insertion
+		 ;; will occur at point.
+		 (goto-char (kotl-mode:tree-end))))))
 
-	(if kotl-import
-	    ;; Importing from a koutline, so handle specially.
-	    (progn (kotl-mode:beginning-of-buffer)
-		   ;; Total number of cells.
-		   (setq total (count-matches "[\n\r][\n\r]")
-			 visible-cells (count-matches "\n\n")
-			 count (save-excursion
-				 ;; Incredible non-local exit to ensure that
-				 ;; recursion ends at the right time.
-				 (catch 'end
-				   (kimport:kcells import-from output-to klabel
-						   output-level 1
-						   count total)))))
-
-	  (outline-show-all)
-	  (goto-char (point-min))
-	  ;; Total number of paragraphs.
-	  (setq total (kimport:count-paragraphs)
-		count (kimport:text-paragraphs import-from output-to klabel
-					       output-level count total))))
+      (if kotl-import
+	  ;; Importing from a koutline, so handle specially.
+	  (progn (kotl-mode:beginning-of-buffer)
+		 ;; Total number of cells.
+		 (setq total (count-matches "[\n\r][\n\r]")
+		       visible-cells (count-matches "\n\n")
+		       count (save-excursion
+			       ;; Incredible non-local exit to ensure that
+			       ;; recursion ends at the right time.
+			       (catch 'end
+				 (kimport:kcells import-from output-to klabel
+						 output-level 1
+						 count total)))))
+	(outline-show-all)
+	(goto-char (point-min))
+	;; Total number of paragraphs.
+	(setq total (kimport:count-paragraphs)
+	      count (kimport:text-paragraphs import-from output-to klabel
+					     output-level count total)))
       (pop-to-buffer output-to)
       (kfile:narrow-to-kcells)
-      (if no-renumber nil (klabel-type:update-labels klabel))
+      (unless no-renumber
+	(klabel-type:update-labels klabel))
       (goto-char orig-point)
-      (if (kotl-mode:buffer-empty-p)
-	  nil
+      (unless (kotl-mode:buffer-empty-p)
 	(kotl-mode:to-valid-position))
       (if kotl-import
 	  (message "Imported %d of %d visible cells from a %d cell outline."
@@ -429,28 +451,34 @@ The variable, `paragraph-start,' is used to determine paragraphs."
 	  (t nil))))
 
 (defun kimport:aug-post-statements (import-from output-to klabel output-level
- 			            import-level count total)
-  "Insert post-numbered Augment statements (contents only) from IMPORT-FROM into existing OUTPUT-TO.
+ 			            import-level count total max-pos)
+  "Insert post-numbered Augment statements (contents only) from IMPORT-FROM.
+Is inserted in existing OUTPUT-TO.
 
 KLABEL is the label to use for the first imported statement.
 OUTPUT-LEVEL is the level at which to insert the first statement.
-IMPORT-LEVEL is the depth of the current statement in the import file,
-\(initially 1).
+IMPORT-LEVEL is the depth of the current statement in the import
+file, \(initially 1).
 
-COUNT of inserted cells starts at 0.  TOTAL is the total number of statements
-in IMPORT-FROM, used to show a running tally of the imported statements."
+COUNT of inserted cells starts at 0.  TOTAL is the total number
+of statements in IMPORT-FROM, used to show a running tally of the
+imported statements.  MAX-POS is the furthest position searched
+in IMPORT-FROM so far (initially 0).
+
+Return a cons of MAX-POS and COUNT."
   (set-buffer import-from)
-  (let ((cell-end-regexp " +\\([0-9][0-9a-z]*\\)\n\\(\n+\\|\\'\\)")
-	contents start subtree-p end end-contents statement-level
+  (let ((start (point))
+	(cell-end-regexp " +\\([0-9][0-9a-z]*\\)\n\\(\n+\\|\\'\\)")
+	(case-fold-search)
+	max-pos-and-count
+	contents subtree-p end end-contents statement-level
 	child-label)
     ;; While find cells at import-level or deeper ...
     (while (and (setq start (point))
 		(re-search-forward cell-end-regexp nil t)
 		(<= import-level
 		   (setq statement-level
-			 (klabel:level-alpha
-			  (buffer-substring
-			   (match-beginning 1) (match-end 1))))))
+			 (klabel:level-alpha (match-string 1)))))
       (setq end-contents (match-beginning 0)
 	    end (match-end 0))
       (goto-char start)
@@ -460,35 +488,43 @@ in IMPORT-FROM, used to show a running tally of the imported statements."
       (setq subtree-p (save-excursion
 			(if (re-search-forward cell-end-regexp nil t)
 			    (< statement-level
-			       (klabel:level-alpha
-				(buffer-substring
-				 (match-beginning 1) (match-end 1)))))))
+			       (klabel:level-alpha (match-string 1))))))
       (with-current-buffer output-to
-	;; Add the cell starting at point.
-	(kview:add-cell klabel output-level contents nil t)
+	(if (and (zerop count) (string-empty-p (kcell-view:contents)))
+	    ;; Reuse this initial empty cell in koutline
+	    (progn (kview:insert-contents (kcell-view:cell) contents 'no-fill
+					  (make-string (kcell-view:indent) ?\ ))
+		   (goto-char (kcell-view:end)))
+	  ;; Add the cell starting at point.
+	  (kview:add-cell klabel output-level contents nil t))
 	(if subtree-p (setq child-label (klabel:child klabel)))
 	(message "%d of %d statements converted..."
-		 (setq count (1+ count)) total)
+		 (if (= statement-level 1) (setq count (1+ count)) count)
+		 total)
 	(setq klabel (klabel:increment klabel)))
       ;;
       ;; Current buffer returns to `import-from' here.
       ;; Handle each sub-level through recursion.
-      (if subtree-p
-	  ;; Subtree exists so insert its cells.
-	  (setq count
-		(kimport:aug-post-statements
-		 import-from output-to child-label (1+ output-level)
-		 (1+ import-level) count total))))
-    (goto-char start))
-  count)
+      (when subtree-p
+	;; Subtree exists so insert its cells.
+	(setq max-pos-and-count
+	      (kimport:aug-post-statements
+	       import-from output-to child-label (1+ output-level)
+	       (1+ import-level) count total max-pos)
+	      max-pos (car max-pos-and-count)
+	      count (cdr max-pos-and-count))))
+    (goto-char (setq max-pos (max end max-pos))))
+  (cons max-pos count))
 
 ;;;###autoload
 (defun kimport:copy-and-set-buffer (source)
-  "Copy and untabify SOURCE, set copy buffer as current buffer for this command and return the copy buffer.
-SOURCE may be a buffer name, a buffer or a file name.
-If SOURCE buffer name begins with a space, it is not copied under the
-assumption that it already has been.  If SOURCE is a koutline, it is not
-copied since there is no need to copy it to import it."
+  "Copy and untabify SOURCE.
+Set copy buffer as current buffer for this command and return the
+copy buffer.  SOURCE may be a buffer name, a buffer or a file
+name.  If SOURCE buffer name begins with a space, it is not
+copied under the assumption that it already has been.  If SOURCE
+is a koutline, it is not copied since there is no need to copy it
+to import it."
   (setq source (set-buffer (or (get-buffer source)
 			       (find-file-noselect source))))
   (let ((mode (or (if (boundp 'kotl-previous-mode) kotl-previous-mode)
@@ -517,15 +553,19 @@ copied since there is no need to copy it to import it."
       ;; Ensure buffer ends with a newline so that we don't miss the last
       ;; element during the import.
       (goto-char (point-max))
-      (if (not (eq (preceding-char) ?\n)) (insert "\n"))
+      (unless (eq (preceding-char) ?\n)
+	(insert "\n"))
       (set-buffer-modified-p nil)
       copy)))
 
-(defun kimport:initialize (output-to)
+(defun kimport:initialize (output-to &optional erase-flag)
   "Setup to import elements into koutline OUTPUT-TO.
 Return OUTPUT-TO buffer and set current buffer for the current command
 to OUTPUT-TO.
 
+With optional ERASE-FLAG non-nil, erase OUTPUT-TO before importing
+elements (don't append them).
+
 OUTPUT-TO may be a buffer, `buffer-name' or file name.  If OUTPUT-TO exists
 already, it must be a koutline or an error will be signaled.  For an existing
 OUTPUT-TO, the text cells are inserted after the cell at point or after the
@@ -538,44 +578,46 @@ OUTPUT-TO within this function, point is left at the end of this buffer so
 that imported cells will be appended to the buffer.  For a new file, this
 means the first cell imported will become the first outline cell.
 
-If a non-nil third argument, CHILDREN-P, is given to the caller of this
+If a non-nil third argument, CHILDREN-FLAG, is given to the caller of this
 function and OUTPUT-TO contains at least one cell, then the imported cells
 will be added as children of the cell where this function leaves point
 \(either the current cell or for a newly read in outline, the last cell)."
   (let* ((output-existing-buffer-p
-	  (if output-to
-	     (or (get-buffer output-to) (get-file-buffer output-to))))
+	  (when output-to
+	    (or (get-buffer output-to) (get-file-buffer output-to))))
 	 (output-exists-p
 	  (if output-to
-	     (or output-existing-buffer-p (file-exists-p output-to))
-	   ;; current buffer will be used for output and it exists.
-	   t)))
+	      (or output-existing-buffer-p (file-exists-p output-to))
+	    ;; current buffer will be used for output and it exists.
+	    t)))
     (setq output-to (if output-to
 			(or (get-buffer output-to)
 			    (find-file-noselect output-to))
 		      (current-buffer)))
     (set-buffer output-to)
-    (if output-exists-p
+    (if (and output-exists-p (not erase-flag))
 	(if (eq major-mode 'kotl-mode)
-	    (if (kotl-mode:buffer-empty-p)
-		nil
+	    (unless (kotl-mode:buffer-empty-p)
 	      ;; Make imported cells be appended if the output buffer was
 	      ;; just read in.
-	      (if output-existing-buffer-p nil (goto-char (point-max)))
+	      (unless output-existing-buffer-p
+		(goto-char (point-max)))
 	      (kotl-mode:to-valid-position))
 	  (error
 	   "(kimport:initialize): Second arg, %s, must be a koutline file."
 	   (buffer-name output-to)))
-      (if (eq major-mode 'kotl-mode)
-	  nil
-	(setq kview nil)
-	(kotl-mode))
-      (delete-region (point-min) (point-max))))
+      (when erase-flag
+	(widen)
+	(delete-region (point-min) (point-max)))
+      (unless (kfile:is-p)
+	(setq kotl-kview nil)
+	(kotl-mode))))
   output-to)
 
 (defun kimport:kcells (import-from output-to klabel output-level
 		       import-level count total)
-  "Insert visible koutline cells (contents and attributes) from IMPORT-FROM into existing OUTPUT-TO.
+  "Insert visible koutline cells (contents and attributes) from IMPORT-FROM.
+Insert into existing OUTPUT-TO.
 
 KLABEL is the label to use for the first imported cell.
 OUTPUT-LEVEL is the level at which to insert the first cell.
@@ -583,7 +625,9 @@ IMPORT-LEVEL is the depth of the current cell in the import file,
 \(initially 1).
 
 COUNT of inserted cells starts at 0.  TOTAL is the total number of cells
-in IMPORT-FROM, used to show a running tally of the imported cells."
+in IMPORT-FROM, used to show a running tally of the imported cells.
+
+Return COUNT."
   (set-buffer import-from)
   (goto-char (kcell-view:start))
   (let ((again t) contents subtree-p child-label)
@@ -602,17 +646,18 @@ in IMPORT-FROM, used to show a running tally of the imported cells."
       ;;
       ;; Current buffer returns to `import-from' here.
       ;; Handle each sub-level through recursion.
-      (if (and (setq again (kcell-view:next t)) subtree-p)
-	  ;; Subtree exists so insert its cells.
-	  (setq count
-		(kimport:kcells
-		 import-from output-to child-label (1+ output-level)
-		 (1+ import-level) count total)))
-      (if again nil (throw 'end count))))
+      (when (and (setq again (kcell-view:next t)) subtree-p)
+	;; Subtree exists so insert its cells.
+	(setq count
+	      (kimport:kcells
+	       import-from output-to child-label (1+ output-level)
+	       (1+ import-level) count total)))
+      (unless again
+	(throw 'end count))))
   count)
 
 (defun kimport:star-entries (import-from output-to klabel output-level
-                             import-level count total)
+                             import-level count total max-pos)
   "Insert visible star outline entries from IMPORT-FROM into existing OUTPUT-TO.
 
 KLABEL is the label to use for the first imported entry.
@@ -620,31 +665,39 @@ OUTPUT-LEVEL is the level at which to insert the first entry.
 IMPORT-LEVEL is the depth of the current entry in the import file,
 \(initially 1).
 
-COUNT of inserted entries starts at 0.  TOTAL is the total number of entries
-in IMPORT-FROM, used to show a running tally of the imported entries."
+COUNT of inserted entries starts at 0.  TOTAL is the total number
+of entries in IMPORT-FROM, used to show a running tally of the
+imported entries.  MAX-POS is the furthest position searched in
+IMPORT-FROM so far (initially 0).
+
+Return a cons of MAX-POS and COUNT."
   (set-buffer import-from)
   (let ((start (point))
-	(hyrolo-entry-regexp kimport:star-heading)
+	(hyrolo-hdr-and-entry-regexp kimport:star-heading)
+	(case-fold-search)
+	max-pos-and-count
 	subtree-p end contents node-level child-label)
     ;; While find cells at import-level or deeper ...
-    (while (and (re-search-forward hyrolo-entry-regexp nil t)
+    (while (and (re-search-forward hyrolo-hdr-and-entry-regexp nil t)
 		(<= import-level
 		    (setq node-level
-			  (length
-			   (buffer-substring
-			    (match-beginning 1) (match-end 1))))))
+			  (length (match-string 1)))))
       (skip-chars-forward " \t")
       (setq start (point)
 	    end (hyrolo-to-entry-end)
-	    subtree-p (if (looking-at hyrolo-entry-regexp)
+	    subtree-p (if (looking-at hyrolo-hdr-and-entry-regexp)
 			  (< node-level
-			     (length (buffer-substring
-				      (match-beginning 1) (match-end 1))))))
+			     (length (match-string 1)))))
       (skip-chars-backward "\n\r")
       (setq contents (kimport:unindent-region start (point)))
       (with-current-buffer output-to
-	;; Add the cell starting at point.
-	(kview:add-cell klabel output-level contents nil t)
+	(if (and (zerop count) (string-empty-p (kcell-view:contents)))
+	    ;; Reuse this initial empty cell in koutline
+	    (progn (kview:insert-contents (kcell-view:cell) contents 'no-fill
+					  (make-string (kcell-view:indent) ?\ ))
+		   (goto-char (kcell-view:end)))
+	  ;; Add the cell starting at point.
+	  (kview:add-cell klabel output-level contents nil t))
 	(if subtree-p (setq child-label (klabel:child klabel)))
 	(message "%d of %d trees converted..."
 		 (if (= node-level 1) (setq count (1+ count)) count)
@@ -653,28 +706,33 @@ in IMPORT-FROM, used to show a running tally of the imported entries."
       ;;
       ;; Current buffer returns to `import-from' here.
       (goto-char end)
+      
       ;;
       ;; Handle each sub-level through recursion.
-      (if subtree-p
-	  ;; Subtree exists so insert its cells.
-	  (setq count
-		(kimport:star-entries import-from output-to child-label
-				      (1+ output-level) (1+ import-level)
-				      count total))))
-    (goto-char start))
-  count)
+      (when subtree-p
+	;; Subtree exists so insert its cells.
+	(setq max-pos-and-count
+	      (kimport:star-entries import-from output-to child-label
+				    (1+ output-level) (1+ import-level)
+				    count total max-pos)
+	      max-pos (car max-pos-and-count)
+	      count (cdr max-pos-and-count))))
+    (goto-char (setq max-pos (max end max-pos))))
+  (cons max-pos count))
 
 (defun kimport:text-paragraphs (import-from output-to klabel
-			        output-level _count total)
+			        output-level count total)
   "Insert text paragraphs from IMPORT-FROM into existing OUTPUT-TO.
 First cell is inserted with KLABEL at OUTPUT-LEVEL, as the sibling of the
 previous cell, with the COUNT of inserted paragraphs starting at 0.  TOTAL is
 the total number of paragraphs in IMPORT-FROM, used to show a running tally
 of the imported paragraphs.
 
-The variable, `paragraph-start' is used to determine paragraphs."
+The variable, `paragraph-start' is used to determine paragraphs.
+
+Return COUNT."
   (set-buffer import-from)
-  (let* ((count 0) start end contents)
+  (let* (start end contents)
     ;; Next line is needed when importing into an existing kview.
     (goto-char (point-min))
     ;; Move past blank lines at point.
@@ -688,13 +746,18 @@ The variable, `paragraph-start' is used to determine paragraphs."
 		    (setq end (goto-char (match-end 0))))
 		(/= start end))
       (setq contents (kimport:unindent-region start end))
-      (set-buffer output-to)
-      ;; Add the cell starting at point.
-      (kview:add-cell klabel output-level contents nil t)
-      (setq count (1+ count))
-      (message "%d of %d paragraphs converted..."
-	       count total)
-      (setq klabel (klabel:increment klabel))
+      (with-current-buffer output-to
+	(if (and (zerop count) (string-empty-p (kcell-view:contents)))
+	    ;; Reuse this initial empty cell in koutline
+	    (progn (kview:insert-contents (kcell-view:cell) contents 'no-fill
+					  (make-string (kcell-view:indent) ?\ ))
+		   (goto-char (kcell-view:end)))
+	  ;; Add the cell starting at point.
+	  (kview:add-cell klabel output-level contents nil t))
+	(setq count (1+ count))
+	(message "%d of %d paragraphs converted..."
+		 count total)
+	(setq klabel (klabel:increment klabel)))
       (set-buffer import-from)
       (goto-char end)
       ;; Move past blank lines separating paragraphs.
@@ -716,8 +779,8 @@ Remove the indent and return the remaining region as a string."
 	    (if (re-search-forward "[\n\r][ \t]+" end t)
 		(concat "^" (make-string (current-column) ?\ ))))
       (if indent-regexp
-	  (hypb:replace-match-string
-			  indent-regexp (buffer-substring start end) "" t)
+	  (replace-regexp-in-string
+			  indent-regexp "" (buffer-substring start end) nil t)
 	(buffer-substring start end)))))
 
 ;; Do this at the end so kotl-mode can utilize kimport definitions.
