@@ -3,9 +3,11 @@
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    17-Apr-94
-;; Last-Mod:     12-Feb-22 at 10:42:20 by Mats Lidell
+;; Last-Mod:      4-Oct-23 at 19:13:00 by Mats Lidell
 ;;
-;; Copyright (C) 1994-2021  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1994-2022  Free Software Foundation, Inc.
 ;; See the "../HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -13,6 +15,15 @@
 ;;; Commentary:
 
 ;;; Code:
+
+(require 'kview)
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function kotl-mode:first-cell-p "kotl-mode")
+
 ;;; ************************************************************************
 ;;; Public variables
 ;;; ************************************************************************
@@ -30,7 +41,7 @@
 
 (defun klabel:child (label)
   "Return LABEL's child cell label."
-  (funcall (kview:get-attr kview 'label-child) label))
+  (funcall (kview:get-attr kotl-kview 'label-child) label))
 
 (defun klabel:idstamp-p (label)
   "Return t if LABEL is an idstamp label, else nil."
@@ -41,11 +52,12 @@
 
 (defun klabel:increment (label)
   "Return LABEL's sibling label."
-  (funcall (kview:get-attr kview 'label-increment) label))
+  (funcall (kview:get-attr kotl-kview 'label-increment) label))
 
 (defun klabel:format (label)
-  "Format a generic cell LABEL (a string) and return it in the proper display type for the current kview."
-  (let ((label-type (or (kview:get-attr kview 'label-type) kview:default-label-type)))
+  "Format a generic cell LABEL (a string) and return the display type.
+Return the proper display type for the current kview."
+  (let ((label-type (or (kview:get-attr kotl-kview 'label-type) kview:default-label-type)))
     (cond ((memq label-type '(alpha id legal partial-alpha))
 	   label)
 	  ((eq label-type 'no) "")
@@ -55,7 +67,7 @@
 
 (defun klabel:level (label)
   "Return outline level of LABEL using current kview label type."
-  (let ((label-type (kview:label-type kview)))
+  (let ((label-type (kview:label-type kotl-kview)))
     (cond ((memq label-type '(alpha legal))
 	   (funcall (intern-soft (concat "klabel:level-"
 					 (symbol-name label-type)))
@@ -69,7 +81,7 @@
 
 (defun klabel:parent (label)
   "Return LABEL's parent label."
-  (funcall (kview:get-attr kview 'label-parent) label))
+  (funcall (kview:get-attr kotl-kview 'label-parent) label))
 
 ;;;
 ;;; klabel-type - kview-specific label type functions
@@ -84,14 +96,16 @@
 	((eq label-type 'star)
 	 (lambda (label) (concat label "*")))
 	((eq label-type 'id)
-	 (lambda (_label) (format "0%s" (or (kview:id-counter kview) ""))))
+	 (lambda (_label) (format "0%s" (or (kview:id-counter kotl-kview) ""))))
 	(t (error
 	    "(klabel-type:child): Invalid label type setting: `%s'"
 	    label-type))))
 
 (defun klabel-type:increment (label-type)
-  "Return a function that takes a single label argument and computes the next cell label of LABEL-TYPE.
-If the label is \"0\", its first child is computed, otherwise, the next sibling is computed."
+  "Return a function and compute the next cell label of LABEL-TYPE.
+The function takes a single label argument.  If the label is
+\"0\", its first child is computed, otherwise, the next sibling
+is computed."
   (cond ((memq label-type '(alpha legal partial-alpha))
 	 (intern-soft (concat "klabel:increment-" (symbol-name label-type))))
 	((eq label-type 'no)
@@ -99,7 +113,7 @@ If the label is \"0\", its first child is computed, otherwise, the next sibling
 	((eq label-type 'star)
 	 (lambda (label) (if (string-equal label "0") "*" label)))
 	((eq label-type 'id)
-	 (lambda (_label) (format "0%s" (or (kview:id-increment kview) ""))))
+	 (lambda (_label) (format "0%s" (or (kview:id-increment kotl-kview) ""))))
 	(t (error
 	    "(klabel:increment): Invalid label type setting: `%s'" label-type))))
 
@@ -255,12 +269,14 @@ First visible outline cell is level 1."
 ;;
 (defun klabel-type:function (&optional label-type)
   "Return function which will return display label for current cell.
-Label format is optional LABEL-TYPE or the default label type for the current view.
+Label format is optional LABEL-TYPE or the default label type for
+the current view.
 
-Function signature is: (func prev-label &optional child-p), where prev-label
-is the display label of the cell preceding the current one and child-p is
-non-nil if cell is to be the child of the preceding cell."
-  (or label-type (setq label-type (kview:label-type kview)))
+Function signature is: (func prev-label &optional child-p), where
+prev-label is the display label of the cell preceding the current
+one and child-p is non-nil if cell is to be the child of the
+preceding cell."
+  (or label-type (setq label-type (kview:label-type kotl-kview)))
   (cond ((eq label-type 'no)
 	 (lambda (_prev-label &optional _child-p)
 	   ""))
@@ -327,11 +343,11 @@ With optional CHILD-P, return label for first child cell of PREV-LABEL cell."
       (let ((klabel-type:changing-flag t))
 	(klabel-type:update-labels-from-point label-type first-label)))))
 
-(defun klabel-type:set-alpha (current-cell-label label-sep-len current-indent
+(defun klabel-type:set-alpha (current-cell-label lbl-sep-len current-indent
 			      per-level-indent &optional current-tree-only)
   "Set the labels of current cell, its following siblings and their subtrees.
 CURRENT-CELL-LABEL is the label to display for the current cell.
-LABEL-SEP-LEN is the length of the separation between a cell's label
+LBL-SEP-LEN is the length of the separation between a cell's label
 and the start of its contents."
   (let (label-prefix label-suffix suffix-val suffix-function opoint)
     (if current-cell-label
@@ -347,42 +363,42 @@ and the start of its contents."
 				'kotl-label:int-to-alpha)))
     (while current-cell-label
       ;; Set current cell's label.
-      (klabel:set current-cell-label label-sep-len)
+      (klabel:set current-cell-label lbl-sep-len)
       ;; Process any subtrees of current cell.
-      (if (kcell-view:child nil label-sep-len)
+      (if (kcell-view:child nil lbl-sep-len)
 	  ;; Recurse over subtree.
 	  (klabel-type:set-alpha
 	   (klabel:child-alpha current-cell-label)
-	   label-sep-len
+	   lbl-sep-len
 	   (+ current-indent per-level-indent)
 	   per-level-indent))
       ;; Process next sibling of current cell if any.
       (setq opoint (point))
       (if (and (not current-tree-only)
-	       (kcell-view:next nil label-sep-len)
-	       (< (abs (- (kcell-view:indent nil label-sep-len) current-indent))
-		  (kview:level-indent kview)))
+	       (kcell-view:next nil lbl-sep-len)
+	       (< (abs (- (kcell-view:indent nil lbl-sep-len) current-indent))
+		  (kview:level-indent kotl-kview)))
 	  (setq suffix-val (1+ suffix-val)
 		label-suffix (funcall suffix-function suffix-val)
 		current-cell-label (concat label-prefix label-suffix))
 	(goto-char opoint)
 	(setq current-cell-label nil)))))
 
-(defun klabel-type:set-id (_current-cell-label label-sep-len &rest _ignore)
+(defun klabel-type:set-id (_current-cell-label lbl-sep-len &rest _ignore)
   "Set the labels of current cell, its following siblings and their subtrees.
 CURRENT-CELL-LABEL is the label to display for the current cell."
   ;; Only need to do this when switching from one label type to another,
   ;; i.e. when every cell label will be updated.  So if not starting with the
   ;; first cell, do nothing.
   (if (kotl-mode:first-cell-p)
-      (while (and (klabel:set (kcell-view:idstamp) label-sep-len)
-		  (kcell-view:next nil label-sep-len)))))
+      (while (and (klabel:set (kcell-view:idstamp) lbl-sep-len)
+		  (kcell-view:next nil lbl-sep-len)))))
 
-(defun klabel-type:set-legal (current-cell-label label-sep-len current-indent
+(defun klabel-type:set-legal (current-cell-label lbl-sep-len current-indent
 			      per-level-indent &optional current-tree-only)
   "Set the labels of current cell, its following siblings and their subtrees.
 CURRENT-CELL-LABEL is the label to display for the current cell.
-LABEL-SEP-LEN is the length of the separation between a cell's label
+LBL-SEP-LEN is the length of the separation between a cell's label
 and the start of its contents."
   (let (label-prefix label-suffix suffix-val opoint)
     (if current-cell-label
@@ -392,43 +408,43 @@ and the start of its contents."
 	      suffix-val (string-to-number label-suffix)))
     (while current-cell-label
       ;; Set current cell's label.
-      (klabel:set current-cell-label label-sep-len)
+      (klabel:set current-cell-label lbl-sep-len)
       ;; Process any subtrees of current cell.
-      (if (kcell-view:child nil label-sep-len)
+      (if (kcell-view:child nil lbl-sep-len)
 	  ;; Recurse over subtree.
 	  (klabel-type:set-legal
 	   (klabel:child-legal current-cell-label)
-	   label-sep-len
+	   lbl-sep-len
 	   (+ current-indent per-level-indent)
 	   per-level-indent))
       ;; Process next sibling of current cell if any.
       (setq opoint (point))
       (if (and (not current-tree-only)
-	       (kcell-view:next nil label-sep-len)
-	       (< (abs (- (kcell-view:indent nil label-sep-len) current-indent))
-		  (kview:level-indent kview)))
+	       (kcell-view:next nil lbl-sep-len)
+	       (< (abs (- (kcell-view:indent nil lbl-sep-len) current-indent))
+		  (kview:level-indent kotl-kview)))
 	  (setq suffix-val (1+ suffix-val)
 		label-suffix (int-to-string suffix-val)
 		current-cell-label (concat label-prefix label-suffix))
 	(goto-char opoint)
 	(setq current-cell-label nil)))))
 
-(defun klabel-type:set-no (_current-cell-label label-sep-len &rest _ignore)
+(defun klabel-type:set-no (_current-cell-label lbl-sep-len &rest _ignore)
   "Set the labels of current cell, its following siblings and their subtrees.
 CURRENT-CELL-LABEL is the label to display for the current cell."
   ;; Only need to do this when switching from one label type to another,
   ;; i.e. when every cell label will be updated.  So if not starting with the
   ;; first cell, do nothing.
   (if (kotl-mode:first-cell-p)
-      (while (and (klabel:set "" label-sep-len)
-		  (kcell-view:next nil label-sep-len)))))
+      (while (and (klabel:set "" lbl-sep-len)
+		  (kcell-view:next nil lbl-sep-len)))))
 
-(defun klabel-type:set-partial-alpha (current-cell-label label-sep-len
+(defun klabel-type:set-partial-alpha (current-cell-label lbl-sep-len
                                       current-indent per-level-indent
 				      &optional current-tree-only)
   "Set the labels of current cell, its following siblings and their subtrees.
 CURRENT-CELL-LABEL is the label to display for the current cell.
-LABEL-SEP-LEN is the length of the separation between a cell's label
+LBL-SEP-LEN is the length of the separation between a cell's label
 and the start of its contents."
   (let (label-suffix suffix-val suffix-function opoint)
     (if current-cell-label
@@ -442,46 +458,46 @@ and the start of its contents."
 				'kotl-label:int-to-alpha)))
     (while current-cell-label
       ;; Set current cell's label.
-      (klabel:set current-cell-label label-sep-len)
+      (klabel:set current-cell-label lbl-sep-len)
       ;; Process any subtrees of current cell.
-      (if (kcell-view:child nil label-sep-len)
+      (if (kcell-view:child nil lbl-sep-len)
 	  ;; Recurse over subtree.
 	  (klabel-type:set-partial-alpha
 	   (klabel:child-partial-alpha current-cell-label)
-	   label-sep-len
+	   lbl-sep-len
 	   (+ current-indent per-level-indent)
 	   per-level-indent))
       ;; Process next sibling of current cell if any.
       (setq opoint (point))
       (if (and (not current-tree-only)
-	       (kcell-view:next nil label-sep-len)
-	       (< (abs (- (kcell-view:indent nil label-sep-len) current-indent))
-		  (kview:level-indent kview)))
+	       (kcell-view:next nil lbl-sep-len)
+	       (< (abs (- (kcell-view:indent nil lbl-sep-len) current-indent))
+		  (kview:level-indent kotl-kview)))
 	  (setq suffix-val (1+ suffix-val)
 		label-suffix (funcall suffix-function suffix-val)
 		current-cell-label label-suffix)
 	(goto-char opoint)
 	(setq current-cell-label nil)))))
 
-(defun klabel-type:set-star (_current-cell-label label-sep-len &rest _ignore)
+(defun klabel-type:set-star (_current-cell-label lbl-sep-len &rest _ignore)
   "Set the labels of current cell, its following siblings and their subtrees.
 CURRENT-CELL-LABEL is the label to display for the current cell.
-LABEL-SEP-LEN is the length of the separation between a cell's label
+LBL-SEP-LEN is the length of the separation between a cell's label
 and the start of its contents."
   ;; Only need to do this when switching from one label type to another,
   ;; i.e. when every cell label will be updated.  So if not starting with the
   ;; first cell, do nothing.
   (if (kotl-mode:first-cell-p)
       (while (and (klabel:set (make-string
-			       (kcell-view:level nil label-sep-len) ?*)
-			      label-sep-len)
-		  (kcell-view:next nil label-sep-len)))))
+			       (kcell-view:level nil lbl-sep-len) ?*)
+			      lbl-sep-len)
+		  (kcell-view:next nil lbl-sep-len)))))
 
 (defun klabel-type:update-labels (current-cell-label)
-  "Update the labels of current cell, its following siblings and their subtrees if need be.
+  "Update the labels of current cell, its following siblings and their subtrees.
 CURRENT-CELL-LABEL is the label to display for the current cell.
 If, however, it is \"0\", then all cell labels are updated."
-  (let ((label-type (kview:label-type kview)))
+  (let ((label-type (kview:label-type kotl-kview)))
     (when (memq label-type '(alpha legal partial-alpha))
       (if (string-equal current-cell-label "0")
 	  ;; Update all cells in view.
@@ -494,14 +510,14 @@ If, however, it is \"0\", then all cell labels are updated."
   "Update the labels of current cell and its subtree.
 CURRENT-CELL-LABEL is the label to display for the current cell.
 Use `(klabel-type:update-labels \"0\")' to update all cells in an outline."
-  (let ((label-type (kview:label-type kview))
-	(label-sep-len (kview:label-separator-length kview)))
+  (let ((label-type (kview:label-type kotl-kview))
+	(lbl-sep-len (kview:label-separator-length kotl-kview)))
     (save-excursion
       (funcall (intern-soft (concat "klabel-type:set-"
 				    (symbol-name label-type)))
-	       first-label label-sep-len
-	       (kcell-view:indent nil label-sep-len)
-	       (kview:level-indent kview)
+	       first-label lbl-sep-len
+	       (kcell-view:indent nil lbl-sep-len)
+	       (kview:level-indent kotl-kview)
 	       ;; Update current tree only.
 	       t))))
 
@@ -614,14 +630,15 @@ N may be an integer or a string containing an integer."
 ;;; Private functions
 ;;; ************************************************************************
 
-(defun klabel:set (new-label &optional label-sep-len)
-  "Replace label displayed in cell at point with NEW-LABEL, which may be a different label type.
+(defun klabel:set (new-label &optional lbl-sep-len)
+  "Replace label displayed in cell at point with NEW-LABEL.
+The new label type may differ from the old type.
 Return NEW-LABEL string."
   (let ((modified (buffer-modified-p))
 	(buffer-read-only)
-	(thru-label (- (kcell-view:indent nil label-sep-len)
-		       (or label-sep-len
-			   (kview:label-separator-length kview)))))
+	(thru-label (- (kcell-view:indent nil lbl-sep-len)
+		       (or lbl-sep-len
+			   (kview:label-separator-length kotl-kview)))))
     (save-excursion
       (kcell-view:to-label-end)
       ;; delete backwards thru label
@@ -639,13 +656,13 @@ For example, the full label \"1a2\" has kotl-label \"2\", as does \"1.1.2\"."
     (error "(klabel:to-kotl-label): Invalid label, `%s'" label)))
 
 (defun klabel-type:update-labels-from-point (label-type first-label)
-  (let ((label-sep-len (kview:label-separator-length kview)))
+  (let ((lbl-sep-len (kview:label-separator-length kotl-kview)))
     (save-excursion
       (funcall (intern-soft (concat "klabel-type:set-"
 				    (symbol-name label-type)))
-	       first-label label-sep-len
-	       (kcell-view:indent nil label-sep-len)
-	       (kview:level-indent kview)))))
+	       first-label lbl-sep-len
+	       (kcell-view:indent nil lbl-sep-len)
+	       (kview:level-indent kotl-kview)))))
 
 (provide 'klabel)
 
